import React, { PropsWithChildren, useState } from "react";

type InformasjonsmeldingKategori = "advarsel" | "info" | "error";

export enum InformasjonsmeldingType {
    ingenDataForFiltere = "ingenDataForFiltere",
    ingenDataForStengteVeier = "ingenDataForStengteVeier",
    lasterStengteVeier = "lasterStengteVeier",
    ingenProddataForValgteTimer = "ingenProddataForValgteTimer",
    tidspunktFremITid = "tidspunktFremITid",
    viserAltSisteProduksjon = "viserAltSisteProduksjon",
    ingenDataIKartet = "ingenDataIKartet",
}

const ingenDataForFiltere: Informasjonsmelding = {
    tekst: "Ingen data for valgte filtere.",
    type: InformasjonsmeldingType.ingenDataForFiltere,
    kategori: "advarsel",
};

const ingenProddataForValgteTimer: Informasjonsmelding = {
    tekst: "Vi fant ikke noe produksjonsdata for de valgte 12 timene.",
    type: InformasjonsmeldingType.ingenProddataForValgteTimer,
    kategori: "advarsel",
};

const ingenDataForStengteVeier: Informasjonsmelding = {
    tekst: "Laster informasjon om stengte veier.",
    type: InformasjonsmeldingType.ingenDataForStengteVeier,
    kategori: "info",
};

const lasterStengteVeier: Informasjonsmelding = {
    tekst: "Laster informasjon om stengte veier.",
    type: InformasjonsmeldingType.lasterStengteVeier,
    kategori: "info",
};

const tidspunktFremITid: Informasjonsmelding = {
    tekst: "Du kan ikke velge et tidspunkt som er frem i tid.",
    type: InformasjonsmeldingType.tidspunktFremITid,
    kategori: "error",
};

const viserAltSisteProduksjon: Informasjonsmelding = {
    tekst: "Sist oppdaterte produksjon vises allerede i kartet. Vi henter data fra Elrapp hvert femte minutt.",
    type: InformasjonsmeldingType.viserAltSisteProduksjon,
    kategori: "info",
};

const ingenDataIKartet: Informasjonsmelding = {
    tekst: "Det er ingen produksjon i kartet.",
    type: InformasjonsmeldingType.ingenDataIKartet,
    kategori: "advarsel",
};

export const informasjonsmeldinger = {
    ingenDataForFiltere: ingenDataForFiltere,
    ingenProddataForValgteTimer: ingenProddataForValgteTimer,
    ingenDataForStengteVeier: ingenDataForStengteVeier,
    lasterStengteVeier: lasterStengteVeier,
    tidspunktFremITid: tidspunktFremITid,
    viserAltSisteProduksjon: viserAltSisteProduksjon,
    ingenDataIKartet: ingenDataIKartet,
};

export interface Informasjonsmelding {
    tekst: string;
    type: InformasjonsmeldingType;
    kategori: InformasjonsmeldingKategori;
    onClose?: () => void;
}

export interface InformasjonsmeldingValue {
    verdi?: Informasjonsmelding;
    oppdater: (informasjonsmelding: Informasjonsmelding | undefined, skalVisesTid?: number) => void;
    fjern: () => void;
}

export const InformasjonsmeldingContext = React.createContext<InformasjonsmeldingValue>({
    verdi: undefined,
    oppdater: (_informasjonsmelding, _skalVisesTid) => {},
    fjern: () => {},
});

export const InformasjonsmeldingProvider: React.FC<PropsWithChildren<{}>> = ({ children }) => {
    const [informasjonsmelding, setInformasjonsmelding] = useState<
        Informasjonsmelding | undefined
    >();
    const defaultSkalVisesTid = 10000;

    const håndterSetInformasjonsmelding = (
        informasjonsmelding: Informasjonsmelding | undefined,
        skalVisesTid: number
    ) => {
        setInformasjonsmelding(informasjonsmelding);
        setTimeout(() => {
            setInformasjonsmelding(undefined);
        }, skalVisesTid);
    };

    return (
        <InformasjonsmeldingContext.Provider
            value={{
                verdi: informasjonsmelding && {
                    ...informasjonsmelding,
                    onClose: () => setInformasjonsmelding(undefined),
                },
                oppdater: (informasjonsmelding, skalVisesTid = defaultSkalVisesTid) =>
                    håndterSetInformasjonsmelding(informasjonsmelding, skalVisesTid),
                fjern: () => setInformasjonsmelding(undefined),
            }}
        >
            {children}
        </InformasjonsmeldingContext.Provider>
    );
};
