import React, { PropsWithChildren, useContext, useState } from "react";
import { ApiRespons, fetcherPost } from "../api";
import { GCP_MEDIATOR_URL } from "../../utils/baseUrl";

export interface KjoretoyProduksjonsdetaljerDto {
    produksjonsdataRestRec: number;
    kjoeretoyid?: string;
    vegnr?: string;
    rodeid?: string;
    datoLogget: string;
    hastighetkmh: number;
    sprederbreddetoerrstoffcm: number;
    sprederbreddevaatstoffcm: number;
    doseringtoerrstoffgm2: number;
    doseringvaatstoffmlm2: number;
    ertoerrstoffsprederpaa?: boolean;
    ervaatstoffsprederpaa?: boolean;
    erplognede?: boolean;
    erhoevelskjaernede?: boolean;
    erunderliggendeskjaernede?: boolean;
    ersideplogibruk?: boolean;
    materiale?: string;
    eksternReferanse?: string;
}

export interface KjoretoyProduksjonsdetaljerCache {
    [id: string]: ApiRespons<KjoretoyProduksjonsdetaljerDto[]>;
}

interface CacheContextValue {
    pushToCache: (key: string, verdi: ApiRespons<KjoretoyProduksjonsdetaljerDto[]>) => void;

    hentKjoretoyProduksjonsdetaljer: (
        målepunkter: number[]
    ) => Promise<ApiRespons<KjoretoyProduksjonsdetaljerDto[]>> | undefined;

    tømCache: () => void;
}

export const KjoretoyProduksjonsdetaljerCacheContext = React.createContext<CacheContextValue>({
    pushToCache: (key: string, verdi: ApiRespons<KjoretoyProduksjonsdetaljerDto[]>) => {},
    hentKjoretoyProduksjonsdetaljer: (målepunkter) => undefined,
    tømCache: () => {},
});

export const useKjoretoyProduksjonsdetaljerCache = () =>
    useContext(KjoretoyProduksjonsdetaljerCacheContext);

export const KjoretoyProduksjonsdetaljerCacheProvider: React.FC<PropsWithChildren<{}>> = ({
    children,
}) => {
    const [cache, setCache] = useState<KjoretoyProduksjonsdetaljerCache>({});

    const pushToCache = (key: string, verdi: ApiRespons<KjoretoyProduksjonsdetaljerDto[]>) => {
        cache[key] = verdi;
        setCache(cache);
    };

    const hentData = async (
        målepunkter: number[]
    ): Promise<ApiRespons<KjoretoyProduksjonsdetaljerDto[]>> => {
        const url = `${GCP_MEDIATOR_URL}/produksjonsdetaljer`;
        const body = { målepunkter: målepunkter };

        if (cache[målepunkter.toString()]) {
            return cache[målepunkter.toString()];
        }
        return fetcherPost<KjoretoyProduksjonsdetaljerDto[]>(url, JSON.stringify(body));
    };

    const tømCache = () => {
        setCache({});
    };

    return (
        <KjoretoyProduksjonsdetaljerCacheContext.Provider
            value={{
                pushToCache: pushToCache,
                hentKjoretoyProduksjonsdetaljer: hentData,
                tømCache: tømCache,
            }}
        >
            {children}
        </KjoretoyProduksjonsdetaljerCacheContext.Provider>
    );
};
