import React, { PropsWithChildren, useState } from "react";
import { useDropdownFilterUtils } from "../../kartSide/filtre/filtervelger/DropdownFilterUtils";

export enum Filterkategori {
    Brøyting = "Brøyting",
    Høvling = "Høvling",
    Sand = "Sand",
    Salt = "Salt",
    IngenProduksjon = "IngenProduksjon",
}

export enum Filtertype {
    Tørrsand = "Tørrsand",
    Fastsand = "Fastsand",
    SandMedSaltInnblanding = "SandMedSaltInnblanding",
    Tørrsalt = "Tørrsalt",
    BefuktetSalt = "BefuktetSalt",
    BefuktetFinkornetSalt = "BefuktetFinkornetSalt",
    Saltløsning = "Saltløsning",
    Magnesiumklorid = "Magnesiumklorid",
    PlogNede = "PlogNede",
    Sideplog = "Sideplog",
    UnderliggendeSkjær = "UnderliggendeSkjær",
    TungHøvel = "TungHøvel",
    IngenProduksjon = "IngenProduksjon",
}

export enum MaterialOgAktivitetsNavn {
    Tørrsand = "Tørrsand",
    Fastsand = "Fastsand",
    SandMedSaltInnblanding = "Sand med saltinnblanding",
    Tørrsalt = "Tørrsalt",
    BefuktetSalt = "Befuktet salt",
    BefuktetFinkornetSalt = "Saltslurry",
    Saltløsning = "Saltløsning",
    Magnesiumklorid = "Magnesiumklorid",
    PlogNede = "Plog nede",
    Sideplog = "Sideplog",
    UnderliggendeSkjær = "Underliggende skjær",
    TungHøvel = "Tung høvel",
    IngenProduksjon = "Ingen produksjon",
}

export interface Filter {
    navn: string;
    kategori: Filterkategori;
    type: Filtertype;
    valgt: boolean;
}

interface ProduksjonsfilterContextValue {
    produksjonsfilter: Filter[];
    oppdaterFilter: (verdi: Filter[]) => void;
    filtrertekategorierIProduksjonslenker?: Filtertype[];
    defaultFiltreValgt: boolean;
    setDefaultFilterForProduksjonsdata: () => void;
}

export const defaultProduksjonsfilter: Filter[] = [
    {
        navn: MaterialOgAktivitetsNavn.PlogNede,
        type: Filtertype.PlogNede,
        kategori: Filterkategori.Brøyting,
        valgt: true,
    },
    {
        navn: MaterialOgAktivitetsNavn.Sideplog,
        type: Filtertype.Sideplog,
        kategori: Filterkategori.Brøyting,
        valgt: true,
    },
    {
        navn: MaterialOgAktivitetsNavn.UnderliggendeSkjær,
        type: Filtertype.UnderliggendeSkjær,
        kategori: Filterkategori.Høvling,
        valgt: true,
    },
    {
        navn: MaterialOgAktivitetsNavn.TungHøvel,
        type: Filtertype.TungHøvel,
        kategori: Filterkategori.Høvling,
        valgt: true,
    },
    {
        navn: MaterialOgAktivitetsNavn.Tørrsand,
        type: Filtertype.Tørrsand,
        kategori: Filterkategori.Sand,
        valgt: true,
    },
    {
        navn: MaterialOgAktivitetsNavn.Fastsand,
        type: Filtertype.Fastsand,
        kategori: Filterkategori.Sand,
        valgt: true,
    },
    {
        navn: MaterialOgAktivitetsNavn.SandMedSaltInnblanding,
        type: Filtertype.SandMedSaltInnblanding,
        kategori: Filterkategori.Sand,
        valgt: true,
    },
    {
        navn: MaterialOgAktivitetsNavn.Tørrsalt,
        type: Filtertype.Tørrsalt,
        kategori: Filterkategori.Salt,
        valgt: true,
    },
    {
        navn: MaterialOgAktivitetsNavn.BefuktetSalt,
        type: Filtertype.BefuktetSalt,
        kategori: Filterkategori.Salt,
        valgt: true,
    },
    {
        navn: MaterialOgAktivitetsNavn.BefuktetFinkornetSalt,
        type: Filtertype.BefuktetFinkornetSalt,
        kategori: Filterkategori.Salt,
        valgt: true,
    },
    {
        navn: MaterialOgAktivitetsNavn.Saltløsning,
        type: Filtertype.Saltløsning,
        kategori: Filterkategori.Salt,
        valgt: true,
    },
    {
        navn: MaterialOgAktivitetsNavn.Magnesiumklorid,
        type: Filtertype.Magnesiumklorid,
        kategori: Filterkategori.Salt,
        valgt: true,
    },
    {
        navn: MaterialOgAktivitetsNavn.IngenProduksjon,
        type: Filtertype.IngenProduksjon,
        kategori: Filterkategori.IngenProduksjon,
        valgt: false,
    },
];

const defaultValue = {
    produksjonsfilter: defaultProduksjonsfilter,
    oppdaterFilter: (verdi: Filter[]) => {},
    filtrertekategorierIProduksjonslenker: [],
    defaultFiltreValgt: true,
    setDefaultFilterForProduksjonsdata: () => {},
};

export const ProduksjonsfilterContext =
    React.createContext<ProduksjonsfilterContextValue>(defaultValue);

export const ProduksjonsfilterProvider: React.FC<PropsWithChildren<{}>> = ({ children }) => {
    const [filter, setFilter] = useState<Filter[]>(defaultValue.produksjonsfilter);
    const [defaultFiltreValgt, setDefaultFiltreValgt] = useState<boolean>(
        defaultValue.defaultFiltreValgt
    );
    const { setDefaultFilterForProduksjonsdata, filtrertekategorierIProduksjonslenker } =
        useDropdownFilterUtils({
            setDefaultFiltreValgt,
            produksjonsfilter: filter,
            oppdaterFilter: setFilter,
        });
    return (
        <ProduksjonsfilterContext.Provider
            value={{
                produksjonsfilter: filter,
                oppdaterFilter: setFilter,
                defaultFiltreValgt: defaultFiltreValgt,
                setDefaultFilterForProduksjonsdata: setDefaultFilterForProduksjonsdata,
                filtrertekategorierIProduksjonslenker: filtrertekategorierIProduksjonslenker,
            }}
        >
            {children}
        </ProduksjonsfilterContext.Provider>
    );
};
