import React, { PropsWithChildren, useState } from "react";
import { Produksjonsdetaljer } from "../../kartSide/types/produksjonsdetaljer";
import { loggFrontend, LoggNivå } from "../../utils/loggFrontend";
import { ApiRespons, fetcher } from "../api";
import { sideInfo } from "../../sider";
import {
    dtoTilProduksjonsdetaljer,
    ProduksjonsdetaljerDto,
} from "./ProduksjonsdetaljerCacheContext";
import { GCP_MEDIATOR_URL } from "../../utils/baseUrl";

export interface ØktDetaljerContextValue {
    produksjonsdetaljer?: Produksjonsdetaljer[];
    hentØktDetaljer: (øktId: string) => Promise<ApiRespons<Produksjonsdetaljer[]>> | undefined;
    laster: boolean;
}

const defaultValue = {
    produksjonsdetaljer: [],
    hentØktDetaljer: (øktId: string) => undefined,
    laster: false,
};

export const ØktDetaljerContext = React.createContext<ØktDetaljerContextValue>(defaultValue);

export const ØktDetaljerProvider: React.FC<PropsWithChildren<{}>> = ({ children }) => {
    const [laster, setLaster] = useState<boolean>(false);
    const [øktdetaljer, setØktdetaljer] = useState<Produksjonsdetaljer[]>([]);

    const hentData = async (øktId: string): Promise<ApiRespons<Produksjonsdetaljer[]>> => {
        setLaster(true);
        return fetcher<ProduksjonsdetaljerDto[]>(
            `${GCP_MEDIATOR_URL}/økt-detaljer?økt=${øktId}`
        ).then((response) => {
            setLaster(false);
            if (response.suksess) {
                const produksjonsdetaljer = response.data.map(dtoTilProduksjonsdetaljer);
                setØktdetaljer(produksjonsdetaljer);
                return {
                    ...response,
                    data: produksjonsdetaljer,
                };
            }
            return response;
        });
    };

    return (
        <ØktDetaljerContext.Provider
            value={{ produksjonsdetaljer: øktdetaljer, hentØktDetaljer: hentData, laster: laster }}
        >
            {children}
        </ØktDetaljerContext.Provider>
    );
};

const finnØktid = (): string => {
    const urlParam = hentDetaljerUrlParam();
    const øktid = urlParam?.økt;
    return øktid ?? "";
};

interface DetaljerUrlData {
    økt: string;
}

export enum DetaljerUrlSearchParameter {
    okt = "oktid",
}

export const hentDetaljerUrlParam = (): DetaljerUrlData | undefined => {
    if (typeof window !== "undefined") {
        const queryParam = new URLSearchParams(window.location.search);
        const øktParam = queryParam.get(DetaljerUrlSearchParameter.okt);
        if (øktParam === null) {
            return undefined;
        }

        try {
            const parsetØktid = Number(øktParam);
            return parsetØktid
                ? {
                      økt: parsetØktid.toString(),
                  }
                : undefined;
        } catch (e) {
            loggFrontend({
                loggNivå: LoggNivå.Warn,
                customInformasjon: `Klarte ikke parse queryparametre ${øktParam}: ${e}`,
            });
            return undefined;
        }
    }
};

export const settDetaljerUrlParam = (
    nåværendePath: string,
    detaljerUrlData: DetaljerUrlData
): void => {
    if (nåværendePath.includes(sideInfo.øktdetaljer.href)) {
        const urlParametre = new URLSearchParams();
        urlParametre.set(DetaljerUrlSearchParameter.okt, detaljerUrlData.økt.toString());
        history.replaceState("", "", "?" + urlParametre.toString());
        history.go();
    }
};
