import useSWR from "swr";
import { ApiRespons, fetcher, ikkeRevalidate } from "./api";
import { GCP_MEDIATOR_URL, WEBKAMERA_URL } from "../utils/baseUrl";
import { Dayjs } from "dayjs";
import { Geometry, Point } from "geojson";
import { tilEnkelTimestampStr } from "../utils/tid";
import React from "react";

export interface TrafikkmeldingerDto {
    causeRecordId?: string;
    recordType?: string;
    recordSubtypes: string[];
    overallStartTime?: string;
    overallEndTime?: string;
    recurringTimePeriods: RecurringTimePeriods[];
    dataProcessingNote?: string;
    description?: string;
    locations: Locations[];
    roadClosed: boolean;
}

export interface RecurringTimePeriods {
    startTime?: string;
    endTime?: string;
    daysOfWeek: string[];
}

interface Locations {
    locationType?: string;
    coordinatesForDisplay?: Point;
    locationDescriptor?: string;
    roadNumber?: string;
    roadName?: string;
    countyNumber?: number;
    carriageway?: string;
    geographyType?: string;
    geography?: Geometry;
}

export function useHentTrafikkmeldinger(
    tidspunkt: Dayjs
): ApiRespons<TrafikkmeldingerDto[]> | undefined {
    const { data } = useSWR<ApiRespons<TrafikkmeldingerDto[]>>(
        `${GCP_MEDIATOR_URL}/trafikkmeldinger?tidspunkt=${tilEnkelTimestampStr(tidspunkt)}`,
        fetcher,
        ikkeRevalidate
    );
    return data;
}

export const reccuringTimePeriodToString = (period: RecurringTimePeriods) =>
    `${period.startTime} - ${period.endTime}: ${reccuringTimePeriodDaysOfWeekToString(
        period.daysOfWeek
    )}`;

const reccuringTimePeriodDaysOfWeekToString = (days: string[]) => {
    let outString = "";
    if (days.includes("MONDAY")) {
        outString += "Mandag, ";
    }
    if (days.includes("TUESDAY")) {
        outString += "Tirsdag, ";
    }
    if (days.includes("WEDNESDAY")) {
        outString += "Onsdag, ";
    }
    if (days.includes("THURSDAY")) {
        outString += "Torsdag, ";
    }
    if (days.includes("FRIDAY")) {
        outString += "Fredag, ";
    }
    if (days.includes("SATURDAY")) {
        outString += "Lørdag, ";
    }
    if (days.includes("SUNDAY")) {
        outString += "Søndag, ";
    }
    return outString.slice(0, -2);
};
