import React, { useContext } from "react";
import { AuthenticationContext, AuthenticationStatus } from "../api/context/AuthenticationContext";
import { Spinner } from "./fellesKomponenter/spinner/Spinner";

interface AuthenicationWrapperProps {
    children: React.ReactNode;
}

export const AuthenticationWrapper = ({ children }: AuthenicationWrapperProps) => {
    const authentication = useContext(AuthenticationContext);

    switch (authentication.status) {
        case AuthenticationStatus.Initialisert:
        case AuthenticationStatus.Henter:
            return (
                <Spinner storrelse="liten" sentrert={true} inline={false}>
                    Autentiserer...
                </Spinner>
            );
        case AuthenticationStatus.Autentisert:
            return <>{children}</>;
        case AuthenticationStatus.Feilet:
            return <div>Ikke autentisert</div>;
        case AuthenticationStatus.LoggerUt:
            return <div>Logger ut</div>;
        default:
            return <div>Noe feilet under autentisering</div>;
    }
};
