import styled from "styled-components";
import Popup from "reactjs-popup";
import { InfotekstIkon } from "../Ikon/InfotekstIkon";
import React, { useRef, useState } from "react";
import { KnappUtenStyling } from "./knapper/KnappUtenStyling";

interface Props {
    children: React.ReactNode;
    størrelse?: "S" | "L";
    plassering?: HjelpePopupPosition;
}

export type HjelpePopupPosition =
    | "top left"
    | "top center"
    | "top right"
    | "right top"
    | "right center"
    | "right bottom"
    | "bottom left"
    | "bottom center"
    | "bottom right"
    | "left top"
    | "left center"
    | "left bottom"
    | "center center";

export const HjelpetekstPopup = ({
    children,
    størrelse = "L",
    plassering = "top center",
}: Props) => {
    const bredde = størrelse === "S" ? 18 : undefined;
    const høyde = størrelse === "S" ? 21 : undefined;
    const [open, setOpen] = useState<boolean>(false);
    const ikonKnappRef = useRef<HTMLButtonElement>(null);

    const handlePopupClose = () => {
        setOpen(false);
        ikonKnappRef?.current?.focus();
    };

    return (
        <StyledPopup
            open={open}
            trigger={
                <div>
                    <KnappUtenStyling
                        onClick={() => setOpen(true)}
                        reference={ikonKnappRef}
                        type={"button"}
                    >
                        <InfotekstIkon width={bredde} height={høyde} />
                    </KnappUtenStyling>
                </div>
            }
            position={plassering}
            closeOnDocumentClick={true}
            onClose={() => setOpen(false)}
        >
            <CloseButton onClick={handlePopupClose}>&times;</CloseButton>
            {children}
        </StyledPopup>
    );
};

const StyledPopup = styled(Popup)`
    &-content {
        background-color: var(--blue-bg);
        border: 2px solid var(--blue-regular);
        padding: 15px;
        font-size: 14px;
        max-width: 300px;
        color: var(--gray-dark);
    }

    &-arrow {
        color: var(--blue-bg);
        stroke-width: 4px;
        stroke: var(--blue-regular);
        stroke-dasharray: 30px;
        stroke-dashoffset: -54px;
    }
`;

const CloseButton = styled.button`
    position: absolute;
    top: 0;
    right: 0;
    background: transparent;
    border: none;
    font-size: 24px;
    cursor: pointer;
    color: var(--gray-dark);
`;
