import React, { ReactNode } from "react";
import styled from "styled-components";
import { SideRef } from "../../utils/scrollTo";
import { Heading } from "./typografi/Heading";
import { breakpointNettbrett } from "../../utils/constants";

interface PanelProps {
    tittel: string;
    children: ReactNode;
}

export const Panel = React.forwardRef(({ tittel, children }: PanelProps, ref) => (
    <StyledArticle ref={ref as SideRef}>
        <HeadingWrapper>
            <StyledHeading as={"h1"} styleLevel={"3"}>
                {tittel}
            </StyledHeading>
        </HeadingWrapper>
        <ContentWrapper>{children}</ContentWrapper>
    </StyledArticle>
));

Panel.displayName = "Panel";

const StyledArticle = styled.article`
    background-color: var(--gray-light);
    padding: 0;
`;

const ContentWrapper = styled.div`
    padding: 0 2rem 2rem 2rem;
    width: auto;

    @media (max-width: ${breakpointNettbrett}) {
        padding: 0 1rem 1rem 1rem;
    }
`;

const HeadingWrapper = styled.div`
    border-bottom: var(--orange) solid 2px;
`;

const StyledHeading = styled(Heading)`
    padding: 1.5rem 1.5rem 1rem 1.5rem;
`;
