import React from "react";
import styled from "styled-components";
import { FormLabel } from "./FormLabel";
import { Inputfelt } from "./Inputfelt";
import { Dayjs } from "dayjs";
import { fraEnkelKlokkeslettString, tilEnkelKlokkeslettString } from "../../../utils/tid";

interface KlokkeslettFeltProps {
    klokkeslett: Dayjs;
    settKlokkeslett: (nyttKlokkeslett: Dayjs) => void;
}

export const KlokkeslettFelt = ({ klokkeslett, settKlokkeslett }: KlokkeslettFeltProps) => (
    <KlokkeslettfeltWrapper>
        <FormLabel htmlFor="klokkeslett">Velg klokkeslett</FormLabel>
        <Inputfelt
            type="time"
            id="klokkeslett"
            størrelse="L"
            onInput={(e: React.ChangeEvent<HTMLInputElement>) => {
                const nyttKlokkeslett = fraEnkelKlokkeslettString(e.target.value);
                settKlokkeslett(
                    klokkeslett
                        .clone()
                        .set("hour", nyttKlokkeslett.hour())
                        .set("minute", nyttKlokkeslett.minute())
                );
            }}
            required
            pattern="[0-9]{2}:[0-9]{2}"
            step={300}
            value={tilEnkelKlokkeslettString(klokkeslett)}
        />
    </KlokkeslettfeltWrapper>
);

const KlokkeslettfeltWrapper = styled.div`
    display: flex;
    flex-direction: column;

    input {
        text-align: center;
    }
`;
