import React from "react";
import styled from "styled-components";
import { TekstBasisStyles } from "./TekstStyles";
import { breakpointNettbrett } from "../../../utils/constants";

interface HeaderProps {
    styleLevel: "1" | "2" | "3" | "4" | "5" | "6";
    as: string;
    bold?: boolean;
}

export const Heading = styled.h1.attrs((props: HeaderProps) => ({
    as: props.as,
    styleLevel: props.styleLevel,
    bold: props.bold || false,
}))`
    ${TekstBasisStyles};
    margin: 0;
    font-size: ${({ styleLevel }) => headingTheme.fontsize[styleLevel]};
    line-height: ${({ styleLevel }) => headingTheme.lineheight[styleLevel]};
    font-weight: ${({ bold }) => (bold ? "bold" : "normal")};

    @media (max-width: ${breakpointNettbrett}) {
        font-size: ${({ styleLevel }) => headingThemeSmall.fontsize[styleLevel]};
        line-height: ${({ styleLevel }) => headingThemeSmall.lineheight[styleLevel]};
    }
`;

const headingTheme = {
    fontsize: {
        "1": "48px",
        "2": "30px",
        "3": "24px",
        "4": "20px",
        "5": "18px",
        "6": "16px",
    },
    lineheight: {
        "1": "56px",
        "2": "38px",
        "3": "28px",
        "4": "26px",
        "5": "24px",
        "6": "24px",
    },
};

const headingThemeSmall = {
    fontsize: {
        "1": "36px",
        "2": "24px",
        "3": "20px",
        "4": "18px",
        "5": "16px",
        "6": "16px",
    },
    lineheight: {
        "1": "42px",
        "2": "28px",
        "3": "26px",
        "4": "24px",
        "5": "22px",
        "6": "22px",
    },
};
