import styled from "styled-components";
import { useRef } from "react";
import { useDisableClickPropagation } from "../../../hooks/useDisableClickPropagation";
import { useDisableScrollPropagtaion } from "../../../hooks/useDisableScrollPropagtaion";
import { motion } from "framer-motion";
import { useWindowSize } from "../../../hooks/useWindowSize";

interface MenyContainerProps {
    children: React.ReactNode;
}

export const MenyContainer = ({ children }: MenyContainerProps) => {
    const menyRef = useRef(null);
    useDisableClickPropagation(menyRef);
    useDisableScrollPropagtaion(menyRef);
    const { windowHeight } = useWindowSize();

    return (
        <Container
            initial="collapsed"
            animate="open"
            exit="collapsed"
            windowheight={windowHeight}
            ref={menyRef}
            variants={{
                open: { x: 0 },
                collapsed: { x: `calc(var(--meny-width) * -1)` },
            }}
            transition={{
                duration: 0.3,
            }}
        >
            <ContentWrapper>{children}</ContentWrapper>
        </Container>
    );
};

const Container = styled(motion.nav)<{ windowheight: number }>`
    position: absolute;
    top: 0;
    left: 0;
    height: calc(${({ windowheight }) => windowheight}px - var(--header-height, 0px));
    width: var(--meny-width);
    background-color: var(--gray-light);
    z-index: 1510;
    overflow: auto;
    box-shadow: 0px 4px 10px rgba(0, 0, 0, 0.25);
    cursor: auto;
    display: block;
`;

const ContentWrapper = styled.div`
    display: flex;
    flex-direction: column;
    padding: 20px;
    gap: 1.5rem;
`;
