import styled from "styled-components";
import React from "react";
import { Heading } from "../components/fellesKomponenter/typografi/Heading";
import { breakpointMobil, breakpointNettbrett } from "../utils/constants";

interface SideHeaderProps {
    children: React.ReactNode;
}

export const DriftsportalHeader = ({ children }: SideHeaderProps) => (
    <>
        <StyledHeader>
            <Logo />
            <Heading as="h1" styleLevel="3">
                Driftsportal for byggherrepersonell
            </Heading>
        </StyledHeader>
        <StyledNav>{children}</StyledNav>
    </>
);

const Logo = styled.i`
    background: url("/ikon/svv-logo.svg") no-repeat;

    @media (min-width: ${breakpointNettbrett}) {
        width: 95.38px;
        height: 50px;
    }
`;

const StyledHeader = styled.header`
    display: flex;
    z-index: 5;
    background: var(--gray-light);
    padding: 0 1.5rem;
    height: 6rem;
    align-items: center;
    justify-content: center;
    gap: 1rem;

    @media (max-width: ${breakpointNettbrett}) {
        height: 4rem;
    }
`;

const StyledNav = styled.nav`
    position: sticky;
    padding: 0 2rem;
    margin: auto;
    max-width: 100rem;
    top: 0;
    background: var(--white);
    z-index: 10;

    @media (max-width: ${breakpointMobil}) {
        position: relative;
    }
`;
