import { ReactNode, useState } from "react";
import styled from "styled-components";
import { motion } from "framer-motion";
import { breakpointNettbrett } from "../utils/constants";

export type KarusellProps = {
    children: ReactNode[];
};

export const Karusell = ({ children }: KarusellProps) => {
    const [karusellIndex, settKarusellIndex] = useState(0);
    const [karusellSlideRetning, settKarusellSlideRetning] = useState<"høyre" | "venstre">("høyre");

    const gåTilForrigeSlide = () => {
        settKarusellSlideRetning("venstre");
        karusellIndex > 0
            ? settKarusellIndex(karusellIndex - 1)
            : settKarusellIndex(children.length - 1);
    };

    const gåTilNesteSlide = () => {
        settKarusellSlideRetning("høyre");
        karusellIndex < children.length - 1
            ? settKarusellIndex(karusellIndex + 1)
            : settKarusellIndex(0);
    };
    return (
        <>
            <KarusellWrapper>
                <PilKnapp id="pil1" onClick={gåTilForrigeSlide} rotert={true}>
                    <span />
                </PilKnapp>
                <KarusellPanel
                    key={karusellIndex}
                    variants={slideVariants}
                    initial={karusellSlideRetning === "høyre" ? "hiddenRight" : "hiddenLeft"}
                    animate="visible"
                    exit="exit"
                >
                    {children[karusellIndex]}
                </KarusellPanel>
                <PilKnapp id="pil2" onClick={gåTilNesteSlide} rotert={false}>
                    <span />
                </PilKnapp>
            </KarusellWrapper>
            <BulletContainer>
                {children.map((_, index) => (
                    <Bullet key={index} active={index === karusellIndex} />
                ))}
            </BulletContainer>
        </>
    );
};

const KarusellPanel = styled(motion.div)`
    margin: 0 2rem;
    width: 100%;
`;

const PilKnapp = styled.button`
    background: transparent;
    border: 0;
    height: 3.7rem;
    padding: 0;
    width: 3.7rem;
    margin: 6rem 0 0;

    > span {
        background: var(--white) url("/ikon/chevron-right.svg") no-repeat calc(50% + 1px) center;
        border: var(--gray-dark-65) 1px solid;
        border-radius: 50%;
        box-shadow: 0 0 6px 2px #353e4314;
        display: block;
        height: 3.5rem;
        width: 3.5rem;
        transform: ${(props: { rotert: boolean }) => (props.rotert ? "rotate(180deg)" : 0)};
    }

    :hover > span {
        background-color: var(--gray-light);
        border: var(--orange) 1px solid;
        border-radius: 50%;
    }

    @media (max-width: ${breakpointNettbrett}) {
        margin: 0.8rem;
    }
`;

const KarusellWrapper = styled.div`
    align-items: start;
    background: var(--gray-light);
    display: flex;
    flex-direction: row;
    width: 100%;
    flex-wrap: nowrap;

    @media (max-width: ${breakpointNettbrett}) {
        flex-wrap: wrap;
        justify-content: center;

        #pil1 {
            order: 1;
        }
        #pil2 {
            order: 1;
        }
    }
`;

interface BulletProps {
    active: boolean;
}

const Bullet = styled.div<BulletProps>`
    width: 1rem;
    height: 1rem;
    background: transparent;
    border: 2px solid var(--orange);
    border-radius: 50%;
    opacity: 1;

    ${(props) => props.active && "background: var(--orange);"}
`;

const BulletContainer = styled.div`
    display: flex;
    flex-direction: row;
    justify-content: center;
    margin-top: 1rem;
    gap: 0.5rem;
`;

const slideVariants = {
    hiddenRight: {
        x: "3%",
        opacity: 0,
    },
    hiddenLeft: {
        x: "-3%",
        opacity: 0,
    },
    visible: {
        x: "0",
        opacity: 1,
        transition: {
            duration: 0.5,
        },
    },
    exit: {
        opacity: 0,
        scale: 0.8,
        transition: {
            duration: 0.5,
        },
    },
};
