import { Lenke } from "../components/fellesKomponenter/lenker/Lenke";
import { Tekst } from "../components/fellesKomponenter/typografi/Tekst";
import styled from "styled-components";
import { Heading } from "../components/fellesKomponenter/typografi/Heading";

interface TipsOgTriksProps {
    tittel: string;
    tekst: string[];
    lenker?: TipsOgTriksLenke[];
    bilder?: TipsOgTriksBilde[];
}

interface TipsOgTriksLenke {
    tekst: string;
    href: string;
}

interface TipsOgTriksBilde {
    src: string;
    alt: string;
    vekt?: string;
}

export const TipsOgTriks = ({ tekst, lenker, bilder, tittel }: TipsOgTriksProps) => (
    <StyledSection>
        <StyledHeading styleLevel={"3"}>{tittel}</StyledHeading>
        {tekst.map((tekst, index) => (
            <Tekst key={index}>{tekst}</Tekst>
        ))}
        {lenker && (
            <LenkeContainer>
                {lenker.map((lenke, index) => (
                    <li key={index}>
                        <Lenke href={lenke.href}>{lenke.tekst}</Lenke>
                    </li>
                ))}
            </LenkeContainer>
        )}
        {bilder && (
            <BildeSeksjon bildeVekter={genererBildeVektStreng(bilder)}>
                {bilder.map((bilde, index) => (
                    <BildeContainer key={index}>
                        <StyledImage src={bilde.src} alt={bilde.alt} loading={"lazy"} />
                    </BildeContainer>
                ))}
            </BildeSeksjon>
        )}
    </StyledSection>
);

const genererBildeVektStreng = (bilder: TipsOgTriksBilde[]) => {
    return bilder.map((bilde) => `${bilde.vekt || 1}fr`).join(" ");
};

const StyledSection = styled.section`
    display: flex;
    flex-direction: column;
    align-content: flex-start;
    height: fit-content;
    width: 100%;
    min-height: 500px;
`;

const StyledHeading = styled(Heading)`
    margin-top: 1rem;
`;

const LenkeContainer = styled.ul`
    display: flex;
    flex-direction: column;
    list-style: none;
    padding: 0;
`;

interface BildeSeksjonProps {
    bildeVekter: string;
}

const BildeSeksjon = styled.div<BildeSeksjonProps>`
    flex-grow: 1;
    display: grid;
    grid-template-columns: ${(props) => props.bildeVekter};
    min-height: 100px;
`;

const BildeContainer = styled.div`
    position: relative;
    width: 100%;
    height: 100%;
`;

const StyledImage = styled.img`
    position: absolute;
    top: 0;
    left: 0;
    max-width: 95%;
    max-height: 95%;
`;
