import styled from "styled-components";
import { Kortlenke } from "./Kortlenke";
import { Tekst } from "../../components/fellesKomponenter/typografi/Tekst";
import React from "react";
import { Heading } from "../../components/fellesKomponenter/typografi/Heading";
import { breakpointNettbrett } from "../../utils/constants";

interface EksterntLenkeKortProps {
    tittel: string;
    lenke: string;
    ingress: string;
    erFavoritt: boolean;
    håndterFavorittKlikket: (linkTittel: string, erFavoritt: boolean) => void;
}

export const EksterntLenkeKort = ({
    tittel,
    lenke,
    ingress,
    erFavoritt,
    håndterFavorittKlikket,
}: EksterntLenkeKortProps) => (
    <ListeElement>
        <Kortlenke href={lenke} target={"_blank"} rel={"noreferrer"}>
            <TittelWrapper>
                <Heading styleLevel="4" as="h2">
                    {tittel}
                </Heading>
                <FavorittIkon
                    favoritt={erFavoritt}
                    onClick={(event) => {
                        event.preventDefault();
                        håndterFavorittKlikket(tittel, erFavoritt);
                    }}
                />
            </TittelWrapper>
            {ingress && <Tekst>{ingress}</Tekst>}
        </Kortlenke>
    </ListeElement>
);

const ListeElement = styled.li`
    box-sizing: border-box;
    min-width: 17.5rem;
    width: 100%;

    @media (min-width: ${breakpointNettbrett}) {
        min-height: 9rem;
        width: 48%;
    }

    @media (min-width: 1024px) {
        min-height: 10rem;
        width: 31%;
    }
`;

const TittelWrapper = styled.div`
    flex-shrink: 0;
    display: flex;
    justify-content: space-between;
`;

const FavorittIkon = styled.i<{ favoritt?: boolean }>`
    width: 22px;
    height: 22px;
    background: url(${(props) =>
            props.favoritt ? "/ikon/favoritt_valgt.svg" : "/ikon/favoritt.svg"})
        no-repeat;
    background-size: contain;

    &:hover {
        background: url("/ikon/favoritt_valgt.svg") no-repeat;
        background-size: contain;
        cursor: pointer;
    }
`;
