import styled from "styled-components";
import React, { useContext } from "react";
import { Tekst } from "../../components/fellesKomponenter/typografi/Tekst";
import { Heading } from "../../components/fellesKomponenter/typografi/Heading";
import { KartdataContext } from "../../api/context/KartdataContext";
import { Lenke } from "../../components/fellesKomponenter/lenker/Lenke";
import { Modal } from "../../components/fellesKomponenter/modal/Modal";
import { Ikon } from "../../components/Ikon/Ikon";
import { ColumnWrapper } from "../../components/fellesKomponenter/wrappers/wrappers";

export const KartInfo = () => {
    const { intro } = useContext(KartdataContext);
    return intro.vis ? (
        <Modal
            onClose={() => intro.oppdater(false)}
            id="kartinfo"
            maksBredde={500}
            maksHøyde={window.innerHeight - 150}
        >
            <RowWrapper>
                <Ikon height={60} width={60} url="/ikon/kart/truck-plow-solid-sirkel.svg" />
                <ColumnWrapper>
                    <Header>
                        <Heading as="h1" styleLevel="2">
                            Om løsningen
                        </Heading>
                    </Header>
                    <Tekst størrelse={"S"} noMargin={true}>
                        Sist oppdatert: 20. juli 2023
                    </Tekst>
                </ColumnWrapper>
            </RowWrapper>

            <Tekst størrelse={"S"}>
                Vi jobber kontinuerlig for å gjøre løsningen bedre, send oss en mail om du har du
                innspill eller spørsmål (
                <Lenke størrelse={"S"} href="mailto:kos@vegvesen.no">
                    kos@vegvesen.no
                </Lenke>
                )
            </Tekst>
            <Tekst størrelse="S">
                Kartet viser vinterdrift som har blitt utført i kontraktsområdet fra det valgte
                tidspunktet og 12 timer tilbake i tid. Brøytebilene viser siste posisjon til biler
                som har vært aktive de siste 90 minuttene. Kartet henter data fra ELRAPP, Vegvær og
                Yr.no
            </Tekst>
            <br />
            <Heading as="h2" styleLevel="3">
                Mange muligheter i kartet!
            </Heading>
            <br />
            <ForklaringContainer>
                <BlåLinje />
                <ColumnWrapper>
                    <ForklaringTekst>Produksjon, roder eller begge deler?</ForklaringTekst>
                    <InfoTekst>
                        Du kan velge om du vil se bare produksjon, bare roder eller begge deler
                        samtidig.
                    </InfoTekst>

                    <ForklaringTekst>Hvordan finne syklustid?</ForklaringTekst>
                    <InfoTekst>
                        Dersom du har huket av for å se både roder og produksjon kan du bruke
                        tidsintervallene for å finne syklustid. Du kan også huke av og på de ulike
                        tidsintervallene. På denne måten kan du se om aktiviteten på en rode er
                        innenfor syklustiden for roden. NB! Noen roder kan har flere
                        vinterdriftsklasser, du finner den informasjonen ved å trykke på en rode -
                        enten i kartet eller i listen i menyen.
                    </InfoTekst>

                    <ForklaringTekst>Informasjon om produksjon</ForklaringTekst>
                    <RowWrapper>
                        <img
                            src="/kolonnevalg.png"
                            alt="Kolonnevalg"
                            height="120px"
                            loading={"lazy"}
                        />
                        <div>
                            <InfoTekst>
                                Ved å trykke på produksjon vil du få all info vi har på dette
                                punktet. Du kan selv velge hvilken informasjon som skal vises i
                                tabellen. Huk av for de du ønsker å se i tabellen. Du kan også
                                eksportere det du får opp til excel.
                            </InfoTekst>
                        </div>
                    </RowWrapper>

                    <ForklaringTekst>Tilpass kartet til ditt behov</ForklaringTekst>
                    <StyledUl>
                        Du kan skjule og vise informasjon i kartet;
                        <li>Brøytebiler</li>
                        <li>Trafikkmeldinger</li>
                        <li>Værstasjoner/kameraer</li>
                        <li>Forklaring på tidsintervall</li>
                    </StyledUl>
                    <ForklaringTekst>Dele med noen som ikke har tilgang?</ForklaringTekst>
                    <InfoTekst>
                        Du kan eksportere det du ser i kartet eller på en veglenke til excel. I
                        filen vil du få opp all informasjon og en lenke tilbake til kartet på det
                        tidspunktet du har satt.
                    </InfoTekst>
                </ColumnWrapper>
            </ForklaringContainer>
        </Modal>
    ) : null;
};

const ForklaringContainer = styled.div`
    display: flex;
    flex-direction: row;
`;

const BlåLinje = styled.div`
    position: relative;
    margin-right: 20px;

    &:before {
        position: absolute;
        top: 5px;
        bottom: 0;
        left: 0;
        border-left: 2px solid #183b79;
        width: 2px;
        content: "";
    }
`;

const ForklaringTekst = styled(Tekst).attrs(() => ({
    størrelse: "M",
    noMargin: true,
    bold: true,
}))`
    color: #183b79;
    display: flex;
    position: relative;

    &:before {
        position: absolute;
        left: -26px;
        top: 4px;
        background-color: #183b79;
        border-radius: 50%;
        content: "";
        flex-shrink: 0;
        height: 14px;
        width: 14px;
    }
`;

const InfoTekst = styled(Tekst).attrs(() => ({
    størrelse: "S",
    noMargin: true,
}))`
    margin-bottom: 20px;
`;

const StyledUl = styled(Tekst).attrs(() => ({
    as: "ul",
    størrelse: "S",
    noMargin: true,
}))`
    padding: 0;
    margin-bottom: 20px;
    & li {
        margin-left: 20px;
    }
`;

export const Header = styled.div`
    display: flex;
    justify-content: space-between;
    align-items: center;
`;

const RowWrapper = styled.div`
    display: flex;
    flex-direction: row;
    gap: 1.5rem;
    margin-bottom: 20px;
`;
