import React, { useContext, useRef } from "react";
import { AuthenticationWrapper } from "../components/AuthenticationWrapper";
import Head from "next/head";
import { Spinner } from "../components/fellesKomponenter/spinner/Spinner";
import { KartInfo } from "./KartInfo/KartInfo";
import styled, { createGlobalStyle } from "styled-components";
import { KartdataContext } from "../api/context/KartdataContext";
import ErrorBoundary from "../components/feil/ErrorBoundary";
import { Kontakt } from "./kartParameterVelger/Kontakt";
import { KlarteIkkeHenteKontrakterForBrukerFeilmelding } from "./feil/KlarteIkkeHenteKontrakterForBrukerFeilmelding";
import { ProduksjonskartInnhold } from "./kartInnhold/ProduksjonskartInnhold";
import { SideHeader } from "../components/sideheader/SideHeader";
import { HopptilHovedinnholdLenke } from "../components/fellesKomponenter/lenker/HoppTilHovedinnholdLenke";
import { useElementSize } from "../hooks/useElementSize";

export const KartSide = () => {
    const { kontrakt, brukerInfo, tidspunkt } = useContext(KartdataContext);
    const elementRef = useRef<HTMLDivElement>(null);
    const { height } = useElementSize(elementRef);

    return (
        <ErrorBoundary
            info={`${brukerInfo?.suksess ? brukerInfo.data.fulltnavn : "Ukjent bruker"}, ${
                kontrakt.aktiv?.navn
            }, ${tidspunkt.verdi.toISOString()}, elrappid: ${
                kontrakt.aktiv?.elrappId
            }, side: Produksjonskart`}
        >
            <Container>
                <Head>
                    <title>Produksjonskart - Driftsportal</title>
                </Head>
                <AuthenticationWrapper>
                    <HopptilHovedinnholdLenke hovedinnholdId={"kartparametervelger"} />
                    <div ref={elementRef}>
                        <SideHeader side={"produksjonskart"} />
                        <Kontakt />
                    </div>
                    <HeaderVariables height={height} />
                    {!brukerInfo ? (
                        <Spinner storrelse="liten" sentrert={true} inline={false}>
                            Henter kontrakter for bruker...
                        </Spinner>
                    ) : brukerInfo.suksess && kontrakt.finnes ? (
                        <>
                            <ProduksjonskartInnhold />
                            <KartInfo />
                        </>
                    ) : (
                        <KlarteIkkeHenteKontrakterForBrukerFeilmelding brukerInfo={brukerInfo} />
                    )}
                </AuthenticationWrapper>
            </Container>
        </ErrorBoundary>
    );
};

const Container = styled.div`
    display: flex;
    flex-direction: column;
    height: 100vh;
`;

const HeaderVariables = createGlobalStyle<{ height: number }>`
    :root {
        --header-height: ${({ height }) => height}px;
    }
`;
