import styled from "styled-components";
import { Tekst } from "../../../components/fellesKomponenter/typografi/Tekst";
import { Ikon } from "../../../components/Ikon/Ikon";
import { useContext, useRef, useState } from "react";
import { KnappUtenStyling } from "../../../components/fellesKomponenter/knapper/KnappUtenStyling";
import { TidsfilterContext } from "../../../api/context/TidsfilterContext";
import { Intervall } from "./Intervall";
import { useDisableClickPropagation } from "../../../hooks/useDisableClickPropagation";
import { TidsintervallFarge } from "../../utils/tidsintervallUtils";
import { erMuseKlikk } from "../../utils/kartUtils";

export const IntervallLegend = () => {
    const intrervallLegendRef = useRef(null);
    const { tidsfilter, oppdaterFilter } = useContext(TidsfilterContext);
    const [erRedigeringsTilstand, setErRedigeringsTilstand] = useState(false);
    const [automatiskFokus, setAutomatiskFokus] = useState(false);

    useDisableClickPropagation(intrervallLegendRef);

    const oppdaterTidsfilter = (indeks: number) => {
        const tidsfilterKopi = [...tidsfilter];
        tidsfilterKopi[indeks].valgt = !tidsfilterKopi[indeks].valgt;
        oppdaterFilter(tidsfilterKopi);
    };

    const resetTidsfilter = () => {
        const tidsfilterKopi = [...tidsfilter];
        tidsfilterKopi.forEach((filter) => (filter.valgt = true));
        oppdaterFilter(tidsfilterKopi);
    };

    const handleEndreKlikk = (e: React.MouseEvent<HTMLElement>) => {
        setAutomatiskFokus(erMuseKlikk(e));
        setErRedigeringsTilstand(!erRedigeringsTilstand);
    };

    const gråFarge = "#C0C0C0";

    return (
        <Wrapper ref={intrervallLegendRef}>
            <HeaderStyled størrelse="S" bold={true}>
                Tid siden siste aktivitet
            </HeaderStyled>
            <Intervall
                farge={tidsfilter[0].valgt ? TidsintervallFarge.time1h30 : gråFarge}
                erRedigeringsTilstand={erRedigeringsTilstand}
                valgt={tidsfilter[0].valgt}
                oppdaterTidsfilter={() => oppdaterTidsfilter(0)}
                idString={"mindreEnn1,5time"}
                automatiskFokus={automatiskFokus}
            >
                Mindre enn 1,5 time
            </Intervall>
            <Intervall
                farge={tidsfilter[1].valgt ? TidsintervallFarge.time2h : gråFarge}
                erRedigeringsTilstand={erRedigeringsTilstand}
                valgt={tidsfilter[1].valgt}
                oppdaterTidsfilter={() => oppdaterTidsfilter(1)}
                idString={"1,5til2timer"}
            >
                1,5 - 2 timer
            </Intervall>
            <Intervall
                farge={tidsfilter[2].valgt ? TidsintervallFarge.time3h : gråFarge}
                erRedigeringsTilstand={erRedigeringsTilstand}
                valgt={tidsfilter[2].valgt}
                oppdaterTidsfilter={() => oppdaterTidsfilter(2)}
                idString={"2til3timer"}
            >
                2 - 3 timer
            </Intervall>
            <Intervall
                farge={tidsfilter[3].valgt ? TidsintervallFarge.time4h : gråFarge}
                erRedigeringsTilstand={erRedigeringsTilstand}
                valgt={tidsfilter[3].valgt}
                oppdaterTidsfilter={() => oppdaterTidsfilter(3)}
                idString={"3til4timer"}
            >
                3 - 4 timer
            </Intervall>
            <Intervall
                farge={tidsfilter[4].valgt ? TidsintervallFarge.time12h : gråFarge}
                erRedigeringsTilstand={erRedigeringsTilstand}
                valgt={tidsfilter[4].valgt}
                oppdaterTidsfilter={() => oppdaterTidsfilter(4)}
                idString={"4til12timer"}
            >
                4 - 12 timer
            </Intervall>
            <RedigerContainer rediger={erRedigeringsTilstand}>
                {erRedigeringsTilstand && (
                    <KnappUtenStyling onClick={() => resetTidsfilter()}>
                        <KnappTekst>Tilbakestill</KnappTekst>
                    </KnappUtenStyling>
                )}
                <KnappUtenStyling
                    onClick={(e: React.MouseEvent<HTMLElement>) => handleEndreKlikk(e)}
                >
                    <KnappTekst>{erRedigeringsTilstand ? "Lagre" : "Endre"}</KnappTekst>
                    <StyledIkon height={10} width={10} url="/ikon/endre-blyant.svg" />
                </KnappUtenStyling>
            </RedigerContainer>
        </Wrapper>
    );
};

const KnappTekst = styled.span`
    color: var(--gray-dark-65);
    border-bottom: 1px solid var(--gray-dark-65);
    font-size: 10px;

    &:hover {
        color: var(--gray-dark);
        border-bottom: 1px solid var(--gray-dark);
    }
`;

const StyledIkon = styled(Ikon)`
    margin-left: 5px;
`;

const RedigerContainer = styled.div<{ rediger: boolean }>`
    display: flex;
    justify-content: space-between;
    align-items: center;
    margin-top: 4px;
    ${(props) => !props.rediger && `justify-content: flex-end;`}
`;

const Wrapper = styled.div`
    background-color: var(--white);
    box-shadow: 0 2px 4px 0 rgba(0, 0, 0, 0.25);
    padding: 1rem;
    width: max-content;
    position: absolute;
    z-index: 1000;
    right: 3px;
    top: 8px;
    display: flex;
    flex-direction: column;
    cursor: auto;
`;

const HeaderStyled = styled(Tekst)`
    margin-top: 0;
    margin-bottom: 0.5rem;
`;
