import { useContext, useEffect, useRef } from "react";
import { KartdataContext } from "../../api/context/KartdataContext";
import {
    InformasjonsmeldingContext,
    informasjonsmeldinger,
    InformasjonsmeldingType,
} from "../../api/context/InformasjonsmeldingContext";
import { ProduksjonsfilterContext } from "../../api/context/ProduksjonsfilterContext";
import { HastighetsfilterContext } from "../../api/context/HastighetsfilterContext";
import { ØktfilterContext } from "../../api/context/ØktContext";
import { filtrerProduksjonslenker } from "../filtre/filtrerProduksjon/filtrerProduksjon";
import { Produksjonslenke } from "../types/produksjonslenke";
import { TidsfilterContext } from "../../api/context/TidsfilterContext";

interface Props {
    produksjonslenker: Produksjonslenke[];
    settFiltrerteProduksjonslenker: (produksjonslenker: Produksjonslenke[]) => void;
}

export const useSetFiltrerteLenker = ({
    produksjonslenker,
    settFiltrerteProduksjonslenker,
}: Props) => {
    const { tidspunkt } = useContext(KartdataContext);
    const informasjonsmelding = useContext(InformasjonsmeldingContext);
    const { produksjonsfilter } = useContext(ProduksjonsfilterContext);
    const { hastighetsfilter } = useContext(HastighetsfilterContext);
    const { øktfilter, hentØkt } = useContext(ØktfilterContext);
    const { tidsfilter } = useContext(TidsfilterContext);

    useEffect(() => {
        const filtrertelenker = filtrerProduksjonslenker(
            produksjonsfilter,
            hastighetsfilter,
            øktfilter,
            tidsfilter,
            produksjonslenker
        );
        settFiltrerteProduksjonslenker(filtrertelenker);
        øktfilter.verdi && hentOgSettØkt();

        if (skalIngenDataForFiltereMeldingVises(filtrertelenker, produksjonslenker)) {
            informasjonsmelding.oppdater(informasjonsmeldinger.ingenDataForFiltere);
        } else {
            if (informasjonsmelding.verdi?.type === InformasjonsmeldingType.ingenDataForFiltere) {
                informasjonsmelding.fjern();
            }
        }
    }, [produksjonsfilter, hastighetsfilter, øktfilter, tidsfilter]);

    const skalIngenDataForFiltereMeldingVises = (
        filtrerteProduksjonslenker: Produksjonslenke[] | undefined,
        produksjonslenker: Produksjonslenke[]
    ) =>
        filtrerteProduksjonslenker &&
        filtrerteProduksjonslenker.length === 0 &&
        !(produksjonslenker.length === 0);

    const hentOgSettØkt = async () => {
        const øktProduksjonslenker = await hentØkt(øktfilter.verdi, tidspunkt.verdi);
        if (øktProduksjonslenker?.suksess) {
            settFiltrerteProduksjonslenker(
                filtrerProduksjonslenker(
                    produksjonsfilter,
                    hastighetsfilter,
                    øktfilter,
                    tidsfilter,
                    øktProduksjonslenker.data
                )
            );
        }
    };
};
