import { useContext, useEffect, useMemo, useState } from "react";
import { Filter, Filtertype } from "../../../api/context/ProduksjonsfilterContext";
import { ProduksjonslenkerContext } from "../../../api/context/ProduksjonslenkerContext";
import { Produksjonslenke } from "../../types/produksjonslenke";
import { aktivitetTilFiltertype } from "../../types/aktivitet";
import { materialeTilFiltertype } from "../../types/materiale";

interface FilterUtilsProps {
    setDefaultFiltreValgt: (val: boolean) => void;
    produksjonsfilter: Filter[];
    oppdaterFilter: (verdi: Filter[]) => void;
}

export const useDropdownFilterUtils = ({
    setDefaultFiltreValgt,
    produksjonsfilter,
    oppdaterFilter,
}: FilterUtilsProps) => {
    const { produksjonslenker } = useContext(ProduksjonslenkerContext);
    const filtrertekategorierIProduksjonslenker: Filtertype[] | undefined = useMemo(
        () =>
            produksjonslenker?.suksess
                ? produksjonslenkerTilFiltertyper(produksjonslenker.data)
                : undefined,
        [produksjonslenker]
    );

    const setDefaultFilterForProduksjonsdata = () => {
        if (filtrertekategorierIProduksjonslenker != undefined) {
            oppdaterFilter(
                produksjonsfilter.map((valg) => ({
                    ...valg,
                    valgt:
                        filtrertekategorierIProduksjonslenker!.includes(valg.type) &&
                        valg.type !== Filtertype.IngenProduksjon,
                }))
            );
        }
    };

    const [defaultProduksjonstypeFilter, setDefaultProduksjonstypeFilter] = useState<Filtertype[]>(
        []
    );

    useEffect(() => {
        setDefaultFilterForProduksjonsdata();
        const kunMaterialeOgProduksjon = filtrertekategorierIProduksjonslenker
            ? filtrertekategorierIProduksjonslenker
                  .filter((filter) => filter !== Filtertype.IngenProduksjon)
                  .sort()
            : [];
        setDefaultProduksjonstypeFilter(kunMaterialeOgProduksjon);
    }, [filtrertekategorierIProduksjonslenker]);

    const valgteFiltere = produksjonsfilter
        .filter((filter) => filter.valgt)
        .map((filter) => filter.type)
        .sort();

    const defaultOgValgteFiltereErLike =
        JSON.stringify(valgteFiltere) === JSON.stringify(defaultProduksjonstypeFilter);

    useEffect(() => {
        setDefaultFiltreValgt(defaultOgValgteFiltereErLike);
    }, [defaultOgValgteFiltereErLike]);

    return { filtrertekategorierIProduksjonslenker, setDefaultFilterForProduksjonsdata };
};

export const produksjonslenkerTilFiltertyper = (
    produksjonsdata: Produksjonslenke[]
): Filtertype[] => {
    return produksjonsdata.reduce((result: Filtertype[], lenke) => {
        const produksjonsaktiviteter =
            lenke.produksjonstyper.length > 0
                ? lenke.produksjonstyper.reduce((res: Filtertype[], produksjonstype) => {
                      produksjonstype.ibruk &&
                          res.push(aktivitetTilFiltertype(produksjonstype.aktivitet));
                      return res;
                  }, [])
                : [];
        const produksjonsmatriale = lenke.produksjonsmateriale
            ? [materialeTilFiltertype(lenke.produksjonsmateriale)]
            : [];
        const ingenProduksjon =
            lenke.produksjonsmateriale === undefined &&
            (lenke.produksjonstyper.length === 0 ||
                !lenke.produksjonstyper.map((p) => p.ibruk).some((p) => p))
                ? [Filtertype.IngenProduksjon]
                : [];
        [...produksjonsaktiviteter, ...produksjonsmatriale, ...ingenProduksjon].forEach(
            (element) => {
                if (!result.includes(element)) {
                    result.push(element);
                }
            }
        );
        return result;
    }, []);
};
