import { Filtergruppe } from "./Filtergruppe";
import React, { useContext } from "react";
import {
    Filterkategori,
    ProduksjonsfilterContext,
} from "../../../api/context/ProduksjonsfilterContext";
import styled from "styled-components";
import {
    fontsizeLargeScreen,
    fontsizeSmallScreen,
} from "../../../components/fellesKomponenter/typografi/TekstStyles";
import { AnimatePresence } from "framer-motion";
import { breakpointNettbrett } from "../../../utils/constants";

interface ProduksjonsdetaljerFiltrerVelgerProps {
    åpen: boolean;
}

export const ProduksjonsdataFilterVelger = ({ åpen }: ProduksjonsdetaljerFiltrerVelgerProps) => {
    const { filtrertekategorierIProduksjonslenker, setDefaultFilterForProduksjonsdata } =
        useContext(ProduksjonsfilterContext);

    return (
        <AnimatePresence>
            {åpen && (
                <>
                    <NullstillButton onClick={setDefaultFilterForProduksjonsdata}>
                        Velg all produksjon
                    </NullstillButton>
                    <Filtergruppe
                        filternavn="Brøyting"
                        filterkategori={Filterkategori.Brøyting}
                        aktiveFiltertyper={filtrertekategorierIProduksjonslenker ?? []}
                    />
                    <Filtergruppe
                        filternavn="Høvling"
                        filterkategori={Filterkategori.Høvling}
                        aktiveFiltertyper={filtrertekategorierIProduksjonslenker ?? []}
                    />
                    <Filtergruppe
                        filternavn="Sand"
                        filterkategori={Filterkategori.Sand}
                        aktiveFiltertyper={filtrertekategorierIProduksjonslenker ?? []}
                    />
                    <Filtergruppe
                        filternavn="Salt"
                        filterkategori={Filterkategori.Salt}
                        aktiveFiltertyper={filtrertekategorierIProduksjonslenker ?? []}
                    />
                    <Filtergruppe
                        filternavn="Ingen produksjon"
                        filterkategori={Filterkategori.IngenProduksjon}
                        aktiveFiltertyper={filtrertekategorierIProduksjonslenker ?? []}
                    />
                </>
            )}
        </AnimatePresence>
    );
};

const NullstillButton = styled.button`
    background-color: var(--white);
    border: none;
    cursor: pointer;
    display: block;
    text-align: left;
    ${fontsizeLargeScreen};
    margin: 8px 2px 8px 2px;
    padding: 0 0 8px 9px;
    text-decoration: underline;
    border-bottom: 1px solid;
    width: calc(100% - 4px);
    color: var(--gray-dark);

    @media (max-width: ${breakpointNettbrett}) {
        ${fontsizeSmallScreen};
    }
`;
