import { Filter, MaterialOgAktivitetsNavn } from "../../../api/context/ProduksjonsfilterContext";
import { Produksjonstype } from "../../types/produksjonstype";
import { AktivitetKategori, aktivitetTilFiltertype, AktivitetType } from "../../types/aktivitet";
import { Materiale, materialeTilFiltertype } from "../../types/materiale";
import { Produksjonslenke } from "../../types/produksjonslenke";
import { Produksjonsdetaljer } from "../../types/produksjonsdetaljer";
import { Hastighetsfilter } from "../../../api/context/HastighetsfilterContext";
import { Øktfilter } from "../../../api/context/ØktContext";
import { Tidsfilter } from "../../../api/context/TidsfilterContext";
import { RodeDto } from "../../../api/useHentRoder";
import {
    finnDriftsklasseMedHøyesteSyklustid,
    hentSyklusTidFraKlasser,
    Vinterdriftsklasse,
} from "../../popup/roder/vinterdriftsklasserUtils";
import { ApiRespons } from "../../../api/api";
import {
    informasjonsmeldinger,
    InformasjonsmeldingValue,
} from "../../../api/context/InformasjonsmeldingContext";

export type FilterKriterie = {
    produksjonstyper: Produksjonstype[];
    materiale?: Materiale;
    hastighetKmH: number;
};
export const filtrerProduksjonslenker = (
    produksjonsfilter: Filter[],
    hastighetsfilter: Hastighetsfilter,
    øktfilter: Øktfilter,
    tidsfilter: Tidsfilter[],
    produksjonslenker: Produksjonslenke[]
): Produksjonslenke[] =>
    produksjonslenker?.filter((produksjonslenke) =>
        erFilterGjeldene(
            produksjonsfilter,
            hastighetsfilter,
            øktfilter,
            produksjonslenkeTilFilterKriterie(produksjonslenke),
            tidsfilter,
            produksjonslenke.minutterSiden,
            produksjonslenke.hastighetKmH,
            produksjonslenke.økt
        )
    );

export const filtrerProduksjonsdetaljer = (
    produksjonsfilter: Filter[],
    hastighetsfilter: Hastighetsfilter,
    øktfilter: Øktfilter,
    tidsfilter: Tidsfilter[],
    produksjonsdetaljer: Produksjonsdetaljer[]
): Produksjonsdetaljer[] =>
    produksjonsdetaljer.filter((produksjonsdetalj) =>
        erFilterGjeldene(
            produksjonsfilter,
            hastighetsfilter,
            øktfilter,
            produksjonsdetaljTilFilterKriterie(produksjonsdetalj),
            tidsfilter,
            produksjonsdetalj.minutterSiden,
            produksjonsdetalj.hastighetKmHAvg,
            produksjonsdetalj.eksternReferanse
        )
    );

export const erProduksjonInnenSyklustid = (
    produksjonsdetalj: Produksjonsdetaljer,
    roder: RodeDto[]
) => {
    const driftsklasseMedHøyestSyklustid = finnDriftsklasseMedHøyesteSyklustid(roder);
    const maksSyklusTid = driftsklasseMedHøyestSyklustid
        ? hentSyklusTidFraKlasser(driftsklasseMedHøyestSyklustid)
        : 0;
    const tidSidenLoggetProduksjon = produksjonsdetalj.minutterSiden / 60;
    return tidSidenLoggetProduksjon <= maksSyklusTid;
};

export const erFilterGjeldene = (
    produksjonsfilter: Filter[],
    hastighetsfilter: Hastighetsfilter,
    øktfilter: Øktfilter,
    filterkriterie: FilterKriterie,
    tidsfilter: Tidsfilter[],
    minutterSiden: number,
    hastighet: number,
    økt?: string
): boolean => {
    const produksjonsFilterErGjeldende = filtrerProduksjon(produksjonsfilter, filterkriterie);

    const hastighetsfilterErGjeldendeHvisValgt = hastighetsfilter.valgt
        ? erOverHastighet(hastighet, hastighetsfilter.verdi)
        : true;

    const øktfilterErGjeldendeHvisValgt = øktfilter.valgt ? øktfilter.verdi === økt : true;

    const tidsfilterErGjeldende = erInnenforTidsfilter(tidsfilter, minutterSiden);

    return (
        produksjonsFilterErGjeldende &&
        hastighetsfilterErGjeldendeHvisValgt &&
        øktfilterErGjeldendeHvisValgt &&
        tidsfilterErGjeldende
    );
};

export const produksjonslenkeTilFilterKriterie = (
    produksjonslenke: Produksjonslenke
): FilterKriterie => ({
    produksjonstyper: produksjonslenke.produksjonstyper,
    materiale: produksjonslenke.produksjonsmateriale,
    hastighetKmH: produksjonslenke.hastighetKmH,
});

const produksjonsdetaljTilFilterKriterie = (
    produksjonsdetaljer: Produksjonsdetaljer
): FilterKriterie => ({
    produksjonstyper: finnProduksjonstyperForProduksjonsdetaljer(produksjonsdetaljer),
    materiale: produksjonsdetaljer.materiale,
    hastighetKmH: produksjonsdetaljer.hastighetKmHAvg,
});

const finnProduksjonstyperForProduksjonsdetaljer = (
    produksjonsdetaljer: Produksjonsdetaljer
): Produksjonstype[] => {
    const produksjonstyper: Produksjonstype[] = [];

    produksjonsdetaljer.erPlogNede !== undefined &&
        produksjonstyper.push({
            aktivitet: {
                navn: MaterialOgAktivitetsNavn.PlogNede,
                type: AktivitetType.PlogNede,
                kategori: AktivitetKategori.Brøyting,
            },
            ibruk: produksjonsdetaljer.erPlogNede,
        });

    produksjonsdetaljer.erSideplogIBruk !== undefined &&
        produksjonstyper.push({
            aktivitet: {
                navn: MaterialOgAktivitetsNavn.Sideplog,
                type: AktivitetType.Sideplog,
                kategori: AktivitetKategori.Brøyting,
            },
            ibruk: produksjonsdetaljer.erSideplogIBruk,
        });

    produksjonsdetaljer.erUnderliggendeSkjærNede !== undefined &&
        produksjonstyper.push({
            aktivitet: {
                navn: MaterialOgAktivitetsNavn.UnderliggendeSkjær,
                type: AktivitetType.UnderliggendeSkjær,
                kategori: AktivitetKategori.Høvling,
            },
            ibruk: produksjonsdetaljer.erUnderliggendeSkjærNede,
        });

    produksjonsdetaljer.erHøvelskjærNede !== undefined &&
        produksjonstyper.push({
            aktivitet: {
                navn: MaterialOgAktivitetsNavn.TungHøvel,
                type: AktivitetType.TungHøvel,
                kategori: AktivitetKategori.Høvling,
            },
            ibruk: produksjonsdetaljer.erHøvelskjærNede,
        });
    return produksjonstyper;
};

const filtrerProduksjon = (produksjonsfilter: Filter[], filterKriterie: FilterKriterie): boolean =>
    produksjonsfilter.some(
        (filter) =>
            filter.valgt &&
            (erFiltertMedAktivitetFilter(filter, filterKriterie.produksjonstyper) ||
                erFiltertMedMaterialeFilter(filter, filterKriterie) ||
                erFiltertMedIngenProduksjonFilter(filter, filterKriterie))
    );

const erFiltertMedAktivitetFilter = (
    filter: Filter,
    produksjonstyper: Produksjonstype[]
): boolean =>
    produksjonstyper.some(
        (produksjon) =>
            aktivitetTilFiltertype(produksjon.aktivitet) === filter.type && produksjon.ibruk
    );

const erFiltertMedMaterialeFilter = (
    filter: Filter,
    filterKriterie: FilterKriterie
): boolean | undefined =>
    filterKriterie.materiale && materialeTilFiltertype(filterKriterie.materiale) === filter.type;

const erFiltertMedIngenProduksjonFilter = (
    filter: Filter,
    filterKriterie: FilterKriterie
): boolean =>
    erProduksjonsTypeUtenAktivitet(filterKriterie) &&
    filterKriterie.materiale === undefined &&
    filter.type === "IngenProduksjon";

const erProduksjonsTypeUtenAktivitet = (filterKriterie: FilterKriterie): boolean =>
    filterKriterie.produksjonstyper.length === 0 ||
    !filterKriterie.produksjonstyper.some((lenke) => lenke.ibruk);

const erOverHastighet = (hastighet: number, hastighetsfilter: string): boolean =>
    hastighet >= Number(hastighetsfilter);

const erInnenforTidsfilter = (tidsfilter: Tidsfilter[], minutterSiden: number): boolean => {
    return tidsfilter.some(
        (filter) =>
            filter.valgt &&
            minutterSiden >= filter.startTidspunkt &&
            minutterSiden <= filter.sluttTidspunkt
    );
};
