import "leaflet/dist/leaflet.css";
import React, { useContext } from "react";
import { useHentWebkamera } from "../../api/useHentWebkamera";
import BrøytebilSpinner from "../brøytebilSpinner/BrøytebilSpinner";
import { KartvisningFeil } from "../feil/KartvisningFeil";
import dynamic from "next/dynamic";
import { KartdataContext } from "../../api/context/KartdataContext";
import { ProduksjonslenkerContext } from "../../api/context/ProduksjonslenkerContext";
import { useHentSisteKjøretøy } from "../../api/useHentSisteKjøretøy";
import { hentFeil } from "../utils/kartSideUtils";
import { ProduksjonskartVisningProps } from "../Kartvisning/ProduksjonskartVisning";
import { RoderContext } from "../../api/context/RodeContext";

export const ProduksjonskartInnhold = () => {
    const { kontrakt, tidspunkt } = useContext(KartdataContext);
    const { produksjonslenker } = useContext(ProduksjonslenkerContext);
    const { roder } = useContext(RoderContext);
    const kameradata = useHentWebkamera();
    const kjøretøydata = useHentSisteKjøretøy(kontrakt.aktiv?.elrappId, tidspunkt.verdi);

    return (
        <>
            {!roder || !kameradata || !produksjonslenker || !kjøretøydata ? (
                <BrøytebilSpinner />
            ) : !(
                  roder.suksess &&
                  kameradata.suksess &&
                  kjøretøydata.suksess &&
                  produksjonslenker.suksess
              ) ? (
                <KartvisningFeil
                    feilmeldinger={hentFeil([roder, kameradata, produksjonslenker, kjøretøydata])}
                />
            ) : (
                <DynamicKartvisning
                    produksjonslenker={produksjonslenker.suksess ? produksjonslenker.data : []}
                    kjøretøydata={kjøretøydata.suksess ? kjøretøydata.data : []}
                    roder={roder?.suksess ? roder.data : []}
                    kameradata={kameradata.suksess ? kameradata.data : []}
                />
            )}
        </>
    );
};

const DynamicKartvisning = dynamic<ProduksjonskartVisningProps>(
    () => import("../Kartvisning/ProduksjonskartVisning").then((mod) => mod.ProduksjonskartVisning),
    {
        loading: () => <BrøytebilSpinner />,
        ssr: false,
    }
);
