import React, { useState } from "react";
import { FormLabel } from "../../components/fellesKomponenter/inputfelt/FormLabel";
import { HjelpetekstPopup } from "../../components/fellesKomponenter/HjelpetekstPopup";
import { KnappPrimær } from "../../components/fellesKomponenter/knapper/KnappPrimær";
import { KnappSekundær } from "../../components/fellesKomponenter/knapper/KnappSekundær";
import { Inputfelt } from "../../components/fellesKomponenter/inputfelt/Inputfelt";
import { Spinner } from "../../components/fellesKomponenter/spinner/Spinner";
import { ColumnWrapper, RowWrapper } from "../../components/fellesKomponenter/wrappers/wrappers";
import styled from "styled-components";

interface DataInputProps {
    tittel: string;
    handleSubmit: (data: string) => void;
    errormelding: string[];
    error: boolean;
    setError: (error: boolean) => void;
    hjelpetekst?: string;
    laster?: boolean;
    resetData?: () => void;
}

export const DataInput = ({
    handleSubmit,
    resetData,
    hjelpetekst,
    tittel,
    errormelding,
    laster,
    error,
    setError,
}: DataInputProps) => {
    const [data, setData] = useState<string>("");
    const nullstillData = () => {
        setData("");
        setError(false);
        resetData && resetData();
    };

    return (
        <ColumnWrapper>
            <StyledRowWrapper>
                <FormLabel htmlFor={tittel}>{tittel}</FormLabel>
                {hjelpetekst && (
                    <IkonWrapper>
                        <HjelpetekstPopup plassering={"right center"} størrelse={"S"}>
                            {hjelpetekst}
                        </HjelpetekstPopup>
                    </IkonWrapper>
                )}
            </StyledRowWrapper>

            <StyledRowWrapper>
                <Inputfelt
                    error={error}
                    type="text"
                    id={tittel}
                    størrelse="S"
                    onChange={(e) => setData(e.target.value)}
                    value={data}
                    pattern="[0-9]+"
                />
                <KnappPrimær size={"Small"} onClick={() => handleSubmit(data)} disabled={laster}>
                    {laster ? <Spinner storrelse="mindre" inline={true} /> : "Ok"}
                </KnappPrimær>
                {resetData && (
                    <KnappSekundær size={"Small"} onClick={nullstillData} disabled={laster}>
                        Nullstill
                    </KnappSekundær>
                )}
            </StyledRowWrapper>
            {error &&
                errormelding.map((melding, index) => (
                    <ErrorMelding key={index}>{melding}</ErrorMelding>
                ))}
        </ColumnWrapper>
    );
};

const StyledRowWrapper = styled(RowWrapper)`
    gap: 0.5rem;
    align-items: flex-start;
`;

export const IkonWrapper = styled.div`
    align-self: flex-start;
    margin: 0 0 5px 0;
`;

export const ErrorMelding = styled.p`
    color: var(--gray-dark);
    margin: 0 0 0 0;
    position: relative;
    background-color: var(--red-bg);
    padding: 2px;
`;
