import React, { useContext, useEffect, useState } from "react";
import styled from "styled-components";
import { Datofelt } from "../../components/fellesKomponenter/inputfelt/Datofelt";
import { KlokkeslettFelt } from "../../components/fellesKomponenter/inputfelt/Klokkelsettfelt";
import { KnappPrimær } from "../../components/fellesKomponenter/knapper/KnappPrimær";
import dayjs, { Dayjs } from "dayjs";
import { KartdataContext } from "../../api/context/KartdataContext";
import { settKartUrlParam } from "../utils/kartUrlUtils";
import { KnappSekundær } from "../../components/fellesKomponenter/knapper/KnappSekundær";
import { useRouter } from "next/router";
import { useKjoretoyProduksjonsdetaljerCache } from "../../api/context/KjoretoyProduksjonsdetaljerCacheContext";
import { useProduksjonsdetaljerCache } from "../../api/context/ProduksjonsdetaljerCacheContext";
import {
    InformasjonsmeldingContext,
    informasjonsmeldinger,
} from "../../api/context/InformasjonsmeldingContext";
import { ProduksjonslenkerContext } from "../../api/context/ProduksjonslenkerContext";
import {
    defaultProduksjonsfilter,
    ProduksjonsfilterContext,
} from "../../api/context/ProduksjonsfilterContext";

export const DatoInput = () => {
    const { tidspunkt, kontrakt } = useContext(KartdataContext);
    const { produksjonslenker } = useContext(ProduksjonslenkerContext);
    const informasjonsmelding = useContext(InformasjonsmeldingContext);
    const filter = useContext(ProduksjonsfilterContext);
    const kjoretoyProduksjonsdetaljerCache = useKjoretoyProduksjonsdetaljerCache();
    const produksjonsdetaljerCache = useProduksjonsdetaljerCache();
    const { pathname } = useRouter();
    const lasterKartdata = produksjonslenker === undefined;
    const [formDato, setFormDato] = useState(tidspunkt.verdi);
    const [formKlokkeslett, setFormKlokkeslett] = useState(tidspunkt.verdi);

    useEffect(() => {
        kontrakt.aktiv &&
            settKartUrlParam(pathname, {
                elrappKontraktId: kontrakt.aktiv.elrappId,
                timestamp: tidspunkt.verdi,
            });
    }, [produksjonslenker]);

    const handleSubmit = (e: React.FormEvent<HTMLFormElement>) => {
        e.preventDefault();
        validerOgSubmit();
    };

    const validerOgSubmit = (nå?: boolean) => {
        const endreTilDato = nå ? dayjs() : formDato;
        const endreTilKlokkeslett = nå ? dayjs() : formKlokkeslett;
        const formOgContextErLike =
            endreTilDato.date() === tidspunkt.verdi.date() &&
            endreTilDato.month() === tidspunkt.verdi.month() &&
            endreTilDato.year() === tidspunkt.verdi.year() &&
            endreTilKlokkeslett.hour() === tidspunkt.verdi.hour() &&
            endreTilKlokkeslett.minute() === tidspunkt.verdi.minute();
        if (!validerSkjema(endreTilDato, endreTilKlokkeslett)) {
            informasjonsmelding.oppdater(informasjonsmeldinger.tidspunktFremITid);
        } else if (formOgContextErLike) {
            informasjonsmelding.oppdater(informasjonsmeldinger.viserAltSisteProduksjon);
        } else if (!lasterKartdata) {
            informasjonsmelding.fjern();
            tidspunkt.oppdater(
                endreTilDato
                    .set("hour", endreTilKlokkeslett.hour())
                    .set("minute", endreTilKlokkeslett.minute())
            );
            tømCache();
            filter.oppdaterFilter(defaultProduksjonsfilter);
        }
    };

    const settNåtid = (e: React.FormEvent<HTMLButtonElement>) => {
        e.preventDefault();
        const nå = dayjs();
        setFormDato(nå);
        setFormKlokkeslett(nå);
        validerOgSubmit(true);
    };

    const tømCache = () => {
        kjoretoyProduksjonsdetaljerCache.tømCache();
        produksjonsdetaljerCache.tømCache();
    };

    return (
        <StyledForm onSubmit={handleSubmit}>
            <Datofelt dato={formDato} setDato={setFormDato} />
            <RowWrapper>
                <KlokkeslettFelt
                    klokkeslett={formKlokkeslett}
                    settKlokkeslett={setFormKlokkeslett}
                    aria-controls="informasjonsmelding"
                />
                <KnappSekundær
                    size="Small"
                    type="button"
                    onClick={settNåtid}
                    disabled={lasterKartdata}
                >
                    Nå
                </KnappSekundær>
            </RowWrapper>
            <KnappPrimær type="submit" disabled={lasterKartdata} size="Small">
                Hent data
            </KnappPrimær>
        </StyledForm>
    );
};

const validerSkjema = (dato: Dayjs, tidspunkt: Dayjs) =>
    dato.isBefore(dayjs(), "day") ||
    (dato.isSame(dayjs(), "day") && (tidspunkt.isBefore(dayjs()) || tidspunkt.isSame(dayjs())));

const StyledForm = styled.form`
    max-width: max-content;
    display: flex;
    flex-direction: column;
    gap: 0.7rem;
    align-items: flex-start;
    flex-shrink: 0;
`;

const RowWrapper = styled.div`
    display: flex;
    flex-direction: row;
    gap: 0.7rem;
    align-items: flex-end;
`;
