import { Produksjonslenke } from "../types/produksjonslenke";
import { useContext, useRef, useState } from "react";
import { KartdataContext } from "../../api/context/KartdataContext";
import { ProduksjonsfilterContext } from "../../api/context/ProduksjonsfilterContext";
import { ProduksjonsdetaljerCacheContext } from "../../api/context/ProduksjonsdetaljerCacheContext";
import { ApiRespons } from "../../api/api";
import { loggFrontend, LoggNivå } from "../../utils/loggFrontend";
import styled from "styled-components";
import { downloadExcel, EksporterTilExcel } from "../../utils/EksporterTilExcel";
import { Spinner } from "../../components/fellesKomponenter/spinner/Spinner";
import { Tekst } from "../../components/fellesKomponenter/typografi/Tekst";
import { Tekstpil } from "../../components/Ikon/Tekstpil";
import { RoderContext } from "../../api/context/RodeContext";
import { RodeDto } from "../../api/useHentRoder";
import {
    InformasjonsmeldingContext,
    informasjonsmeldinger,
} from "../../api/context/InformasjonsmeldingContext";
import { KnappUtenStyling } from "../../components/fellesKomponenter/knapper/KnappUtenStyling";
import { Produksjonsdetaljer } from "../types/produksjonsdetaljer";
import { filtrerProduksjonsdetaljer } from "../filtre/filtrerProduksjon/filtrerProduksjon";
import { HastighetsfilterContext } from "../../api/context/HastighetsfilterContext";
import { ØktfilterContext } from "../../api/context/ØktContext";
import { TidsfilterContext } from "../../api/context/TidsfilterContext";

interface EksporterAllDataProps {
    produksjonslenker: ApiRespons<Produksjonslenke[]> | undefined;
}
export const EksporterAllData = ({ produksjonslenker }: EksporterAllDataProps) => {
    const { tidspunkt } = useContext(KartdataContext);
    const { produksjonsfilter } = useContext(ProduksjonsfilterContext);
    const { hastighetsfilter } = useContext(HastighetsfilterContext);
    const { øktfilter } = useContext(ØktfilterContext);
    const { tidsfilter } = useContext(TidsfilterContext);
    const { hentProduksjonsdetaljer } = useContext(ProduksjonsdetaljerCacheContext);
    const [loading, setLoading] = useState<boolean>(false);
    const { roder } = useContext(RoderContext);
    const informasjonsContext = useContext(InformasjonsmeldingContext);

    const hentMålepunkter = () => {
        const punkter = produksjonslenker?.data.flatMap(
            (produksjonslenke: Produksjonslenke) => produksjonslenke.målepunkter
        );

        if (punkter.length === 0) {
            const valgteFiltere = produksjonsfilter
                .filter((f) => f.valgt)
                .map((f) => f.navn)
                .toString();
            const loggInfo = `Ingen produksjonsdetaljer for kontrakt på valgt tidspunkt,\n tidspunkt: ${tidspunkt.verdi.toISOString()},\n valgteFiltere:${valgteFiltere}`;
            loggFrontend({
                loggNivå: LoggNivå.Warn,
                customInformasjon: loggInfo,
            });
        }

        return punkter;
    };

    const filtrerteProduksjonsdetaljer = (produksjonsdetaljer: Produksjonsdetaljer[]) =>
        filtrerProduksjonsdetaljer(
            produksjonsfilter,
            hastighetsfilter,
            øktfilter,
            tidsfilter,
            produksjonsdetaljer
        );
    const hentData = async (roder: RodeDto[] | undefined) => {
        setLoading(true);
        const målepunkter = hentMålepunkter();
        const produksjonsdetaljer = await hentProduksjonsdetaljer(målepunkter, tidspunkt.verdi);
        if (produksjonsdetaljer?.suksess) {
            if (produksjonsdetaljer.data.length === 0) {
                informasjonsContext.oppdater(informasjonsmeldinger.ingenDataIKartet);
            } else
                await downloadExcel(
                    produksjonsdetaljer.data,
                    filtrerteProduksjonsdetaljer(produksjonsdetaljer.data),
                    [],
                    roder
                );
        }
        setLoading(false);
    };

    return (
        <VisSkjulKomponent>
            <Innhold>
                <Tekst størrelse={"S"} noMargin={true}>
                    Du kan laste ned det du ser i kartet som regneark til Excel. NB! Det kan ta opp
                    mot 2 minutter.
                </Tekst>
                {loading ? (
                    <HenterData />
                ) : (
                    produksjonslenker?.suksess && (
                        <EksporterTilExcel
                            tekst={"Last ned"}
                            onClick={() => hentData(roder?.data)}
                        />
                    )
                )}
            </Innhold>
        </VisSkjulKomponent>
    );
};

interface VisSkjulProps {
    children: React.ReactNode;
}
const VisSkjulKomponent = ({ children }: VisSkjulProps) => {
    const [åpen, settÅpen] = useState<boolean>(false);
    return (
        <Container>
            <KnappUtenStyling onClick={() => settÅpen(!åpen)}>
                <Tekst noMargin={true} størrelse={"M"} bold={true}>
                    Last ned data til Excel?
                </Tekst>
                <Tekstpil størrelse={14} rotert={åpen} />
            </KnappUtenStyling>
            {åpen && children}
        </Container>
    );
};

const HenterData = () => (
    <HenterDataWrapper>
        <Spinner inline={true} storrelse={"mindre"} />
        <Tekst størrelse={"S"} noMargin={true}>
            Laster ned...
        </Tekst>
    </HenterDataWrapper>
);

const Innhold = styled.div`
    display: flex;
    flex-direction: column;
    gap: 8px;
`;

const Container = styled.div`
    margin-top: 0.5rem;
    display: flex;
    flex-direction: column;
`;

const HenterDataWrapper = styled.div`
    display: flex;
    flex-direction: row;
    justify-content: flex-start;
    align-items: center;
    gap: 4px;
    padding: 0;
`;
