import { RodeDto } from "../../api/useHentRoder";
import styled from "styled-components";
import React, { useEffect, useState } from "react";
import { getPopupType } from "../popup/popupUtils";
import { InformasjonsPopupType, PopupType } from "../Kartvisning/KartvisningGenerell";
import { Søkefelt } from "../../components/fellesKomponenter/søkefelt/Søkefelt";
import { Trekkspill } from "../../components/fellesKomponenter/trekkspill/Trekkspill";
import { ColumnWrapper, RowWrapper } from "../../components/fellesKomponenter/wrappers/wrappers";
import { FormLabel } from "../../components/fellesKomponenter/inputfelt/FormLabel";
import { HjelpetekstPopup } from "../../components/fellesKomponenter/HjelpetekstPopup";
import tekster from "../../tekster.json";
import { IkonWrapper } from "../../components/fellesKomponenter/inputfelt/Datofelt";

export interface RodeListeProps {
    roder: RodeDto[];
    settPopupInfo: (popupInfo?: PopupType) => void;
    popupInfo?: PopupType;
    settSelektertRodeFraListe: (rode?: RodeDto) => void;
    visRoder: boolean;
    håndterSettVisRoder: (visRoder: boolean) => void;
}
export const RodeListe = ({
    roder,
    settPopupInfo,
    popupInfo,
    settSelektertRodeFraListe,
    visRoder,
    håndterSettVisRoder,
}: RodeListeProps) => {
    const aktivRodePopupId =
        getPopupType(popupInfo) === InformasjonsPopupType.RODE
            ? (popupInfo as RodeDto).rodeId
            : undefined;

    const rodeNavnKlikket = (rode: RodeDto) => {
        settSelektertRodeFraListe(rode);
        settPopupInfo(rode);
        if (!visRoder) {
            håndterSettVisRoder(true);
        }
    };

    useEffect(() => {
        setÅpen(visRoder);
    }, [visRoder]);

    const [åpen, setÅpen] = useState(visRoder);
    const [søkeord, setSøkeord] = useState("");
    const filtrerteRoder =
        roder.filter((r) => r.rodeNavn.toLowerCase().includes(søkeord.toLowerCase())) ?? [];

    return (
        <Container>
            <ColumnWrapper>
                <StyledRowWrapper>
                    <FormLabel htmlFor={"roder"}>Velg rode</FormLabel>
                    <IkonWrapper>
                        <HjelpetekstPopup plassering={"right center"} størrelse={"S"}>
                            {tekster.velgRode}
                        </HjelpetekstPopup>
                    </IkonWrapper>
                </StyledRowWrapper>
                <Trekkspill
                    åpen={åpen}
                    tittel={"Roder"}
                    type={"Ramme"}
                    størrelse={"M"}
                    onClick={() => setÅpen(!åpen)}
                    width={"260px"}
                    makshøyde={"none"}
                    position={"imbedded"}
                >
                    <Innhold>
                        <Søkefelt søkeord={søkeord} setSøkeord={setSøkeord} />
                        {filtrerteRoder.map((rode, index) => (
                            <Rodenavn
                                key={index}
                                onClick={() => rodeNavnKlikket(rode)}
                                erAktiv={aktivRodePopupId === rode.rodeId}
                            >
                                {rode.rodeNavn}
                            </Rodenavn>
                        ))}
                    </Innhold>
                </Trekkspill>
            </ColumnWrapper>
        </Container>
    );
};

const Container = styled.div`
    align-items: flex-end;
    display: flex;
    max-width: max-content;
`;

const Innhold = styled.div`
    background-color: var(--white);
    padding: 0 0.5rem;
    margin-bottom: 1rem;
`;

const Rodenavn = styled.button<{ erAktiv: boolean }>`
    margin: 0 0 5px 0;
    font-size: 14px;
    border: 0;
    border-bottom: ${({ erAktiv }) => (erAktiv ? "2px solid var(--orange)" : "2px solid white")};
    font-weight: ${({ erAktiv }) => erAktiv && "800"};
    color: var(--gray-dark);
    display: flex;
    align-items: center;
    appearance: none;
    width: fit-content;
    justify-content: space-between;
    background-color: transparent;
    padding: 0;
    text-align: left;

    &:hover {
        font-weight: 600;
        cursor: pointer;
        border-bottom: ${({ erAktiv }) =>
            erAktiv ? "2px solid var(--orange)" : "2px solid var(--gray-dark-65)"};
    }
`;

const StyledRowWrapper = styled(RowWrapper)`
    gap: 0.5rem;
    align-items: flex-start;
`;
