import React, { useContext, useEffect } from "react";
import { Tekst } from "../../components/fellesKomponenter/typografi/Tekst";
import { KartdataContext } from "../../api/context/KartdataContext";
import { settKartUrlParam } from "../utils/kartUrlUtils";
import { useRouter } from "next/router";
import { useKjoretoyProduksjonsdetaljerCache } from "../../api/context/KjoretoyProduksjonsdetaljerCacheContext";
import { useProduksjonsdetaljerCache } from "../../api/context/ProduksjonsdetaljerCacheContext";
import localStorageService from "../../utils/storage/localstorageService";
import { LocalStorageKey } from "../../utils/storage/nøkler";
import { useOktContext } from "../../api/context/ØktContext";
import { TrekkspillSøkbar } from "../../components/fellesKomponenter/trekkspill/TrekkspillSøkbar";
import { ProduksjonslenkerContext } from "../../api/context/ProduksjonslenkerContext";
import { InformasjonsmeldingContext } from "../../api/context/InformasjonsmeldingContext";
import {
    defaultProduksjonsfilter,
    ProduksjonsfilterContext,
} from "../../api/context/ProduksjonsfilterContext";

export const Kontraktvelger = () => {
    const { kontrakt, tidspunkt } = useContext(KartdataContext);
    const { produksjonslenker } = useContext(ProduksjonslenkerContext);
    const informasjonsContext = useContext(InformasjonsmeldingContext);
    const produksjonsfilter = useContext(ProduksjonsfilterContext);
    const kjoretoyProduksjonsdetaljerCache = useKjoretoyProduksjonsdetaljerCache();
    const produksjonsdetaljerCache = useProduksjonsdetaljerCache();
    const øktContext = useOktContext();
    const { pathname } = useRouter();

    useEffect(() => {
        kontrakt.aktiv &&
            settKartUrlParam(pathname, {
                elrappKontraktId: kontrakt.aktiv.elrappId,
                timestamp: tidspunkt.verdi,
            });
    }, [produksjonslenker]);
    const setAktivKontrakt = (kontraktString: string) => {
        const klikketKontrakt = kontrakt?.alle?.find(
            (_kontakt) => _kontakt.navn === kontraktString
        );
        if (klikketKontrakt) {
            kontrakt.oppdaterAktiv(klikketKontrakt);
            localStorageService.lagreILocalStorage(
                LocalStorageKey.LOCALSTORAGE_AKTIV_KONTRAKT,
                klikketKontrakt
            );
            kjoretoyProduksjonsdetaljerCache.tømCache();
            produksjonsdetaljerCache.tømCache();
            øktContext.tømCache();
            øktContext.oppdaterFilter({ verdi: "", valgt: false });
            informasjonsContext.fjern();
            produksjonsfilter.oppdaterFilter(defaultProduksjonsfilter);
        }
    };

    return (kontrakt?.alle?.length ?? 0) > 1 ? (
        <TrekkspillSøkbar
            utvalg={kontrakt.alle!.map((kontrakt) => kontrakt.navn).sort()}
            håndterValg={setAktivKontrakt}
            tittel={kontrakt.aktiv?.navn ?? "Ingen valgt kontrakt"}
        />
    ) : (
        <Tekst>{kontrakt.aktiv?.navn ?? "Ingen valgt kontrakt"}</Tekst>
    );
};
