import { KameraMarker } from "./KameraMarker";
import { MarkerClusterGroupWrapper } from "./MarkerClusterGroup";
import React from "react";
import styles from "./Cluster.module.css";
import { divIcon, MarkerCluster } from "leaflet";
import { WebkameraDto } from "../../api/useHentWebkamera";
import { harValgtMarker } from "./markerUtils";
import cn from "classnames";
import { markerWidth } from "../../utils/constants";

interface KameraCluster {
    kameradata: WebkameraDto[];
    kameraPopupInfo?: WebkameraDto;
    settKameraPopupInfo: (kamera?: WebkameraDto) => void;
}

export const KameraCluster = ({
    kameradata,
    kameraPopupInfo,
    settKameraPopupInfo,
}: KameraCluster) => {
    return (
        <MarkerClusterGroupWrapper
            maxClusterRadius={80}
            disableClusteringAtZoom={12}
            zoomToBoundsOnClick={true}
            iconCreateFunction={lagClusterIkon}
        >
            {kameradata.map((kamera, index) => (
                <KameraMarker
                    settKameraPopupInfo={settKameraPopupInfo}
                    kamera={kamera}
                    key={index}
                    kameraPopupInfo={kameraPopupInfo}
                />
            ))}
        </MarkerClusterGroupWrapper>
    );
};

const lagClusterIkon = (cluster: MarkerCluster) => {
    const ikonUrl = harValgtMarker(cluster)
        ? "/ikon/kart/varkamera-gruppe-valgt.svg"
        : "/ikon/kart/varkamera-gruppe.svg";
    const imageHtml = `<img src=${ikonUrl} alt="Vegkameragruppe"/>
            <span class="${cn(styles.clusterMarkerTekst, {
                [styles.clusterMarkerTekstSelected]: harValgtMarker(cluster),
            })}">
                ${cluster.getChildCount()}
            </span>
        </img>`;

    return divIcon({
        html: imageHtml,
        iconSize: [markerWidth, markerWidth],
        className: `cluster-marker ${styles.clusterMarker}`,
    });
};
