import { WebkameraDto } from "../../api/useHentWebkamera";
import { Marker } from "react-leaflet";
import React from "react";
import { KameraPopup } from "../popup/kamera/KameraPopup";
import { AnimatePresence } from "framer-motion";
import { normalIkon, valgtIkon } from "./markerUtils";

interface KameraMarkerProps {
    kamera: WebkameraDto;
    kameraPopupInfo?: WebkameraDto;
    settKameraPopupInfo: (kamera?: WebkameraDto) => void;
}

export const KameraMarker = ({
    kamera,
    kameraPopupInfo,
    settKameraPopupInfo,
}: KameraMarkerProps) => {
    const erValgt = kameraPopupInfo && kameraPopupInfo.id === kamera.id;
    const ikon = erValgt
        ? valgtIkon("/ikon/kart/varkamera-pin-valgt.svg")
        : normalIkon("/ikon/kart/varkamera-pin.svg");

    return (
        <Marker
            position={[kamera.breddegrad, kamera.lengdegrad]}
            icon={ikon}
            autoPan={true}
            eventHandlers={{
                click: () => {
                    settKameraPopupInfo(kamera);
                },
            }}
            keyboard={false}
        >
            <AnimatePresence>
                {erValgt && (
                    <KameraPopup kamera={kamera} onClose={() => settKameraPopupInfo(undefined)} />
                )}
            </AnimatePresence>
        </Marker>
    );
};
