import { MarkerClusterGroupWrapper } from "./MarkerClusterGroup";
import React from "react";
import styles from "./Cluster.module.css";
import { divIcon, MarkerCluster } from "leaflet";
import { SisteKjøretøyDto } from "../../api/useHentSisteKjøretøy";
import { KjøretøyMarker } from "./KjøretøyMarker";
import { harValgtMarker } from "./markerUtils";
import cn from "classnames";
import { markerWidth } from "../../utils/constants";

interface KjøretøyCluster {
    kjøretøyListe: SisteKjøretøyDto[];
    kjøretøyPopupInfo?: SisteKjøretøyDto;
    settKjøretøyPopupInfo: (kjøretøy?: SisteKjøretøyDto) => void;
}

export const KjøretøyCluster = ({
    kjøretøyListe,
    kjøretøyPopupInfo,
    settKjøretøyPopupInfo,
}: KjøretøyCluster) => {
    return (
        <MarkerClusterGroupWrapper
            maxClusterRadius={80}
            disableClusteringAtZoom={12}
            zoomToBoundsOnClick={true}
            iconCreateFunction={lagClusterIkon}
        >
            {kjøretøyListe.map((kjøretøy, index) => (
                <KjøretøyMarker
                    settKjøretøyPopupInfo={settKjøretøyPopupInfo}
                    kjøretøy={kjøretøy}
                    key={index}
                    kjøretøyPopupInfo={kjøretøyPopupInfo}
                />
            ))}
        </MarkerClusterGroupWrapper>
    );
};

const lagClusterIkon = (cluster: MarkerCluster) => {
    const ikonUrl = harValgtMarker(cluster)
        ? "/ikon/kart/broytebil-sirkel-valgt.svg"
        : "/ikon/kart/broytebil-sirkel.svg";
    const imageHtml = `<img src=${ikonUrl} alt="Brøytebilgruppe"/>
            <span class="${cn(styles.clusterMarkerTekst, {
                [styles.clusterMarkerTekstSelected]: harValgtMarker(cluster),
            })}">
                ${cluster.getChildCount()}
            </span>
        </img>`;

    return divIcon({
        html: imageHtml,
        iconSize: [markerWidth, markerWidth],
        className: `cluster-marker ${styles.clusterMarker}`,
    });
};
