import { SisteKjøretøyDto } from "../../api/useHentSisteKjøretøy";
import { Marker } from "react-leaflet";
import React from "react";
import { KjøretøyPopup } from "../popup/kjøretøy/KjøretøyPopup";
import { AnimatePresence } from "framer-motion";
import { normalIkon, valgtIkon } from "./markerUtils";

interface KjøretøyMarkerProps {
    kjøretøy: SisteKjøretøyDto;
    settKjøretøyPopupInfo: (kjøretøy?: SisteKjøretøyDto) => void;
    kjøretøyPopupInfo?: SisteKjøretøyDto;
}

export const KjøretøyMarker = ({
    kjøretøy,
    settKjøretøyPopupInfo,
    kjøretøyPopupInfo,
}: KjøretøyMarkerProps) => {
    const erValgt = kjøretøyPopupInfo && kjøretøyPopupInfo === kjøretøy;
    const ikon = erValgt
        ? valgtIkon("/ikon/kart/broytebil-pin-valgt.svg")
        : normalIkon("/ikon/kart/broytebil-pin.svg");

    return (
        <Marker
            position={[kjøretøy.lokasjon[0].coordinates[1], kjøretøy.lokasjon[0].coordinates[0]]}
            icon={ikon}
            autoPan={true}
            eventHandlers={{
                click: () => {
                    settKjøretøyPopupInfo(kjøretøy);
                },
            }}
            keyboard={false}
        >
            <AnimatePresence>
                {kjøretøyPopupInfo && kjøretøyPopupInfo === kjøretøy && (
                    <KjøretøyPopup
                        kjøretøy={kjøretøy}
                        onClose={() => settKjøretøyPopupInfo(undefined)}
                    />
                )}
            </AnimatePresence>
        </Marker>
    );
};
