import { Marker } from "react-leaflet";
import React from "react";
import { TrafikkmeldingerDto } from "../../api/useHentTrafikkmeldinger";
import { StengtVegPupup } from "../popup/stengtVeg/StengtVegPopup";
import { AnimatePresence } from "framer-motion";
import { normalIkon, valgtIkon } from "./markerUtils";

interface TrafikkmeldingMarker {
    trafikkmelding: TrafikkmeldingerDto;
    settTrafikkmeldingPopupInfo: (trafikkmelding?: TrafikkmeldingerDto) => void;
    trafikkmeldingPopupInfo?: TrafikkmeldingerDto;
}

export const TrafikkmeldingMarker = ({
    trafikkmelding,
    settTrafikkmeldingPopupInfo,
    trafikkmeldingPopupInfo,
}: TrafikkmeldingMarker) => {
    const erValgt = trafikkmeldingPopupInfo && trafikkmeldingPopupInfo === trafikkmelding;
    const ikon = finnTrafikkmeldingMarker(erValgt ?? false, trafikkmelding.roadClosed);

    return (
        <div>
            {trafikkmelding.locations[0].coordinatesForDisplay && (
                <Marker
                    position={[
                        trafikkmelding.locations[0].coordinatesForDisplay.coordinates[1],
                        trafikkmelding.locations[0].coordinatesForDisplay.coordinates[0],
                    ]}
                    icon={ikon}
                    autoPan={true}
                    eventHandlers={{
                        click: () => {
                            settTrafikkmeldingPopupInfo(trafikkmelding);
                        },
                    }}
                    keyboard={false}
                >
                    <AnimatePresence>
                        {trafikkmeldingPopupInfo && trafikkmeldingPopupInfo === trafikkmelding && (
                            <StengtVegPupup
                                trafikkmelding={trafikkmeldingPopupInfo}
                                onClose={() => settTrafikkmeldingPopupInfo(undefined)}
                            />
                        )}
                    </AnimatePresence>
                </Marker>
            )}
        </div>
    );
};

const finnTrafikkmeldingMarker = (erValgt: boolean, erStengtVei: boolean) =>
    erStengtVei
        ? erValgt
            ? valgtIkon("/ikon/kart/stengt-pin-valgt.svg")
            : normalIkon("/ikon/kart/stengt-pin.svg")
        : erValgt
        ? valgtIkon("/ikon/kart/alert-valgt.svg")
        : normalIkon("/ikon/kart/alert.svg");
