import React, { useContext, useRef } from "react";
import { Tekst } from "../../components/fellesKomponenter/typografi/Tekst";
import styled from "styled-components";
import { Lukknapp } from "../../components/fellesKomponenter/knapper/Lukknapp";
import { IkonBase } from "../../components/Ikon/IkonBase";
import { Heading } from "../../components/fellesKomponenter/typografi/Heading";
import { useDisableClickPropagation } from "../../hooks/useDisableClickPropagation";
import { useDisableScrollPropagtaion } from "../../hooks/useDisableScrollPropagtaion";
import { motion } from "framer-motion";
import { useWindowSize } from "../../hooks/useWindowSize";
import { MenyÅpenContext } from "../../api/context/MenyÅpenContext";
import { useElementSize } from "../../hooks/useElementSize";

interface InformasjonPopupProps {
    tittel: string;
    undertekst?: string;
    undertekstHtml?: React.ReactNode;
    ikonPath: string;
    detaljWidth: number;
    children: React.ReactNode;
    ikonBorder?: number;
    onClose?: () => void;
    skalHaDetaljMinHeight?: boolean;
}

export const InformasjonPopup = ({
    tittel,
    undertekst,
    undertekstHtml,
    ikonPath,
    detaljWidth,
    children,
    ikonBorder = undefined,
    onClose = undefined,
    skalHaDetaljMinHeight = false,
}: InformasjonPopupProps) => {
    const { menyÅpen } = useContext(MenyÅpenContext);
    const popupRef = useRef(null);
    useDisableClickPropagation(popupRef);
    useDisableScrollPropagtaion(popupRef);
    const { windowHeight } = useWindowSize();
    const elementRef = useRef<HTMLDivElement>(null);
    const popupHeaderHeight = useElementSize(elementRef).height;
    const detaljHeight = windowHeight - popupHeaderHeight - 30;

    return (
        <InformasjonPopupWrapper
            initial={{
                opacity: 1,
                scale: 0,
                x: 0,
            }}
            animate={{
                opacity: 1,
                scale: 1,
                transition: {
                    duration: 0.3,
                },
                x: menyÅpen ? `calc(var(--meny-width, 0px) + 42px)` : 42,
                y: 0,
            }}
            exit={{
                opacity: 0,
                scale: 0.75,
                transition: {
                    ease: "easeOut",
                    duration: 0.3,
                },
            }}
            ref={popupRef}
        >
            <StyledArticle>
                <PopupHeader ref={elementRef}>
                    <PopupTittelgruppe>
                        <IkonBase path={ikonPath} border={ikonBorder} />
                        <div>
                            <PopupOverskrift as="h1" styleLevel="4">
                                {tittel}
                            </PopupOverskrift>
                            {undertekst && (
                                <PopupUndertext størrelse="S">{undertekst}</PopupUndertext>
                            )}
                            {undertekstHtml && undertekstHtml}
                        </div>
                    </PopupTittelgruppe>
                    <Lukknapp
                        onClick={() => {
                            onClose && onClose();
                        }}
                    >
                        Lukk
                    </Lukknapp>
                </PopupHeader>
                <DetaljWrapper
                    height={detaljHeight}
                    width={detaljWidth}
                    skalHaDetaljMinHeight={skalHaDetaljMinHeight}
                    className="leaflet-popup-content"
                >
                    {children}
                </DetaljWrapper>
            </StyledArticle>
        </InformasjonPopupWrapper>
    );
};

const InformasjonPopupWrapper = styled(motion.div)`
    z-index: 1050;
    position: fixed;
    top: calc(var(--header-height, 0px) + 10px);
    background-color: var(--white);
    box-shadow: rgba(0, 0, 0, 0.25) 0 2px 4px 0;
    cursor: auto;
    max-width: calc(100vw - 56px);

    .leaflet-popup-content p {
        margin: 0;
    }
`;

const StyledArticle = styled.article`
    max-height: inherit;
`;

interface detaljWrapperProps {
    width: number;
    height: number;
    skalHaDetaljMinHeight: boolean;
}

const DetaljWrapper = styled.section<detaljWrapperProps>`
    width: ${({ width }) => width}px;
    ${({ skalHaDetaljMinHeight, height }) =>
        skalHaDetaljMinHeight && `height: calc( ${height}px - var(--header-height)  );`}
    max-height: calc(${({ height }) => height}px - var(--header-height));
    min-width: ${({ width }) => width}px;
    max-width: 100%;
    margin: 0;
    overflow: auto;
    resize: both;
`;

const PopupHeader = styled.div`
    display: flex;
    background: var(--blue-bg);
    align-items: flex-start;
    padding: 16px;
    justify-content: space-between;
`;

const PopupTittelgruppe = styled.div`
    display: flex;
    gap: 16px;
    margin-right: 5px;
`;

const PopupUndertext = styled(Tekst)`
    margin: 0;
`;

const PopupOverskrift = styled(Heading)`
    max-width: 550px;
`;
