import { WEBKAMERA_URL } from "../../../utils/baseUrl";
import React, { useContext } from "react";
import { KartdataContext } from "../../../api/context/KartdataContext";
import { Tekst } from "../../../components/fellesKomponenter/typografi/Tekst";
import { sideInfo } from "../../../sider";
import { Strek } from "../../../components/fellesKomponenter/Strek";
import { erIDag } from "../../../utils/tid";
import styled from "styled-components";
import { KameraLenke } from "./KameraLenke";
import { EksternLenke } from "../../../components/fellesKomponenter/lenker/EksternLenke";

interface KameraPanelProps {
    id: number;
    stedsnavn: string;
    veg: string;
    info: string;
    værvarsel: string;
}

export const KameraPanel = ({ id, stedsnavn, veg, værvarsel }: KameraPanelProps) => {
    const stortBildeUrl = (kameraId: number) => `${WEBKAMERA_URL}/public/kamera?id=${kameraId}`;
    const { tidspunkt } = useContext(KartdataContext);
    return (
        <KameraWrapper>
            <Vegkamera src={stortBildeUrl(id)} alt={`Vegkamera ${veg} ${stedsnavn}`} />
            <Kamerainfo>
                <Kamerainfogruppe>
                    <Tekst størrelse="S">Alle vegkameraene samlet på en side</Tekst>
                    <KameraLenke href={sideInfo.webkamera.href} />
                </Kamerainfogruppe>
                <Strek />
                <Kamerainfogruppe>
                    <Tekst størrelse="S">Vegvær har tidligere kamerabilder</Tekst>
                    <EksternLenke størrelse="M" href="https://vegvar.atlas.vegvesen.no/">
                        Åpne Vegvær
                    </EksternLenke>
                </Kamerainfogruppe>
                <Strek />
                <Kamerainfogruppe>
                    <Tekst størrelse="S">
                        {erIDag(tidspunkt.verdi)
                            ? "Værmelding er hentet fra yr.no"
                            : "Vi har ikke historisk værdata"}
                    </Tekst>
                    <EksternLenke størrelse="M" href={værvarsel}>
                        Åpne værstasjon i Yr
                    </EksternLenke>
                </Kamerainfogruppe>
            </Kamerainfo>
        </KameraWrapper>
    );
};

const Vegkamera = styled.img`
    width: 55%;
    min-height: 276px;
`;

const KameraWrapper = styled.section`
    display: flex;
    gap: 16px;
    flex-wrap: wrap;
    overflow: hidden;
`;

const Kamerainfo = styled.section``;

const Kamerainfogruppe = styled.section`
    display: flex;
    flex-direction: column;
    gap: 6px;
    margin-bottom: 16px;
`;
