import React, { useContext } from "react";
import { WebkameraDto } from "../../../api/useHentWebkamera";
import { InformasjonPopup } from "../InformasjonPopup";
import styled from "styled-components";
import { KartdataContext } from "../../../api/context/KartdataContext";
import { erIDag } from "../../../utils/tid";
import { KameraPanel } from "./KameraPanel";
import { Værpanel } from "./Værpanel";

export interface KameraPopupProps {
    kamera: WebkameraDto;
    kameraPopupInfo?: WebkameraDto;
    onClose?: () => void;
}

export const KameraPopup = ({ kamera, onClose }: KameraPopupProps) => {
    const { tidspunkt } = useContext(KartdataContext);

    return (
        <InformasjonPopup
            tittel={`${kamera.veg} ${kamera.stedsnavn}`}
            undertekst={kamera.info}
            detaljWidth={696}
            ikonPath="/ikon/kart/varkamera-gruppe.svg"
            ikonBorder={2}
            onClose={onClose}
        >
            <PopupInnhold>
                <KameraPanel
                    id={kamera.id}
                    stedsnavn={kamera.stedsnavn}
                    veg={kamera.veg}
                    info={kamera.info}
                    værvarsel={kamera.vaervarsel}
                />
                {erIDag(tidspunkt.verdi) && <Værpanel værvarsel={kamera.vaervarsel} />}
            </PopupInnhold>
        </InformasjonPopup>
    );
};

const PopupInnhold = styled.div`
    padding: 16px;
    margin: 0;
`;
