import React from "react";
import { RodeDto } from "../../../api/useHentRoder";

export enum Vinterdriftsklasse {
    DkA = "DkA",
    DkB = "DkB",
    DkBl = "DkBl",
    DkBm = "DkBm",
    DkBh = "DkBh",
    DkC = "DkC",
    DkD = "DkD",
    DkE = "DkE",
    Sideanlegg = "Sideanlegg",
}

export interface KlasseMedSyklus {
    klasse: Vinterdriftsklasse;
    syklus: string;
    godkjentFøreforhold: string;
}

export const hentKlasserMedSyklus = (vinterdriftsKlasser: string[] | undefined) => {
    const renamedKlasser: string[] | undefined = vinterdriftsKlasser?.map((k) =>
        k === "si" ? "Sideanlegg" : k
    );

    const klasser: KlasseMedSyklus[] = (renamedKlasser ?? []).reduce(
        (aggregerteKlasser: KlasseMedSyklus[], k: string) => {
            if (
                !aggregerteKlasser.some((klasse) => klasse.klasse === k) &&
                !erKlasseGangOgSykkelvei(k)
            ) {
                aggregerteKlasser.push({
                    klasse: k as Vinterdriftsklasse,
                    syklus: hentSyklusFraKlasser(k as Vinterdriftsklasse),
                    godkjentFøreforhold: hentGodkjentFørefolhold(k as Vinterdriftsklasse),
                });
            }
            return aggregerteKlasser;
        },
        []
    );
    return klasser;
};

export const finnDriftsklasseMedHøyesteSyklustid = (roderPåVeglenken: RodeDto[]) => {
    return roderPåVeglenken.reduce(
        (driftsklasseMedHøyestSyklustid: Vinterdriftsklasse | undefined, rode) => {
            const driftsklasseMedHøyestSyklustidPåRode =
                finnDriftsklasseMedHøyesteSyklustidPåRode(rode);
            if (driftsklasseMedHøyestSyklustidPåRode === undefined) {
                return driftsklasseMedHøyestSyklustid;
            }
            if (
                driftsklasseMedHøyestSyklustid === undefined ||
                hentSyklusTidFraKlasser(driftsklasseMedHøyestSyklustidPåRode) >
                    hentSyklusTidFraKlasser(driftsklasseMedHøyestSyklustid)
            ) {
                return driftsklasseMedHøyestSyklustidPåRode;
            }
            return driftsklasseMedHøyestSyklustid;
        },
        undefined
    );
};
const finnDriftsklasseMedHøyesteSyklustidPåRode = (
    rode: RodeDto
): Vinterdriftsklasse | undefined => {
    return rode.vinterdriftsKlasser?.reduce(
        (driftsklasseMedHøyestSyklustid: Vinterdriftsklasse | undefined, klasse) => {
            if (
                (driftsklasseMedHøyestSyklustid === undefined ||
                    hentSyklusTidFraKlasser(klasse as Vinterdriftsklasse) >
                        hentSyklusTidFraKlasser(driftsklasseMedHøyestSyklustid)) &&
                klasse !== "si"
            ) {
                return klasse as Vinterdriftsklasse;
            }
            return driftsklasseMedHøyestSyklustid;
        },
        undefined
    );
};

const hentSyklusFraKlasser = (klasse: Vinterdriftsklasse) => {
    switch (klasse) {
        case Vinterdriftsklasse.DkA:
            return "1,5 timer.";
        case Vinterdriftsklasse.DkB:
        case Vinterdriftsklasse.DkBl:
        case Vinterdriftsklasse.DkBm:
        case Vinterdriftsklasse.DkBh:
            return "2 timer.";
        case Vinterdriftsklasse.DkC:
            return "3 timer.";
        case Vinterdriftsklasse.DkD:
            return "3 timer for brøyting, 4 timer for strøing.";
        case Vinterdriftsklasse.DkE:
            return "3 timer for brøyting, 4 timer for strøing.";
        case Vinterdriftsklasse.Sideanlegg:
            return "som for tilliggende veg, men ikke større enn 2 timer.";
        default:
            return "ikke funnet, ukjent driftsklasse.";
    }
};

export const hentSyklusTidFraKlasser = (klasse: Vinterdriftsklasse): number => {
    switch (klasse) {
        case Vinterdriftsklasse.DkA:
            return 1.5;
        case Vinterdriftsklasse.DkB:
        case Vinterdriftsklasse.DkBl:
        case Vinterdriftsklasse.DkBm:
        case Vinterdriftsklasse.DkBh:
        case Vinterdriftsklasse.Sideanlegg:
            return 2;
        case Vinterdriftsklasse.DkC:
            return 3;
        case Vinterdriftsklasse.DkD:
        case Vinterdriftsklasse.DkE:
            return 4;
        default:
            return 0;
    }
};

const hentGodkjentFørefolhold = (klasse: Vinterdriftsklasse) => {
    switch (klasse) {
        case Vinterdriftsklasse.DkA:
            return "bar vei (tørr eller våt). Salt brukes som preventivt tiltak og for å opprettholde og gjenopprette bar vei.";
        case Vinterdriftsklasse.DkB:
        case Vinterdriftsklasse.DkBl:
        case Vinterdriftsklasse.DkBm:
        case Vinterdriftsklasse.DkBh:
            return "bar veg (tørr eller våt), hard snø/is tillates utenom hjulspor i begrenset tidsrom. Salt brukes som preventivt tiltak og for å opprettholde og gjenopprette bar vei.";
        case Vinterdriftsklasse.DkC:
            return "bar vei (tørr eller våt) i milde perioder og hard snø/is i kalde perioder. Krav til friksjon på hard snø/is er større enn 0,25. I milde perioder brukes salt, mens det i kalde perioder med snø/isdekke brukes sand.";
        case Vinterdriftsklasse.DkD:
            return (
                "hard snø/is. Krav til friksjon er større enn 0,25. " +
                "Sand brukes for å oppnå godkjent friksjon på snø/isdekke. " +
                "Salt kan brukes for å forhindre tynn is og rim."
            );
        case Vinterdriftsklasse.DkE:
            return (
                "hard snø/is. Krav til friksjon er større enn 0,20. " +
                "DkE skal ikke nyttes på riksvei. " +
                "Sand brukes for å oppnå godkjent friksjon på snø/isdekke. " +
                "Salt kan brukes for å forhindre tynn is og rim."
            );
        case Vinterdriftsklasse.Sideanlegg:
            return "hardt og jevnt snø/is-dekke med maksimalt 2 cm løs snø eller etter spesiell beskrivelse";
        default:
            return "ikke funnet for ukjent driftsklasse";
    }
};

const erKlasseGangOgSykkelvei = (klasse: String) => {
    return klasse.startsWith("Gs");
};
