import styled from "styled-components";
import React, { useState } from "react";
import { ProduksjonsdetaljRad } from "./ProduksjonsdetaljRad";
import { Tekst } from "../../../../components/fellesKomponenter/typografi/Tekst";
import { Produksjonsdetaljer } from "../../../types/produksjonsdetaljer";
import { sorterProduksjonsdetaljer } from "./produksjonsdetaljerSorteringUtils";
import { ProduksjonsdetaljerHeader } from "./ProduksjonsdetaljerHeader/ProduksjonsdetaljerHeader";
import {
    ProduksjonsdetaljerKolonne,
    ProduksjonsdetaljerKolonneNavn,
} from "../kolonneVelger/ProduksjonsdetaljerKolonneVelger";

export type Sortering = {
    sorterEtter: SorterEtter;
    stigende: boolean;
};

export enum SorterEtter {
    datoLogget = "datoLogget",
    eksternReferanse = "eksternReferanse",
    kjøretøyId = "kjøretøyId",
    hastighetKmH = "hastighetKmHAvg",
    materiale = "materiale",
    erPlogNed = "erPlogNede",
    sideplog = "erSideplogIBruk",
    tungHøvel = "erHøvelskjærNede",
    underliggendeSkjær = "erUnderliggendeSkjærNede",
    sprederbredde = "sprederbredde",
    dosering = "dosering",
    vegsystemreferansekortform = "vegsystemreferansekortform",
}

interface ProduksjonsdetaljerTabellProps {
    filtrerteProduksjonsdetaljer: Produksjonsdetaljer[];
    kolonner: ProduksjonsdetaljerKolonne[];
    visSyklusFeil: boolean;
}

export const ProduksjonsdetaljerTabell = ({
    filtrerteProduksjonsdetaljer,
    kolonner,
    visSyklusFeil,
}: ProduksjonsdetaljerTabellProps) => {
    const [sortering, setSortering] = useState<Sortering>({
        sorterEtter: SorterEtter.datoLogget,
        stigende: true,
    });

    const håndterSortering = (sorteringsNøkkel: SorterEtter) => {
        setSortering({
            sorterEtter: sorteringsNøkkel,
            stigende: sorteringsNøkkel === sortering.sorterEtter ? !sortering.stigende : true,
        });
    };

    const finnSorteringsretning = (sorteringsnøkkel: SorterEtter): boolean | undefined =>
        sortering.sorterEtter === sorteringsnøkkel ? sortering.stigende : undefined;

    if (visSyklusFeil) {
        return <Tekst style={{ padding: "16px" }}>Klarte ikke å filtrere for syklustid</Tekst>;
    }
    if (filtrerteProduksjonsdetaljer.length === 0) {
        return (
            <Tekst style={{ padding: "16px" }}>
                Det er ingen produksjonsdetaljer for valgte filtre
            </Tekst>
        );
    }

    const sorterteProduksjonsdetaljer = sorterProduksjonsdetaljer(
        filtrerteProduksjonsdetaljer,
        sortering
    );

    return (
        <StyledTabell>
            <ProduksjonsdetaljerHeader
                finnSorteringsretning={finnSorteringsretning}
                håndterSortering={håndterSortering}
                kolonner={kolonner}
            />
            <tbody>
                {sorterteProduksjonsdetaljer.map((rad, index) => (
                    <ProduksjonsdetaljRad produksjonsdetalj={rad} kolonner={kolonner} key={index} />
                ))}
            </tbody>
        </StyledTabell>
    );
};

const StyledTabell = styled.table`
    display: flex;
    padding: 0 16px;
    flex-direction: column;
    border-collapse: collapse;
    height: fit-content;
`;

export const hentKolonneBredde = (kolonneNavn: ProduksjonsdetaljerKolonneNavn): number => {
    switch (kolonneNavn) {
        case ProduksjonsdetaljerKolonneNavn.Økt:
            return 60;
        case ProduksjonsdetaljerKolonneNavn.Tid:
            return 86;
        case ProduksjonsdetaljerKolonneNavn.Retning:
            return 86;
        case ProduksjonsdetaljerKolonneNavn.Kjøretøy:
            return 90;
        case ProduksjonsdetaljerKolonneNavn.Fart:
            return 155;
        case ProduksjonsdetaljerKolonneNavn.Materiale:
            return 100;
        case ProduksjonsdetaljerKolonneNavn.Plog:
            return 60;
        case ProduksjonsdetaljerKolonneNavn.Sideplog:
            return 100;
        case ProduksjonsdetaljerKolonneNavn.TungHøvel:
            return 120;
        case ProduksjonsdetaljerKolonneNavn.UnderliggendeSkjær:
            return 190;
        case ProduksjonsdetaljerKolonneNavn.Dosering:
            return 120;
        case ProduksjonsdetaljerKolonneNavn.Sprederbredde:
            return 150;
        case ProduksjonsdetaljerKolonneNavn.Vegsystemreferansekortform:
            return 190;
        default:
            return 24;
    }
};
