import styled from "styled-components";
import { Tekst } from "../../../components/fellesKomponenter/typografi/Tekst";
import { Boldtext } from "../../../components/fellesKomponenter/typografi/Boldtext";
import {
    ProduksjonsdetaljerTabell,
    SorterEtter,
} from "./ProduksjonsdetaljerTabell/ProduksjonsdetaljerTabell";
import { Produksjonsdetaljer } from "../../types/produksjonsdetaljer";
import { ProduksjonsdetaljerKolonneDropdown } from "./kolonneVelger/ProduksjonsdetaljerKolonneDropdown";
import React, { useContext, useEffect, useState } from "react";
import {
    ProduksjonsdetaljerKolonne,
    ProduksjonsdetaljerKolonneNavn,
} from "./kolonneVelger/ProduksjonsdetaljerKolonneVelger";
import localStorageService from "../../../utils/storage/localstorageService";
import { LocalStorageKey } from "../../../utils/storage/nøkler";
import {
    erAktiveKolonnerGyldige,
    genererMaksSyklustidStreng,
    manglerGyldigRodeData,
} from "./VeglenkeDetaljerUtils";
import { downloadExcel, EksporterTilExcel } from "../../../utils/EksporterTilExcel";
import { RoderContext } from "../../../api/context/RodeContext";
import { Checkbox } from "../../../components/fellesKomponenter/checkboxer/Checkbox";
import {
    erProduksjonInnenSyklustid,
    filtrerProduksjonsdetaljer,
} from "../../filtre/filtrerProduksjon/filtrerProduksjon";
import { RodeDto } from "../../../api/useHentRoder";
import { hentKlasserMedSyklus } from "../roder/vinterdriftsklasserUtils";
import { VinterdriftklasserInfo } from "../roder/VinterdriftklasserInfo";
import { ColumnWrapper } from "../../../components/fellesKomponenter/wrappers/wrappers";
import { ProduksjonsfilterContext } from "../../../api/context/ProduksjonsfilterContext";
import { HastighetsfilterContext } from "../../../api/context/HastighetsfilterContext";
import { ØktfilterContext } from "../../../api/context/ØktContext";
import { TidsfilterContext } from "../../../api/context/TidsfilterContext";

interface VeglenkedetaljerProps {
    timerOgMinutterSiden?: string;
    produksjonsdetaljer: Produksjonsdetaljer[];
    roderPåVeglenken: RodeDto[];
}

export const defaultAktiveKolonner: ProduksjonsdetaljerKolonne[] = [
    {
        navn: ProduksjonsdetaljerKolonneNavn.Økt,
        iBruk: true,
        sorteringsnøkkel: SorterEtter.eksternReferanse,
    },
    {
        navn: ProduksjonsdetaljerKolonneNavn.Tid,
        iBruk: true,
        sorteringsnøkkel: SorterEtter.datoLogget,
    },
    {
        navn: ProduksjonsdetaljerKolonneNavn.Retning,
        iBruk: false,
        sorteringsnøkkel: SorterEtter.datoLogget,
    },
    {
        navn: ProduksjonsdetaljerKolonneNavn.Kjøretøy,
        iBruk: true,
        sorteringsnøkkel: SorterEtter.kjøretøyId,
    },
    {
        navn: ProduksjonsdetaljerKolonneNavn.Fart,
        iBruk: true,
        sorteringsnøkkel: SorterEtter.hastighetKmH,
    },
    {
        navn: ProduksjonsdetaljerKolonneNavn.Materiale,
        iBruk: true,
        sorteringsnøkkel: SorterEtter.materiale,
    },
    {
        navn: ProduksjonsdetaljerKolonneNavn.Plog,
        iBruk: true,
        sorteringsnøkkel: SorterEtter.erPlogNed,
    },
    {
        navn: ProduksjonsdetaljerKolonneNavn.Sideplog,
        iBruk: false,
        sorteringsnøkkel: SorterEtter.sideplog,
    },
    {
        navn: ProduksjonsdetaljerKolonneNavn.TungHøvel,
        iBruk: false,
        sorteringsnøkkel: SorterEtter.tungHøvel,
    },
    {
        navn: ProduksjonsdetaljerKolonneNavn.UnderliggendeSkjær,
        iBruk: false,
        sorteringsnøkkel: SorterEtter.underliggendeSkjær,
    },
    {
        navn: ProduksjonsdetaljerKolonneNavn.Sprederbredde,
        iBruk: false,
        sorteringsnøkkel: SorterEtter.sprederbredde,
    },
    {
        navn: ProduksjonsdetaljerKolonneNavn.Dosering,
        iBruk: false,
        sorteringsnøkkel: SorterEtter.dosering,
    },
    {
        navn: ProduksjonsdetaljerKolonneNavn.Vegsystemreferansekortform,
        iBruk: false,
        sorteringsnøkkel: SorterEtter.vegsystemreferansekortform,
    },
];

export const Veglenkedetaljer = ({
    timerOgMinutterSiden,
    produksjonsdetaljer,
    roderPåVeglenken,
}: VeglenkedetaljerProps) => {
    const { roder } = useContext(RoderContext);
    const { produksjonsfilter } = useContext(ProduksjonsfilterContext);
    const { hastighetsfilter } = useContext(HastighetsfilterContext);
    const { øktfilter } = useContext(ØktfilterContext);
    const { tidsfilter } = useContext(TidsfilterContext);
    const [aktiveKolonner, setAktiveKolonner] =
        useState<ProduksjonsdetaljerKolonne[]>(defaultAktiveKolonner);
    const [visSyklustid, setVisSyklustid] = useState<boolean>(false);
    const [filtrerteProduksjonsdetaljerMedSyklustid, setFiltrerteProduksjonsdetaljerMedSyklustid] =
        useState(produksjonsdetaljer);
    const [visSyklusFeil, setVisSyklusFeil] = useState<boolean>(false);

    useEffect(() => {
        const kolonnerFraLocalStorage = localStorageService.hentFraLocalStorage(
            LocalStorageKey.LOCALSTORAGE_AKTIVE_KOLLONER,
            defaultAktiveKolonner
        );
        setAktiveKolonner(
            erAktiveKolonnerGyldige(kolonnerFraLocalStorage)
                ? kolonnerFraLocalStorage
                : defaultAktiveKolonner
        );
    }, []);

    const filtrerProduksjonsdetaljerMedSyklustid = () => {
        if (manglerGyldigRodeData(roderPåVeglenken, roder)) {
            setVisSyklusFeil(true);
            return;
        }
        const filtrertData = filtrerteProduksjonsdetaljer(produksjonsdetaljer).filter(
            (produksjonsdetalj: Produksjonsdetaljer) =>
                erProduksjonInnenSyklustid(produksjonsdetalj, roderPåVeglenken)
        );
        setFiltrerteProduksjonsdetaljerMedSyklustid(filtrertData);
    };

    useEffect(() => {
        if (visSyklustid) {
            filtrerProduksjonsdetaljerMedSyklustid();
        } else {
            setVisSyklusFeil(false);
        }
    }, [visSyklustid, produksjonsdetaljer]);

    const filtrerteProduksjonsdetaljer = (produksjonsdetaljer: Produksjonsdetaljer[]) =>
        filtrerProduksjonsdetaljer(
            produksjonsfilter,
            hastighetsfilter,
            øktfilter,
            tidsfilter,
            produksjonsdetaljer
        );

    const filtrerteProduksjonsdetaljerSyklus = visSyklustid
        ? filtrerteProduksjonsdetaljerMedSyklustid
        : filtrerteProduksjonsdetaljer(produksjonsdetaljer);
    const toggleAktiveKolonner = (kolonneNavn: ProduksjonsdetaljerKolonneNavn): void => {
        const nyeAktiveKolonner = aktiveKolonner.map((kolonne) =>
            kolonne.navn === kolonneNavn ? { ...kolonne, iBruk: !kolonne.iBruk } : kolonne
        );
        localStorageService.lagreILocalStorage(
            LocalStorageKey.LOCALSTORAGE_AKTIVE_KOLLONER,
            nyeAktiveKolonner
        );
        setAktiveKolonner(nyeAktiveKolonner);
    };

    const nullstillAktiveKolonner = () => {
        localStorageService.lagreILocalStorage(
            LocalStorageKey.LOCALSTORAGE_AKTIVE_KOLLONER,
            defaultAktiveKolonner
        );
        setAktiveKolonner(defaultAktiveKolonner);
    };

    const erForrigePasseringNegativ = (timerOgMinutterSiden: string) =>
        /(?<=-)\d+/.test(timerOgMinutterSiden);

    return (
        <>
            <EksportWrapper>
                <ColumnWrapper>
                    <CheckboxWrapper>
                        <Checkbox
                            størrelse={"S"}
                            key="syklustidcheck"
                            onChange={() => setVisSyklustid(!visSyklustid)}
                            checked={visSyklustid}
                            value={"Syklustidvisning"}
                            tekstStørrelse={"M"}
                        >
                            Vis kun produksjon innenfor syklustiden
                        </Checkbox>
                        <VinterdriftklasserInfo
                            klasserMedSyklus={hentKlasserMedSyklus(
                                roderPåVeglenken
                                    .filter((rode) => rode.vinterdriftsKlasser !== undefined)
                                    .flatMap((rode: RodeDto) => rode.vinterdriftsKlasser!)
                            )}
                        />
                    </CheckboxWrapper>
                    {visSyklustid && (
                        <Tekst størrelse={"S"}>
                            {genererMaksSyklustidStreng(roderPåVeglenken)}
                        </Tekst>
                    )}
                </ColumnWrapper>

                <EksporterTilExcel
                    tekst={"Last ned som regneark"}
                    onClick={() =>
                        downloadExcel(
                            produksjonsdetaljer,
                            filtrerteProduksjonsdetaljerSyklus,
                            aktiveKolonner,
                            roder?.data
                        )
                    }
                />
            </EksportWrapper>
            <ProduksjonsdetaljerHeader>
                <TidSidenWrapper>
                    {timerOgMinutterSiden && !erForrigePasseringNegativ(timerOgMinutterSiden) && (
                        <>
                            <Tekst størrelse="S">Tid siden forrige passering: </Tekst>
                            <Boldtext størrelse="M">{timerOgMinutterSiden}</Boldtext>
                        </>
                    )}
                </TidSidenWrapper>
                <ProduksjonsdetaljerKolonneDropdown
                    kolonner={aktiveKolonner}
                    oppdaterProduksjonsdetaljerKolonne={toggleAktiveKolonner}
                    nullstillAktiveKolonner={nullstillAktiveKolonner}
                />
            </ProduksjonsdetaljerHeader>
            <ProduksjonsdetaljerTabell
                visSyklusFeil={visSyklusFeil}
                filtrerteProduksjonsdetaljer={filtrerteProduksjonsdetaljerSyklus}
                kolonner={aktiveKolonner}
            />
        </>
    );
};

const TidSidenWrapper = styled.div`
    padding: 0 16px;
    margin: 0;
    display: flex;
    gap: 8px;
    align-items: flex-end;
`;

const ProduksjonsdetaljerHeader = styled.div`
    display: flex;
    justify-content: space-between;
    margin-bottom: 4px;
    margin-top: 16px;
`;

const EksportWrapper = styled.div`
    margin: 1rem 16px 0 16px;
    display: flex;
    gap: 20px;
    justify-content: space-between;
    align-items: flex-start;
`;

const CheckboxWrapper = styled.div`
    display: flex;
    gap: 0.5rem;
`;
