import { render, screen } from "@testing-library/react";
import React from "react";
import { ApiRespons, FeilType } from "../../api/api";
import "@testing-library/jest-dom";
import { Produksjonslenke } from "../types/produksjonslenke";
import { KartContextValue, KartdataContext } from "../../api/context/KartdataContext";
import dayjs from "dayjs";
import { ProduksjonslenkerContext } from "../../api/context/ProduksjonslenkerContext";
import { RodeDto } from "../../api/useHentRoder";
import { SisteKjøretøyDto } from "../../api/useHentSisteKjøretøy";
import { ProduksjonskartInnhold } from "../kartInnhold/ProduksjonskartInnhold";
import { RoderContext } from "../../api/context/RodeContext";

const rodedata: ApiRespons<RodeDto[]> = {
    suksess: true,
    data: [],
};

const kjøretøyData: ApiRespons<SisteKjøretøyDto[]> = {
    suksess: true,
    data: [],
};
jest.mock("../../api/useHentWebkamera", () => ({
    useHentWebkamera: () => rodedata,
}));

jest.mock("../../api/useHentSisteKjøretøy", () => ({
    useHentSisteKjøretøy: () => kjøretøyData,
}));

describe("Kart", () => {
    it("skal vise spinner hvis data er undefined", async () => {
        render(<ProduksjonskartInnhold />);

        await screen.findByTestId("brøytebilspinner");

        expect(screen.getByTestId("brøytebilspinner"));
    });

    it("skal vise feilmelding hvis data har feil", async () => {
        const produksjonslenker: ApiRespons<Produksjonslenke[]> = {
            suksess: false,
            feilmelding: "Kunne ikke hente produksjonslenker",
            type: FeilType.UKJENT_FEIL,
        };

        const contextVerdi: KartContextValue = {
            tidspunkt: {
                verdi: dayjs(),
                oppdater: () => {},
            },
            kontrakt: {
                finnes: true,
                alle: undefined,
                aktiv: undefined,
                oppdaterAktiv: () => {},
            },
            brukerInfo: undefined,
            intro: {
                vis: false,
                oppdater: () => {},
                toggle: () => {},
            },
        };

        render(
            <KartdataContext.Provider value={contextVerdi}>
                <ProduksjonslenkerContext.Provider value={{ produksjonslenker: produksjonslenker }}>
                    <RoderContext.Provider value={{ roder: rodedata }}>
                        <ProduksjonskartInnhold />
                    </RoderContext.Provider>
                </ProduksjonslenkerContext.Provider>
            </KartdataContext.Provider>
        );

        await screen.findByRole("list");

        expect(screen.getByRole("list")).toHaveTextContent("Kunne ikke hente produksjonslenker");
    });
});
