import { Dayjs, fraEnkelTimestampStr, tilEnkelTimestampStr } from "../../utils/tid";
import dayjs from "dayjs";
import { loggFrontend, LoggNivå } from "../../utils/loggFrontend";
import { Kontrakt } from "../../api/context/KartdataContext";
import { sideInfo } from "../../sider";
import { LocalStorageKey } from "../../utils/storage/nøkler";
import localStorageService from "../../utils/storage/localstorageService";

export interface KartUrlData {
    elrappKontraktId: number;
    timestamp: Dayjs;
}

export enum KartUrlSearchParameter {
    elrappkontrakt = "elrappkontrakt",
    tidspunkt = "tidspunkt",
}

export const settKartUrlParam = (nåværendePath: string, kartUrlData: KartUrlData): void => {
    if (nåværendePath.includes(sideInfo.produksjonskart.href)) {
        const urlParametre = new URLSearchParams();
        urlParametre.set(
            KartUrlSearchParameter.elrappkontrakt,
            kartUrlData.elrappKontraktId.toString()
        );
        urlParametre.set(
            KartUrlSearchParameter.tidspunkt,
            tilEnkelTimestampStr(kartUrlData.timestamp)
        );
        history.replaceState("", "", "?" + urlParametre.toString());
    }
};

export const hentKartUrlParam = (): KartUrlData | undefined => {
    if (typeof window !== "undefined") {
        const queryParam = new URLSearchParams(window.location.search);
        const elrappkontraktParam = queryParam.get(KartUrlSearchParameter.elrappkontrakt);
        const tidspunktParam = queryParam.get(KartUrlSearchParameter.tidspunkt);
        try {
            const parsetElrappkontrakt = Number(elrappkontraktParam);
            const parsetIsoTimestamp = tidspunktParam
                ? fraEnkelTimestampStr(tidspunktParam)
                : undefined;

            return parsetIsoTimestamp?.isValid() && parsetElrappkontrakt
                ? {
                      elrappKontraktId: parsetElrappkontrakt,
                      timestamp: parsetIsoTimestamp,
                  }
                : undefined;
        } catch (e) {
            loggFrontend({
                loggNivå: LoggNivå.Warn,
                customInformasjon: `Klarte ikke parse queryparametre ${elrappkontraktParam} og ${tidspunktParam}: ${e}`,
            });
            return undefined;
        }
    }
};

export const finnInitTidspunkt = (): Dayjs => {
    const urlParam = hentKartUrlParam();
    const timestampFraUrl = urlParam?.timestamp;
    const nå = dayjs();
    return timestampFraUrl && !timestampFraUrl.isAfter(nå) ? timestampFraUrl : nå;
};

export const finnInitAktivKontrakt = (kontrakter: Kontrakt[] | undefined): Kontrakt | undefined => {
    if (kontrakter && kontrakter?.length > 0) {
        const urlParam = hentKartUrlParam();
        const kontraktFraUrl =
            urlParam &&
            kontrakter.find((kontrakt) => kontrakt.elrappId === urlParam.elrappKontraktId);
        const kontraktFraLocalstorage = localStorageService.hentFraLocalStorage(
            LocalStorageKey.LOCALSTORAGE_AKTIV_KONTRAKT,
            undefined
        );

        if (kontraktFraUrl) {
            return kontraktFraUrl;
        } else if (kontraktFraLocalstorage) {
            return kontraktFraLocalstorage;
        } else {
            return kontrakter[0];
        }
    }
};
