import React from "react";
import { Tekst } from "../components/fellesKomponenter/typografi/Tekst";
import { EksternLenke } from "../components/fellesKomponenter/lenker/EksternLenke";
import { Fylkesfilter } from "./fylkefilter/Fylkesfilter";
import { ValgteFylker } from "./fylkefilter/ValgteFylker";
import { KnappSekundær } from "../components/fellesKomponenter/knapper/KnappSekundær";
import { KameraStørrelse } from "./WebkameraSide";
import styled from "styled-components";
import { Trekkspill } from "../components/fellesKomponenter/trekkspill/Trekkspill";
import { useCloseOnOutsideClick } from "../hooks/useCloseOnOutsideClick";
import {
    fontsizeLargeScreen,
    fontsizeSmallScreen,
} from "../components/fellesKomponenter/typografi/TekstStyles";
import { breakpointNettbrett } from "../utils/constants";

interface VegkameraHeaderProps {
    valgteFylker: string[];
    skjulFylkesveier: boolean;
    settSkjulFylkesveier: (skjul: boolean) => void;
    håndterKameraStørrelseEndring: (størrelse: KameraStørrelse) => void;
    håndterFylkesValg: (fylke: string[]) => void;
}
export const WebcameraParametervalg = ({
    valgteFylker,
    settSkjulFylkesveier,
    skjulFylkesveier,
    håndterKameraStørrelseEndring,
    håndterFylkesValg,
}: VegkameraHeaderProps) => {
    const [trekkspillÅpen, settTrekkspillÅpen] = React.useState(false);
    const utvalg = [KameraStørrelse.Liten, KameraStørrelse.Medium, KameraStørrelse.Stor];

    const id = "webkameraknapp";
    useCloseOnOutsideClick(id, () => settTrekkspillÅpen(false));

    return (
        <HeaderBanner>
            <Header>
                <Tekst>
                    Kameraene på denne siden er hentet fra{" "}
                    <EksternLenke href="https://www.vegvesen.no/webkamera#/">
                        Webkameraløsningen
                    </EksternLenke>{" "}
                    og viser hvordan vegen ser ut nå. Hvis du ønsker å se bilder fra tidligere må du
                    finne stasjonen i{" "}
                    <EksternLenke href="http://www.vegvar.no">Vegvær</EksternLenke>
                </Tekst>
                <Meny>
                    <FylkeWrapper>
                        <Fylkesfilter
                            valgteFylker={valgteFylker}
                            håndterFylkesValg={håndterFylkesValg}
                        />
                        <ValgteFylker
                            valgteFylker={valgteFylker}
                            håndterFylkesValg={håndterFylkesValg}
                        />
                    </FylkeWrapper>
                    <ValgKontainer id={id}>
                        <KnappSekundær
                            size="Small"
                            onClick={() => settSkjulFylkesveier(!skjulFylkesveier)}
                        >
                            {skjulFylkesveier ? "Vis fylkesveier" : "Skjul fylkesveier"}
                        </KnappSekundær>
                        <Trekkspill
                            åpen={trekkspillÅpen}
                            tittel={"Kamera størrelse"}
                            type={"UtenRamme"}
                            størrelse={"M"}
                            onClick={() => settTrekkspillÅpen(!trekkspillÅpen)}
                            width="fit-content"
                        >
                            {trekkspillÅpen &&
                                utvalg.map((valg, i) => (
                                    <TrekkspillElementKnapp
                                        key={i}
                                        onClick={() => {
                                            håndterKameraStørrelseEndring(valg as KameraStørrelse);
                                            settTrekkspillÅpen(false);
                                        }}
                                    >
                                        {valg}
                                    </TrekkspillElementKnapp>
                                ))}
                        </Trekkspill>
                    </ValgKontainer>
                </Meny>
            </Header>
        </HeaderBanner>
    );
};

const Header = styled.div`
    max-width: 100rem;
    flex-grow: 1;
    padding: 2rem;

    @media (max-width: ${breakpointNettbrett}) {
        padding: 1rem;
    }
`;

const HeaderBanner = styled.div`
    display: flex;
    justify-content: center;
    width: 100%;
    background-color: var(--gray-light);
`;

const Meny = styled.div`
    display: flex;
    flex-direction: row;
    justify-content: space-between;
    align-items: center;
    flex-wrap: wrap;
    gap: 1rem;
`;

const ValgKontainer = styled.div`
    display: flex;
    flex-direction: row;
    align-items: center;
    justify-content: space-between;
    gap: 10px;
    flex-wrap: wrap;
`;

const FylkeWrapper = styled.div`
    display: flex;
    flex-direction: row;
    align-items: center;
    gap: 16px;
    flex-wrap: wrap;
`;

const TrekkspillElementKnapp = styled.button`
    background: var(--white);
    border: none;
    cursor: pointer;
    ${fontsizeLargeScreen};
    padding: 4px 8px 4px 8px;
    text-align: left;
    width: 100%;

    &:hover {
        background-color: #f2f2f2;
    }

    @media (max-width: ${breakpointNettbrett}) {
        ${fontsizeSmallScreen};
    }
`;
