import React, { useEffect, useState } from "react";
import styled from "styled-components";
import Head from "next/head";
import localStorageService from "../utils/storage/localstorageService";
import { LocalStorageKey } from "../utils/storage/nøkler";
import { WebcameraParametervalg } from "./WebcameraParametervalg";
import { WebcameraInnhold } from "./WebcameraInnhold";
import { SideHeader } from "../components/sideheader/SideHeader";
import ErrorBoundary from "../components/feil/ErrorBoundary";

export enum KameraStørrelse {
    Liten = "Liten",
    Medium = "Medium",
    Stor = "Stor",
}
export const WebkameraSide = () => {
    /**
     * https://nextjs.org/docs/messages/react-hydration-errorFor
     * For å unngå mismatch i det som pre-rendres og det som rendres første gang i browser,
     * bruker vi ikke localstorage initielt og trigger en re-render ved å oppdatere state
     * ved mount
     */
    useEffect(() => {
        settValgteFylker(
            localStorageService.hentFraLocalStorage(
                LocalStorageKey.LOCALSTORAGE_WEBKAMERA_FYLKER,
                []
            )
        );
        settFavoritter(
            localStorageService.hentFraLocalStorage(
                LocalStorageKey.LOCALSTORAGE_FAVORITT_KAMERA,
                []
            )
        );
        settStørrelse(
            localStorageService.hentFraLocalStorage(
                LocalStorageKey.LOCALSTORAGE_KAMERA_STORRELSE,
                KameraStørrelse.Liten
            )
        );
    }, []);

    const [valgteFylker, settValgteFylker] = useState<string[]>([]);
    const [skjulFylker, settSkjulFylker] = useState<boolean>(false);
    const [favoritter, settFavoritter] = useState<string[]>([]);
    const [størrelse, settStørrelse] = useState<KameraStørrelse>(KameraStørrelse.Liten);

    const håndterFavorittKlikk = (kameraId: string, erFavoritt: boolean) => {
        const nyFavorittliste = erFavoritt
            ? favoritter.filter((favoritt) => favoritt !== kameraId)
            : [...favoritter, kameraId];
        settFavoritter(nyFavorittliste);
        localStorageService.lagreILocalStorage(
            LocalStorageKey.LOCALSTORAGE_FAVORITT_KAMERA,
            nyFavorittliste
        );
    };

    const håndterKameraStørrelseEndring = (størrelse: KameraStørrelse) => {
        localStorageService.lagreILocalStorage(
            LocalStorageKey.LOCALSTORAGE_KAMERA_STORRELSE,
            størrelse
        );
        settStørrelse(størrelse);
    };

    const håndterFylkesValg = (fylker: string[]) => {
        localStorageService.lagreILocalStorage(
            LocalStorageKey.LOCALSTORAGE_WEBKAMERA_FYLKER,
            fylker
        );
        settValgteFylker(fylker);
    };

    return (
        <ErrorBoundary info={`side: Webkamerasiden`}>
            <Head>
                <title>Vegkameraer - Driftsportal</title>
            </Head>
            <SideHeader side="webkamera" />
            <Sideinnhold>
                <WebcameraParametervalg
                    valgteFylker={valgteFylker}
                    skjulFylkesveier={skjulFylker}
                    settSkjulFylkesveier={settSkjulFylker}
                    håndterKameraStørrelseEndring={håndterKameraStørrelseEndring}
                    håndterFylkesValg={håndterFylkesValg}
                />
                <WebcameraInnhold
                    favoritter={favoritter}
                    valgteFylker={valgteFylker}
                    skjulFylkesveier={skjulFylker}
                    håndterFavorittKlikk={håndterFavorittKlikk}
                    størrelse={størrelse}
                />
            </Sideinnhold>
        </ErrorBoundary>
    );
};

const Sideinnhold = styled.main`
    display: flex;
    flex-direction: column;
    align-items: center;
    gap: 2rem;
`;
