import React from "react";
import styled from "styled-components";
import { hentUniqueGrafdata } from "./grafUtils";
import { Grafdata } from "./Grafer";
import { KnappUtenStyling } from "../../components/fellesKomponenter/knapper/KnappUtenStyling";

export const CustomLegend = ({
    grafdata,
    setGrafdata,
}: {
    grafdata: Grafdata[];
    setGrafdata: (data: Grafdata[]) => void;
}) => {
    const uniqueGrafdata = hentUniqueGrafdata(grafdata);

    const toggleProduksjonstype = (value: string) => {
        const newGrafdata: Grafdata[] = grafdata.map((e) => {
            if (e.value === value) {
                return {
                    ...e,
                    visible: !e.visible,
                };
            } else {
                return { ...e };
            }
        });
        setGrafdata(newGrafdata);
    };

    return (
        <LegendContainer>
            {Array.isArray(uniqueGrafdata) &&
                uniqueGrafdata?.map((e, index) => (
                    <KnappUtenStyling
                        onClick={() => toggleProduksjonstype(e.value)}
                        key={`item-${index}`}
                    >
                        <LegendItem color={e.color} visible={e.visible}>
                            {e.value}
                        </LegendItem>
                    </KnappUtenStyling>
                ))}
        </LegendContainer>
    );
};

const LegendContainer = styled.span`
    display: flex;
    flex-direction: row;
    background-color: white;
    margin: 10px;
    justify-content: center;
`;

const LegendItem = styled.p<{ color: string; visible: boolean }>`
    display: flex;
    margin: 0 10px;
    color: ${(props) => (props.visible ? props.color : `var(--gray-regular-dark)`)};

    &:before {
        content: "";
        height: 18px;
        width: 18px;
        margin-bottom: 15px;
        border: 1px solid black;
        background-color: ${(props) => (props.visible ? props.color : "white")};
        margin-right: 10px;
    }
`;
