import { TooltipProps } from "recharts";
import { NameType, ValueType } from "recharts/types/component/DefaultTooltipContent";
import dayjs from "dayjs";
import styled from "styled-components";
import React, { useEffect } from "react";
import { GrafType } from "./GeneriskGraf";
import { hentDoseringsBenevning } from "./grafUtils";

interface CustomTooltip extends TooltipProps<ValueType, NameType> {
    setAktivHoverRecord: (record: number[] | undefined) => void;
    type: GrafType;
}

export const CustomTooltip = ({
    active,
    label,
    payload,
    setAktivHoverRecord,
    type,
}: CustomTooltip) => {
    const produksjonsdataRestRec = payload && payload[0]?.payload.produksjonsdataRestRec;

    useEffect(
        () =>
            active && payload && payload.length
                ? setAktivHoverRecord(payload[0].payload.produksjonsdataRestRec)
                : setAktivHoverRecord(undefined),
        [produksjonsdataRestRec]
    );

    if (active && payload && payload.length) {
        return (
            <TooltipContainer>
                {type === GrafType.hastighet ? (
                    <p>{`Hastighet: ${payload[0].value} km/t`}</p>
                ) : (
                    <>
                        <p>{`Dosering: ${payload[0].payload.dosering} ${hentDoseringsBenevning(
                            payload[0].payload
                        )}`}</p>
                        <p>{`Sprederbredde: ${payload[0].payload.sprederbredde} cm`}</p>
                    </>
                )}
                <p>{`Dato: ${dayjs(label).format("DD.MM")}`}</p>
                <p>{`Klokkeslett: ${dayjs(label).format("HH:mm")}`}</p>
                <p>{`Produksjon: ${payload[0].payload.value}`}</p>
            </TooltipContainer>
        );
    }
    return null;
};

const TooltipContainer = styled.div`
    background-color: white;
    border: 1px solid black;
    padding: 10px;

    p {
        color: var(--gray-dark);
        margin: 0;
    }
`;
