import React, { useEffect, useState } from "react";
import {
    hentTicksYaxisDosering,
    hentTittelOgTillattDosering,
    hentUniqueGrafdata,
} from "./grafUtils";
import { GeneriskGraf, GrafType } from "./GeneriskGraf";
import { Grafdata } from "./Grafer";
import { DoseringsValg } from "./DoseringsValg";
import { ReferanselinjeValg } from "./ReferanselinjeValg";
import { ValgContainer } from "./GeneriskGrafvalg";
import { Skillelinje } from "./styled";
import { AktivGrafValg } from "./AktivGrafValg";

export enum DoseringsType {
    dosering = "dosering",
    sprederbredde = "sprederbredde",
}

interface DoseringsGrafProps {
    grafdata: Grafdata[];
    setAktivHoverRecord: (record: number[] | undefined) => void;
    aktivGraf: GrafType;
    setAktivGraf: (graf: GrafType) => void;
}
export const DoseringsGraf = ({
    grafdata,
    setAktivHoverRecord,
    aktivGraf,
    setAktivGraf,
}: DoseringsGrafProps) => {
    const [doseringsType, setDoseringsType] = React.useState<DoseringsType>(DoseringsType.dosering);
    const [reflinjeIKart, setReflinjeIKart] = useState<number | undefined>(undefined);
    const tickArrayYaxis = hentTicksYaxisDosering(grafdata, doseringsType);
    const uniqueGrafdata = hentUniqueGrafdata(grafdata);
    const trekkspillInnhold = hentTittelOgTillattDosering(uniqueGrafdata);

    useEffect(() => {
        setReflinjeIKart(undefined);
    }, [doseringsType]);

    return (
        <>
            <GeneriskGraf
                type={GrafType.dosering}
                dataKey={doseringsType}
                tickArrayYaxis={tickArrayYaxis}
                grafdata={grafdata}
                setAktivHoverRecord={setAktivHoverRecord}
                reflinjeIKart={reflinjeIKart}
            >
                <ValgContainer>
                    <AktivGrafValg aktivGraf={aktivGraf} setAktivGraf={setAktivGraf} />
                    <Skillelinje />
                    <DoseringsValg
                        doseringsType={doseringsType}
                        setDoseringsType={(type: DoseringsType) => setDoseringsType(type)}
                    />
                    <Skillelinje />
                    {doseringsType === DoseringsType.dosering && (
                        <ReferanselinjeValg
                            trekkspillInnhold={trekkspillInnhold}
                            setReflinjeIKart={setReflinjeIKart}
                        />
                    )}
                </ValgContainer>
            </GeneriskGraf>
        </>
    );
};
