import React, { ReactNode } from "react";
import styled from "styled-components";
import { Tekst } from "../../components/fellesKomponenter/typografi/Tekst";
import { RowWrapper } from "../../components/fellesKomponenter/wrappers/wrappers";
import { KnappSekundær } from "../../components/fellesKomponenter/knapper/KnappSekundær";

interface GeneriskGrafvalgProps {
    visTilbakestillknapp: boolean;
    handleReset: () => void;
    children: ReactNode;
}
export const GeneriskGrafvalg = ({
    visTilbakestillknapp,
    handleReset,
    children,
}: GeneriskGrafvalgProps) => {
    return (
        <GrafvalgContainer>
            <Innhold>
                <Tekst noMargin={true} størrelse={"M"}>
                    Zoom i grafen ved å klikke å dra ønsket intervall
                </Tekst>
                {visTilbakestillknapp && (
                    <RowWrapper>
                        <KnappSekundær onClick={handleReset} size={"Small"}>
                            Tilbakestill graf
                        </KnappSekundær>
                    </RowWrapper>
                )}
                <InnstillingerWrapper>
                    <InnstillingerTekst>Innstillinger for graf</InnstillingerTekst>
                    <Tekst noMargin={true} størrelse={"M"}>
                        Verdiene viser gjennomsnitt per minutt.
                    </Tekst>
                    <Tekst noMargin={true} størrelse={"M"}>
                        Du kan legge inn referanseverdier i grafen for å lett kunne se om verdiene
                        er over eller under standarene.
                    </Tekst>
                </InnstillingerWrapper>
                {children}
            </Innhold>
        </GrafvalgContainer>
    );
};

const GrafvalgContainer = styled.div`
    display: flex;
    flex-direction: column;
    width: 100%;
    background-color: var(--gray-light);
    height: fit-content;
    overflow: auto;
    padding-bottom: 100px;
`;

const Innhold = styled.div`
    display: flex;
    flex-direction: column;
    padding: 30px;
    gap: 20px;
`;

const InnstillingerTekst = styled(Tekst).attrs(() => ({
    størrelse: "M",
    noMargin: true,
    bold: true,
}))`
    border-bottom: 2px solid var(--orange);
    width: fit-content;
    margin-top: 10px;
`;

export const ValgContainer = styled.div`
    display: flex;
    flex-direction: row;
    justify-content: flex-start;
    gap: 40px;
    flex-wrap: wrap;
`;

const InnstillingerWrapper = styled.div`
    display: flex;
    flex-direction: column;
    gap: 5px;
`;
