import styled from "styled-components";
import React, { useEffect, useState } from "react";
import { Produksjonsdetaljer } from "../../kartSide/types/produksjonsdetaljer";
import { hentLinjeFarge, hentTittel, lagSegmentertListe } from "./grafUtils";
import { HastighetsGraf } from "./HastighetsGraf";
import { DoseringsGraf } from "./DoseringsGraf";
import { GrafType } from "./GeneriskGraf";

export interface Grafdata {
    color: string;
    value: string;
    data: Produksjonsdetaljer[];
    visible: boolean;
}
export const Grafer = ({
    produksjonsdetaljer,
    setAktivHoverRecord,
}: {
    produksjonsdetaljer: Produksjonsdetaljer[];
    setAktivHoverRecord: (record: number[] | undefined) => void;
}) => {
    const [aktivGraf, setAktivGraf] = useState<GrafType>(GrafType.hastighet);
    const [grafdata, setGrafdata] = useState<Grafdata[]>([]);

    useEffect(() => {
        const segmenterteProddetaljer = lagSegmentertListe(produksjonsdetaljer);
        const grafdata = segmenterteProddetaljer.map((produksjonsdetaljer) => {
            return {
                color: hentLinjeFarge(produksjonsdetaljer),
                value: hentTittel(produksjonsdetaljer),
                data: produksjonsdetaljer,
                visible: true,
            };
        });
        setGrafdata(grafdata);
    }, [produksjonsdetaljer]);

    const grafdataDosering = grafdata?.filter((g) =>
        g.data.some((prod) => prod.dosering !== undefined && prod.dosering !== 0)
    );

    const visDosering = grafdataDosering && grafdataDosering.length > 0;

    return (
        <Container>
            {aktivGraf === GrafType.hastighet && grafdata.length > 0 && (
                <GrafContainer>
                    <HastighetsGraf
                        grafdata={grafdata}
                        setAktivHoverRecord={setAktivHoverRecord}
                        aktivGraf={aktivGraf}
                        setAktivGraf={setAktivGraf}
                        visDosering={visDosering}
                    />
                </GrafContainer>
            )}
            {aktivGraf === GrafType.dosering && visDosering && (
                <GrafContainer>
                    <DoseringsGraf
                        grafdata={grafdataDosering}
                        setAktivHoverRecord={setAktivHoverRecord}
                        aktivGraf={aktivGraf}
                        setAktivGraf={setAktivGraf}
                    />
                </GrafContainer>
            )}
        </Container>
    );
};

const Container = styled.div`
    display: flex;
    flex-direction: column;
`;

const GrafContainer = styled.div`
    display: flex;
    flex-direction: column;
    width: 100%;
    margin: 20px 0;
`;
