import React, { useState } from "react";
import {
    hentTicksYaxisHastighet,
    hentTittelOgTillattHastighet,
    hentUniqueGrafdata,
} from "./grafUtils";
import { HastighetsValg } from "./HastighetsValg";
import { ReferanselinjeValg } from "./ReferanselinjeValg";
import { GeneriskGraf, GrafType } from "./GeneriskGraf";
import { Grafdata } from "./Grafer";
import { ValgContainer } from "./GeneriskGrafvalg";
import { Skillelinje } from "./styled";
import { AktivGrafValg } from "./AktivGrafValg";

export enum HastighetsType {
    hastighetKmHMax = "hastighetKmHMax",
    hastighetKmHAvg = "hastighetKmHAvg",
}

interface HastighetsGrafProps {
    grafdata: Grafdata[];
    setAktivHoverRecord: (record: number[] | undefined) => void;
    aktivGraf: GrafType;
    setAktivGraf: (graf: GrafType) => void;
    visDosering: boolean;
}
export const HastighetsGraf = ({
    grafdata,
    setAktivHoverRecord,
    aktivGraf,
    setAktivGraf,
    visDosering,
}: HastighetsGrafProps) => {
    const [hastighetsType, setHastighetsType] = useState<HastighetsType>(
        HastighetsType.hastighetKmHMax
    );
    const [reflinjeIKart, setReflinjeIKart] = useState<number | undefined>(undefined);
    const tickArrayYaxis = hentTicksYaxisHastighet(grafdata);
    const uniqueGrafdata = hentUniqueGrafdata(grafdata);
    const hastighetTrekkspill = hentTittelOgTillattHastighet(uniqueGrafdata);

    return (
        <>
            <GeneriskGraf
                type={GrafType.hastighet}
                dataKey={hastighetsType}
                tickArrayYaxis={tickArrayYaxis}
                grafdata={grafdata}
                setAktivHoverRecord={setAktivHoverRecord}
                reflinjeIKart={reflinjeIKart}
            >
                <ValgContainer>
                    {visDosering && (
                        <>
                            <AktivGrafValg aktivGraf={aktivGraf} setAktivGraf={setAktivGraf} />
                            <Skillelinje />
                        </>
                    )}
                    <HastighetsValg
                        hastighetsType={hastighetsType}
                        setHastighetsType={(type: HastighetsType) => setHastighetsType(type)}
                    />
                    <Skillelinje />
                    <ReferanselinjeValg
                        trekkspillInnhold={hastighetTrekkspill}
                        setReflinjeIKart={setReflinjeIKart}
                    />
                </ValgContainer>
            </GeneriskGraf>
        </>
    );
};
