import { FormLabel } from "../../components/fellesKomponenter/inputfelt/FormLabel";
import { Inputfelt } from "../../components/fellesKomponenter/inputfelt/Inputfelt";
import { KnappPrimær } from "../../components/fellesKomponenter/knapper/KnappPrimær";
import { KnappSekundær } from "../../components/fellesKomponenter/knapper/KnappSekundær";
import { Trekkspill } from "../../components/fellesKomponenter/trekkspill/Trekkspill";
import React, { useState } from "react";
import styled from "styled-components";
import { erGyldigNumeriskInput } from "../../utils/inputValidering";
import { useCloseOnOutsideClick } from "../../hooks/useCloseOnOutsideClick";
import { ColumnWrapperValg } from "./styled";
import { TrekkspillInnhold } from "./grafUtils";
import {
    fontsizeLargeScreen,
    fontsizeSmallScreen,
} from "../../components/fellesKomponenter/typografi/TekstStyles";
import { breakpointNettbrett } from "../../utils/constants";

interface ReferanselinjeValgProps {
    trekkspillInnhold: { tittel: string; verdi: number | undefined }[];
    setReflinjeIKart: (input: number | undefined) => void;
}
export const ReferanselinjeValg = ({
    trekkspillInnhold,
    setReflinjeIKart,
}: ReferanselinjeValgProps) => {
    const [trekkspillOpen, setTrekkspillOpen] = useState<boolean>(false);
    const [reflinjeInput, setReflinjeInput] = useState<string>("");
    const [error, setError] = useState<boolean>(false);
    const [aktivtTrekkspillElement, setAktivtTrekkspillElement] = useState<
        TrekkspillInnhold | undefined
    >(undefined);

    const handleSubmit = () => {
        const gyldigInput = erGyldigNumeriskInput(reflinjeInput);
        setError(!gyldigInput);
        if (gyldigInput) {
            setReflinjeIKart(parseInt(reflinjeInput));
        }
    };
    const nullstillReflinje = () => {
        setReflinjeInput("");
        setError(false);
        setAktivtTrekkspillElement(undefined);
        setReflinjeIKart(undefined);
    };
    useCloseOnOutsideClick("reflinjeTrekkspill", () => setTrekkspillOpen(false));

    return (
        <>
            <ColumnWrapperValg>
                <FormLabel htmlFor="referanselinjeInput">Skriv inn grenseverdi</FormLabel>
                <RowWrapper>
                    <ColumnWrapperValg>
                        <Inputfelt
                            error={error}
                            type="text"
                            id="referanselinjeInput"
                            størrelse="S"
                            onChange={(e) => setReflinjeInput(e.target.value)}
                            value={reflinjeInput}
                            pattern="[0-9]+"
                        />
                        {error && <ErrorMelding>Skriv inn et tall</ErrorMelding>}
                    </ColumnWrapperValg>

                    <KnappPrimær size={"Small"} onClick={handleSubmit}>
                        Ok
                    </KnappPrimær>
                    <KnappSekundær size={"Small"} onClick={() => nullstillReflinje()}>
                        Nullstill
                    </KnappSekundær>
                </RowWrapper>
            </ColumnWrapperValg>
            <ColumnWrapperValg id={"reflinjeTrekkspill"}>
                <FormLabel htmlFor="referanselinje">Velg standarisert grenseverdi</FormLabel>
                <Trekkspill
                    åpen={trekkspillOpen}
                    tittel={
                        aktivtTrekkspillElement
                            ? aktivtTrekkspillElement.tittel
                            : "Velg standarisert grenseverdi"
                    }
                    type={"Ramme"}
                    onClick={() => setTrekkspillOpen(!trekkspillOpen)}
                    width="300px"
                    størrelse={"M"}
                >
                    <Innhold>
                        {trekkspillInnhold.map((element, index) => (
                            <TrekkspillElementKnapp
                                key={index}
                                onClick={() => {
                                    setReflinjeIKart(element.verdi);
                                    setAktivtTrekkspillElement(element);
                                    setReflinjeInput("");
                                    setError(false);
                                    setTrekkspillOpen(false);
                                }}
                            >
                                {element.tittel}
                            </TrekkspillElementKnapp>
                        ))}
                    </Innhold>
                </Trekkspill>
            </ColumnWrapperValg>
        </>
    );
};

const Innhold = styled.div`
    background-color: var(--white);
    padding: 0 1rem;
    margin-bottom: 0.5rem;
    margin-top: 0.5rem;
`;

const TrekkspillElementKnapp = styled.button`
    background: var(--white);
    border: none;
    cursor: pointer;
    ${fontsizeLargeScreen};
    padding: 4px 8px 4px 8px;
    text-align: left;
    width: 100%;
    color: var(--gray-dark);

    &:hover {
        background-color: #f2f2f2;
    }

    @media (max-width: ${breakpointNettbrett}) {
        ${fontsizeSmallScreen};
    }
`;

const RowWrapper = styled.div`
    display: flex;
    flex-direction: row;
    gap: 0.5rem;
    align-items: flex-start;
`;

export const ErrorMelding = styled.p`
    color: var(--gray-dark);
    margin: 0 0 0 0;
    max-width: 120px;
    position: relative;
    background-color: var(--red-bg);
    padding: 2px;
`;
