import { Produksjonsdetaljer } from "../../kartSide/types/produksjonsdetaljer";
import { MaterialOgAktivitetsNavn } from "../../api/context/ProduksjonsfilterContext";
import { MaterialKategori, MaterialType } from "../../kartSide/types/materiale";
import {
    erLikProduksjon,
    lagSegmentertListe,
    hentTillattMakshastighet,
    hentTittel,
} from "./grafUtils";

describe("Avvik utils", () => {
    it("skal returnere true ved lik produksjon", () => {
        const erLikProduksjon1 = erLikProduksjon(produksjonsdetaljer[0], produksjonsdetaljer[1]);
        const erLikProduksjon2 = erLikProduksjon(produksjonsdetaljer[0], produksjonsdetaljer[6]);
        const erLikProduksjon3 = erLikProduksjon(produksjonsdetaljer[3], produksjonsdetaljer[4]);
        const erLikProduksjon4 = erLikProduksjon(produksjonsdetaljer[4], produksjonsdetaljer[5]);
        expect(erLikProduksjon1).toBe(true);
        expect(erLikProduksjon2).toBe(false);
        expect(erLikProduksjon3).toBe(true);
        expect(erLikProduksjon4).toBe(false);
    });

    it("skal dele opp liste ved ulike produksjonstyper og tid", () => {
        const fasit: Produksjonsdetaljer[][] = [
            [produksjonsdetaljer[0], produksjonsdetaljer[1], produksjonsdetaljer[2]],
            [produksjonsdetaljer[3], produksjonsdetaljer[4]],
            [produksjonsdetaljer[5]],
            [produksjonsdetaljer[6]],
            [produksjonsdetaljer[7]],
        ];

        const hastighetsliste = lagSegmentertListe(produksjonsdetaljer);
        expect(hastighetsliste.length).toEqual(fasit.length);
        expect(hastighetsliste[0][0].datoLogget).toEqual(fasit[0][0].datoLogget);
    });

    it("skal endre dato logget til riktig format for graf", () => {
        const dato = produksjonsdetaljer[0].datoLogget;
        const datoEndret = dato.slice(0, 10) + " " + dato.slice(11, 16);
        expect(datoEndret).toBe("2023-01-28 04:17");
    });

    it("skal hente riktig tittel", () => {
        const brøyting = [produksjonsdetaljer[0], produksjonsdetaljer[1], produksjonsdetaljer[2]];
        const fastsand = [produksjonsdetaljer[3]];
        const tørrsaltOgPlog = [produksjonsdetaljer[5]];
        const ingenProd = [produksjonsdetaljer[7]];

        expect(hentTittel(brøyting)).toBe("Brøyting");
        expect(hentTittel(fastsand)).toBe("Strøing med fastsand");
        expect(hentTittel(tørrsaltOgPlog)).toBe("Brøyting og strøing med tørrsalt");
        expect(hentTittel(ingenProd)).toBe("Ingen produksjon");
    });

    it("skal hente riktig tillatt makshastighet", () => {
        const brøyting = [produksjonsdetaljer[0], produksjonsdetaljer[1], produksjonsdetaljer[2]];
        const fastsand = [produksjonsdetaljer[3]];
        const tørrsaltOgPlog = [produksjonsdetaljer[5]];
        const ingenProd = [produksjonsdetaljer[7]];

        expect(hentTillattMakshastighet(brøyting)).toBe(40);
        expect(hentTillattMakshastighet(fastsand)).toBe(25);
        expect(hentTillattMakshastighet(tørrsaltOgPlog)).toBe(40);
        expect(hentTillattMakshastighet(ingenProd)).toBe(undefined);
    });
});

const produksjonsdetaljer: Produksjonsdetaljer[] = [
    {
        produksjonsdataRestRec: [0],
        kjøretøyId: "6883",
        datoLogget: "2023-01-28T04:17:05",
        hastighetKmHMax: 48.0,
        hastighetKmHAvg: 48.0,
        erTørrstoffsprederPå: false,
        erVåtstoffsprederPå: false,
        erPlogNede: true,
        eksternReferanse: "644453",
        minutterSiden: 50,
        startLokasjon: {
            recordId: 2954550446,
            punkt: {
                type: "Point",
                coordinates: [17.372131, 68.743065],
            },
        },
        sluttLokasjon: {
            recordId: 2954550446,
            punkt: {
                type: "Point",
                coordinates: [17.372131, 68.743065],
            },
        },
        vegsystemreferansekortform: "FV7806 S3D1 m1784",
        rodeId: ["zi_4283"],
    },
    {
        produksjonsdataRestRec: [1],
        kjøretøyId: "6883",
        datoLogget: "2023-01-28T04:18:06",
        hastighetKmHMax: 61.0,
        hastighetKmHAvg: 61.0,
        erTørrstoffsprederPå: false,
        erVåtstoffsprederPå: false,
        erPlogNede: true,
        eksternReferanse: "644453",
        minutterSiden: 50,
        startLokasjon: {
            recordId: 2954550446,
            punkt: {
                type: "Point",
                coordinates: [17.372131, 68.743065],
            },
        },
        sluttLokasjon: {
            recordId: 2954550446,
            punkt: {
                type: "Point",
                coordinates: [17.372131, 68.743065],
            },
        },
        vegsystemreferansekortform: "FV7806 S3D1 m1784",
        rodeId: ["zi_4283"],
    },
    {
        produksjonsdataRestRec: [2],
        kjøretøyId: "6883",
        datoLogget: "2023-01-28T04:19:07",
        hastighetKmHMax: 49.0,
        hastighetKmHAvg: 49.0,
        erTørrstoffsprederPå: false,
        erVåtstoffsprederPå: false,
        erPlogNede: true,
        eksternReferanse: "644453",
        minutterSiden: 50,
        startLokasjon: {
            recordId: 2954550446,
            punkt: {
                type: "Point",
                coordinates: [17.372131, 68.743065],
            },
        },
        sluttLokasjon: {
            recordId: 2954550446,
            punkt: {
                type: "Point",
                coordinates: [17.372131, 68.743065],
            },
        },
        vegsystemreferansekortform: "FV7806 S3D1 m1784",
        rodeId: ["zi_4283"],
    },
    {
        produksjonsdataRestRec: [3],
        kjøretøyId: "6883",
        datoLogget: "2023-01-28T04:20:08",
        hastighetKmHMax: 40.0,
        hastighetKmHAvg: 40.0,
        erTørrstoffsprederPå: true,
        erVåtstoffsprederPå: false,
        erPlogNede: false,
        eksternReferanse: "644453",
        minutterSiden: 50,
        materiale: {
            type: MaterialType.Fastsand,
            kategori: MaterialKategori.Sand,
            navn: MaterialOgAktivitetsNavn.Fastsand,
        },
        startLokasjon: {
            recordId: 2954550446,
            punkt: {
                type: "Point",
                coordinates: [17.372131, 68.743065],
            },
        },
        sluttLokasjon: {
            recordId: 2954550446,
            punkt: {
                type: "Point",
                coordinates: [17.372131, 68.743065],
            },
        },
        vegsystemreferansekortform: "FV7806 S3D1 m1784",
        rodeId: ["zi_4283"],
    },
    {
        produksjonsdataRestRec: [4],
        kjøretøyId: "6883",
        datoLogget: "2023-01-28T04:21:09",
        hastighetKmHMax: 40.0,
        hastighetKmHAvg: 40.0,
        erTørrstoffsprederPå: true,
        erVåtstoffsprederPå: false,
        erPlogNede: false,
        eksternReferanse: "644453",
        minutterSiden: 50,
        materiale: {
            type: MaterialType.Fastsand,
            kategori: MaterialKategori.Sand,
            navn: MaterialOgAktivitetsNavn.Fastsand,
        },
        startLokasjon: {
            recordId: 2954550446,
            punkt: {
                type: "Point",
                coordinates: [17.372131, 68.743065],
            },
        },
        sluttLokasjon: {
            recordId: 2954550446,
            punkt: {
                type: "Point",
                coordinates: [17.372131, 68.743065],
            },
        },
        vegsystemreferansekortform: "FV7806 S3D1 m1784",
        rodeId: ["zi_4283"],
    },
    {
        produksjonsdataRestRec: [5],
        kjøretøyId: "6883",
        datoLogget: "2023-01-28T04:22:10",
        hastighetKmHMax: 50.0,
        hastighetKmHAvg: 50.0,
        erTørrstoffsprederPå: true,
        erVåtstoffsprederPå: false,
        erPlogNede: true,
        eksternReferanse: "644453",
        minutterSiden: 50,
        materiale: {
            type: MaterialType.Tørrsalt,
            kategori: MaterialKategori.Salt,
            navn: MaterialOgAktivitetsNavn.Tørrsalt,
        },
        startLokasjon: {
            recordId: 2954550446,
            punkt: {
                type: "Point",
                coordinates: [17.372131, 68.743065],
            },
        },
        sluttLokasjon: {
            recordId: 2954550446,
            punkt: {
                type: "Point",
                coordinates: [17.372131, 68.743065],
            },
        },
        vegsystemreferansekortform: "FV7806 S3D1 m1784",
        rodeId: ["zi_4283"],
    },
    {
        produksjonsdataRestRec: [6],
        kjøretøyId: "6883",
        datoLogget: "2023-01-28T04:26:15",
        hastighetKmHMax: 50.0,
        hastighetKmHAvg: 50.0,
        erTørrstoffsprederPå: true,
        erVåtstoffsprederPå: false,
        erPlogNede: true,
        eksternReferanse: "644453",
        minutterSiden: 50,
        materiale: {
            type: MaterialType.Tørrsalt,
            kategori: MaterialKategori.Salt,
            navn: MaterialOgAktivitetsNavn.Tørrsalt,
        },
        startLokasjon: {
            recordId: 2954550446,
            punkt: {
                type: "Point",
                coordinates: [17.372131, 68.743065],
            },
        },
        sluttLokasjon: {
            recordId: 2954550446,
            punkt: {
                type: "Point",
                coordinates: [17.372131, 68.743065],
            },
        },
        vegsystemreferansekortform: "FV7806 S3D1 m1784",
        rodeId: ["zi_4283"],
    },
    {
        produksjonsdataRestRec: [7],
        kjøretøyId: "6883",
        datoLogget: "2023-01-28T04:27:16",
        hastighetKmHMax: 52.0,
        hastighetKmHAvg: 52.0,
        erTørrstoffsprederPå: false,
        erVåtstoffsprederPå: false,
        erPlogNede: false,
        eksternReferanse: "644453",
        minutterSiden: 50,
        startLokasjon: {
            recordId: 2954550446,
            punkt: {
                type: "Point",
                coordinates: [17.372131, 68.743065],
            },
        },
        sluttLokasjon: {
            recordId: 2954550446,
            punkt: {
                type: "Point",
                coordinates: [17.372131, 68.743065],
            },
        },
        vegsystemreferansekortform: "FV7806 S3D1 m1784",
        rodeId: ["zi_4283"],
    },
];
