import { Produksjonsdetaljer } from "../../kartSide/types/produksjonsdetaljer";
import { capitalizeFirst } from "../../utils/tekst";
import { MaterialOgAktivitetsNavn } from "../../api/context/ProduksjonsfilterContext";
import dayjs from "dayjs";
import { MaterialKategori } from "../../kartSide/types/materiale";
import { Grafdata } from "./Grafer";
import { DoseringsType } from "./DoseringsGraf";
import { GrafType } from "./GeneriskGraf";
import { HastighetsType } from "./HastighetsGraf";

export const lagSegmentertListe = (
    produksjonsdetaljer: Produksjonsdetaljer[]
): Produksjonsdetaljer[][] => {
    if (produksjonsdetaljer.length === 0) {
        return [];
    }

    const dataMedTittelOgNumeriskTid = produksjonsdetaljer.map((h) => {
        const tittel = hentTittel([h]);
        return {
            ...h,
            value: tittel,
            numeriskTid: dayjs(h.datoLogget).valueOf(),
        };
    });

    let totalArray: Produksjonsdetaljer[][] = [];
    let currentArray: Produksjonsdetaljer[] = [];

    for (let i = 0; i < produksjonsdetaljer.length; i++) {
        if (i === 0) {
            currentArray.push(dataMedTittelOgNumeriskTid[i]);
        } else if (
            erMerEnnToMinutterFraForrige(
                dataMedTittelOgNumeriskTid[i - 1].numeriskTid,
                dataMedTittelOgNumeriskTid[i].numeriskTid
            )
        ) {
            totalArray.push(currentArray);
            currentArray = [];
            currentArray.push(dataMedTittelOgNumeriskTid[i]);
        } else if (
            erLikProduksjon(dataMedTittelOgNumeriskTid[i - 1], dataMedTittelOgNumeriskTid[i])
        ) {
            currentArray.push(dataMedTittelOgNumeriskTid[i]);
        } else {
            totalArray.push(currentArray);
            currentArray = [];
            currentArray.push(dataMedTittelOgNumeriskTid[i]);
        }

        if (i === dataMedTittelOgNumeriskTid.length - 1 && currentArray.length > 0) {
            totalArray.push(currentArray);
        }
    }

    return totalArray;
};

export const erMerEnnToMinutterFraForrige = (tid1: number, tid2: number): boolean => {
    return tid2 - tid1 > 120000;
};

export const erLikProduksjon = (
    produksjon1: Produksjonsdetaljer,
    produksjon2: Produksjonsdetaljer
): boolean => {
    return (
        produksjon1.erTørrstoffsprederPå === produksjon2.erTørrstoffsprederPå &&
        produksjon1.erVåtstoffsprederPå === produksjon2.erVåtstoffsprederPå &&
        produksjon1.erPlogNede === produksjon2.erPlogNede &&
        produksjon1.erSideplogIBruk === produksjon2.erSideplogIBruk &&
        produksjon1.erHøvelskjærNede === produksjon2.erHøvelskjærNede &&
        produksjon1.erUnderliggendeSkjærNede === produksjon2.erUnderliggendeSkjærNede &&
        produksjon1.materiale?.navn === produksjon2.materiale?.navn
    );
};

export const hentTittel = (dataliste: Produksjonsdetaljer[]): string => {
    const data = dataliste[0];
    const erBrøyting = data.erPlogNede || data.erSideplogIBruk;
    const erHøvling = data.erHøvelskjærNede || data.erUnderliggendeSkjærNede;
    const erStrøing = data.erTørrstoffsprederPå || data.erVåtstoffsprederPå;
    const erIngenProuksjon = !erBrøyting && !erHøvling && !erStrøing;

    const brøyting = erBrøyting ? "Brøyting" : "";
    const høvling = erHøvling ? "Høvling" : "";
    const strøing = erStrøing ? "Strøing med " + data.materiale?.navn : "";
    const ingenProduksjon = erIngenProuksjon ? "Ingen produksjon" : "";

    const kombinertstreng = [brøyting, høvling, strøing]
        .filter((label) => label !== "")
        .join(" og ");

    return kombinertstreng !== "" ? capitalizeFirst(kombinertstreng) : ingenProduksjon;
};

export const hentLinjeFarge = (dataliste: Produksjonsdetaljer[]): string => {
    const data = dataliste[0];
    const erBrøyting = data.erPlogNede || data.erSideplogIBruk;
    const erHøvling = data.erHøvelskjærNede || data.erUnderliggendeSkjærNede;
    const erStrøing = data.erTørrstoffsprederPå || data.erVåtstoffsprederPå;
    const erIngenProuksjon = !erBrøyting && !erHøvling && !erStrøing;
    const erSalt =
        data.materiale?.kategori === MaterialKategori.Salt
            ? "Strøing med " + data.materiale?.navn
            : "";
    const erSand =
        data.materiale?.kategori === MaterialKategori.Sand
            ? "Strøing med " + data.materiale?.navn
            : "";
    if (erIngenProuksjon) {
        return "#b2b0b0";
    } else if (erBrøyting && !erHøvling && !erStrøing) {
        return "#403f8c";
    } else if (erBrøyting && erHøvling && !erStrøing) {
        return "#ae36e8";
    } else if (!erBrøyting && erHøvling && !erStrøing) {
        return "#0d9ff1";
    } else if (!erBrøyting && !erHøvling && erStrøing) {
        if (erSalt) {
            return "#29a640";
        } else if (erSand) {
            return "#ffae00";
        }
    } else if ((erBrøyting || erHøvling) && erStrøing) {
        if (erSalt) {
            return "#145d24";
        } else if (erSand) {
            return "#d75f03";
        }
    } else {
        return "#000000";
    }
    return "#000000";
};

export const hentTillattMakshastighet = (dataliste: Produksjonsdetaljer[]): number | undefined => {
    const data = dataliste[0];
    const erBrøyting = data.erPlogNede || data.erSideplogIBruk;
    const erHøvling = data.erHøvelskjærNede || data.erUnderliggendeSkjærNede;
    const erfastsand = data.materiale?.navn === MaterialOgAktivitetsNavn.Fastsand;
    const ersandØvrig =
        data.materiale?.navn === MaterialOgAktivitetsNavn.Tørrsand ||
        data.materiale?.navn === MaterialOgAktivitetsNavn.SandMedSaltInnblanding;
    const erSaltløsning = data.materiale?.navn === MaterialOgAktivitetsNavn.Saltløsning;
    const erSaltslurry = data.materiale?.navn === MaterialOgAktivitetsNavn.BefuktetFinkornetSalt;
    const erSaltØvrig =
        data.materiale?.navn === MaterialOgAktivitetsNavn.Tørrsalt ||
        data.materiale?.navn === MaterialOgAktivitetsNavn.BefuktetSalt ||
        data.materiale?.navn === MaterialOgAktivitetsNavn.Saltløsning ||
        data.materiale?.navn === MaterialOgAktivitetsNavn.Magnesiumklorid;

    const brøytingMaks = erBrøyting ? 40 : undefined;
    const høvlingMaks = erHøvling ? 40 : undefined;
    const fastsandMaks = erfastsand ? 25 : undefined;
    const sandØvrigMaks = ersandØvrig ? 30 : undefined;
    const saltløsningMaks = erSaltløsning ? 50 : undefined;
    const saltslurryMaks = erSaltslurry ? 35 : undefined;
    const saltØvrigMaks = erSaltØvrig ? 30 : undefined;

    const tillattHastighetsListe: number[] = [
        brøytingMaks,
        høvlingMaks,
        fastsandMaks,
        sandØvrigMaks,
        saltløsningMaks,
        saltslurryMaks,
        saltØvrigMaks,
    ].filter((label) => label !== undefined) as number[];

    return tillattHastighetsListe.length > 0 ? Math.max(...tillattHastighetsListe) : undefined;
};

export const hentTillattDosering = (dataliste: Produksjonsdetaljer[]): number | undefined => {
    const data = dataliste[0];
    const erfastsand = data.materiale?.navn === MaterialOgAktivitetsNavn.Fastsand;
    const ersandØvrig =
        data.materiale?.navn === MaterialOgAktivitetsNavn.Tørrsand ||
        data.materiale?.navn === MaterialOgAktivitetsNavn.SandMedSaltInnblanding;
    const erSalt = data.materiale?.kategori === MaterialKategori.Salt;

    const fastsandMin = erfastsand ? 150 : undefined;
    const sandØvrigMin = ersandØvrig ? 180 : undefined;
    const saltMaks = erSalt ? 40 : undefined;

    const tillattDoseringsListe: number[] = [fastsandMin, sandØvrigMin, saltMaks].filter(
        (label) => label !== undefined
    ) as number[];
    return tillattDoseringsListe.length > 0 ? Math.max(...tillattDoseringsListe) : undefined;
};

export function formatAxisTid(tickItem: string) {
    return dayjs(tickItem).format("HH:mm");
}

export const concatVegsystemreferanse = (produksjonsdetaljer: Produksjonsdetaljer[]) => {
    if (produksjonsdetaljer.length === 0) {
        return [];
    }
    const vegreflisteUtenMeter = produksjonsdetaljer.map((produksjonsdetalj) => {
        const lastIndex = produksjonsdetalj.vegsystemreferansekortform?.lastIndexOf(" ");
        return produksjonsdetalj.vegsystemreferansekortform?.substring(0, lastIndex);
    });

    let unikVegrefliste: string[] = [];
    vegreflisteUtenMeter
        .reduce((accumulator, current) => {
            if (!unikVegrefliste.find((item: any) => item === current)) {
                current && unikVegrefliste.push(current);
            }
            return unikVegrefliste;
        }, {} as string[])
        .sort();

    return unikVegrefliste;
};

export const hentUniqueGrafdata = (grafdataliste: Grafdata[]) => {
    let list: Grafdata[] = [];
    const uniqueGrafdata: Grafdata[] = grafdataliste.reduce((accumulator, current) => {
        if (!list.find((item: any) => item.value === current.value)) {
            list.push(current);
        }
        return list;
    }, {} as Grafdata[]);
    return uniqueGrafdata;
};

export interface TrekkspillInnhold {
    tittel: string;
    verdi: number | undefined;
}
export const hentTittelOgTillattHastighet = (grafdataliste: Grafdata[]): TrekkspillInnhold[] => {
    const trekkspillInnhold: TrekkspillInnhold[] = Array.isArray(grafdataliste)
        ? grafdataliste.map((grafdata) => {
              const tittel = hentTittel(grafdata.data);
              const tillattMakshastighet = hentTillattMakshastighet(grafdata.data);
              return {
                  tittel: tittel + " (" + tillattMakshastighet + " km/t)",
                  verdi: tillattMakshastighet,
              };
          })
        : [];

    return trekkspillInnhold.filter((innhold) => innhold.verdi !== undefined);
};

export const hentTittelOgTillattDosering = (grafdataliste: Grafdata[]): TrekkspillInnhold[] => {
    const trekkspillInnhold: TrekkspillInnhold[] = Array.isArray(grafdataliste)
        ? grafdataliste.map((grafdata) => {
              const tittel = hentTittel(grafdata.data);
              const tillattDosering = hentTillattDosering(grafdata.data);
              const benevning = hentDoseringsBenevning(grafdata.data[0]);
              const minEllerMaks =
                  grafdata.data[0].materiale?.kategori === MaterialKategori.Salt ? "maks " : "min ";
              return {
                  tittel: tittel + " (" + minEllerMaks + tillattDosering + " " + benevning + ")",
                  verdi: tillattDosering,
              };
          })
        : [];

    return trekkspillInnhold.filter((innhold) => innhold.verdi !== undefined);
};

const lagArrayMedInkrementer = (lengde: number, inkrement: number) => {
    return Array.from({ length: lengde }, (_, i) => i * inkrement);
};

export const hentTicksYaxisHastighet = (grafdataliste: Grafdata[]) => {
    const maksMåltHastighet =
        Math.max(
            ...grafdataliste
                .map((grafdata) => grafdata.data)
                .flat()
                .map((produksjonsdetalj) => produksjonsdetalj.hastighetKmHMax)
        ) + 10;
    const possibleTicksYaxis = lagArrayMedInkrementer(20, 10);
    return possibleTicksYaxis.filter((tick) => tick <= maksMåltHastighet);
};

export const hentTicksYaxisDosering = (grafdataliste: Grafdata[], doseringsType: DoseringsType) => {
    const maksMåltDosering =
        Math.max(
            ...grafdataliste
                .map((grafdata) => grafdata.data)
                .flat()
                .map((produksjonsdetalj) =>
                    doseringsType === DoseringsType.dosering
                        ? produksjonsdetalj.dosering ?? 0
                        : produksjonsdetalj.sprederbredde ?? 0
                )
        ) + 30;
    const erSand = grafdataliste[0].data.some(
        (produksjonsdetalj) => produksjonsdetalj.materiale?.kategori === MaterialKategori.Sand
    );
    const maksVerdi = erSand ? Math.max(maksMåltDosering, 220) : Math.max(maksMåltDosering, 50);
    const possibleTicksYaxis = lagArrayMedInkrementer(40, 20);
    return possibleTicksYaxis.filter((tick) => tick <= maksVerdi);
};

export const hentStartOgSluttTid = (grafdataliste: Grafdata[]) => {
    const tidRange = grafdataliste
        .map((grafdata) => grafdata.data)
        .flat()
        .map((produksjonsdetalj) => produksjonsdetalj.datoLogget)
        .sort();
    const sluttTid = dayjs(
        tidRange[grafdataliste.map((grafdata) => grafdata.data).flat().length - 1]
    );
    const startTid = dayjs(tidRange[0]);
    return { startTid: startTid, sluttTid: sluttTid };
};

export const hentTicksXaxis = (startTid: dayjs.Dayjs, sluttTid: dayjs.Dayjs) => {
    const durationMinutter = sluttTid.diff(startTid, "minute");
    return hentTidsintervaller(durationMinutter, startTid);
};
export const hentTidsintervaller = (durationMinutter: number, startTid: dayjs.Dayjs) => {
    const antallIntervaller = 20;
    const intervalSizeMinutesRounded = Math.ceil(durationMinutter / antallIntervaller / 5) * 5;
    const startidMinuttRounded = Math.ceil(startTid.minute() / 5) * 5;
    const diffStarttidRounded = startidMinuttRounded - startTid.minute();
    const sluttTidMinutes = startTid.add(durationMinutter, "minute");

    const intervals = [];
    for (let i = 0; i < antallIntervaller; i++) {
        const nyttIntervall = startTid.add(
            diffStarttidRounded + intervalSizeMinutesRounded * i,
            "minute"
        );
        const erFørSluttTid = nyttIntervall.isBefore(sluttTidMinutes);
        erFørSluttTid && intervals.push(nyttIntervall.valueOf());
    }
    return intervals;
};

export const hentDoseringsBenevningAxis = (grafdataliste: Grafdata[]) => {
    const proddetaljer = grafdataliste.map((grafdata) => grafdata.data).flat();
    const tørrstoff = proddetaljer.some(
        (produksjonsdetalj) => produksjonsdetalj.erTørrstoffsprederPå
    )
        ? "g/m² "
        : "";
    const våtstoff = proddetaljer.some((produksjonsdetalj) => produksjonsdetalj.erVåtstoffsprederPå)
        ? "ml/m²"
        : "";
    return tørrstoff + våtstoff;
};

export const hentDoseringsBenevning = (proddetaljer: Produksjonsdetaljer) => {
    if (proddetaljer.erTørrstoffsprederPå) {
        return "g/m²";
    } else {
        return "ml/m²";
    }
};

export const hentYAxisLabel = (
    type: GrafType,
    dataKey: HastighetsType | DoseringsType,
    grafdata: Grafdata[]
) =>
    type === GrafType.hastighet
        ? "Hastighet (km/t)"
        : dataKey === DoseringsType.dosering
        ? `Dosering (${hentDoseringsBenevningAxis(grafdata)})`
        : "Sprederbredde (cm)";

export const hentReflinjeBenevning = (
    type: GrafType,
    dataKey: HastighetsType | DoseringsType,
    grafdata: Grafdata[]
) =>
    type === GrafType.hastighet
        ? " km/t"
        : dataKey === DoseringsType.dosering
        ? ` (${hentDoseringsBenevningAxis(grafdata)})`
        : " cm";

export const hentGraftittel = (type: GrafType, dataKey: HastighetsType | DoseringsType) =>
    type === GrafType.hastighet
        ? dataKey === HastighetsType.hastighetKmHAvg
            ? "Gjennomsnittshastighet"
            : "Maks hastighet"
        : dataKey === DoseringsType.dosering
        ? "Dosering"
        : "Sprederbredde";
