import { Lukknapp } from "../../components/fellesKomponenter/knapper/Lukknapp";
import { Tekst } from "../../components/fellesKomponenter/typografi/Tekst";
import styled from "styled-components";
import { motion } from "framer-motion";
import { KnappWrapper } from "./Metadata";
import { DataInput } from "../../kartSide/kartParameterVelger/DataInput";
import React, { useState } from "react";
import { erGyldigNumeriskInput } from "../../utils/inputValidering";
import { settDetaljerUrlParam } from "../../api/context/ØktDetaljerContext";
import { useRouter } from "next/router";
import { Produksjonsdetaljer } from "../../kartSide/types/produksjonsdetaljer";

interface VelgNyØktProps {
    setSeVelgØkt: (seVelgØkt: boolean) => void;
    seVelgØkt: boolean;
    produksjonsdetaljer: Produksjonsdetaljer[];
}

export const VelgNyØkt = ({ setSeVelgØkt, seVelgØkt, produksjonsdetaljer }: VelgNyØktProps) => {
    const [gyldigØktError, setGyldigØktError] = useState<boolean>(false);
    const { pathname, asPath } = useRouter();

    const erKunneIkkeFinneØkt = produksjonsdetaljer.length === 0 && asPath !== "/oktdetaljer/";
    const handleSubmitØkt = (økt: string) => {
        const gyldigInput = erGyldigNumeriskInput(økt);
        setGyldigØktError(!gyldigInput);
        if (gyldigInput) {
            settDetaljerUrlParam(pathname, { økt: økt.replaceAll(" ", "") });
        }
    };
    const lagErrorMeldinger = () => {
        const errorMeldinger = [];
        if (gyldigØktError) {
            errorMeldinger.push("Skriv inn et tall");
        }
        if (erKunneIkkeFinneØkt) {
            errorMeldinger.push("Kunne ikke finne økt, sjekk at du har skrevet inn rett øktid");
        }
        return errorMeldinger;
    };

    return (
        <MotionContainer
            initial="collapsed"
            animate={seVelgØkt ? "open" : "closed"}
            exit="exit"
            variants={{
                open: { opacity: 1, x: 0 },
                collapsed: { opacity: 0, x: "100%" },
                exit: { opacity: 0 },
            }}
            transition={{
                duration: 0.4,
            }}
            key={"velgØkt"}
        >
            {produksjonsdetaljer.length > 0 && (
                <KnappWrapper>
                    <Lukknapp onClick={() => setSeVelgØkt(false)} />
                </KnappWrapper>
            )}
            <Tekst noMargin={true}>Se øktdetaljer</Tekst>
            <Tekst størrelse={"M"}>
                Her kan du se detaljer om hastighet og dosering per økt. Søk opp økt ved å legge inn
                øktID her. Du kan også velge en økt i produksjonskartet og få opp detaljer ved å
                klikke på øktiden i popupen
            </Tekst>
            <DataInput
                tittel={"Velg økt"}
                handleSubmit={handleSubmitØkt}
                errormelding={lagErrorMeldinger()}
                error={gyldigØktError || erKunneIkkeFinneØkt}
                setError={setGyldigØktError}
            />
        </MotionContainer>
    );
};

const MotionContainer = styled(motion.nav)``;
