import { Produksjonsdetaljer } from "../../kartSide/types/produksjonsdetaljer";
import { hentTittel } from "../grafer/grafUtils";

export interface MetadataGjennomsnittsType {
    tittel: string;
    snitthastighetAvg: number;
    snitthastighetMaks: number;
    snittDosering?: number;
    snittSprederbredde?: number;
    antallMålinger: number;
    doseringsEnhet?: string;
}

export const hentGjennomsnittsverdierForProduksjonsdetaljer = (
    produksjonsdetaljer: Produksjonsdetaljer[]
) => {
    if (produksjonsdetaljer.length === 0) {
        return [];
    }
    const produksjonsdetaljerMedTittel = hentProduksjonsdetaljerMedTittel(produksjonsdetaljer);
    const grupperteProduksjonsdetaljer = grupperProduksjonsdetaljer(produksjonsdetaljerMedTittel);
    const metadataGjennomsnitt = hentMetadataGjennomsnitt(grupperteProduksjonsdetaljer);
    return metadataGjennomsnitt;
};

interface ProduksjonsdetaljMedTittel extends Produksjonsdetaljer {
    tittel: string;
}

const hentProduksjonsdetaljerMedTittel = (produksjonsdetaljer: Produksjonsdetaljer[]) =>
    produksjonsdetaljer.map((produksjonsdetalj) => ({
        ...produksjonsdetalj,
        tittel: hentTittel([produksjonsdetalj]),
    }));

const grupperProduksjonsdetaljer = (produksjonsdetaljerMedTittel: ProduksjonsdetaljMedTittel[]) =>
    produksjonsdetaljerMedTittel.reduce((gruppering: ProduksjonsdetaljMedTittel[][], current) => {
        const finnesIGruppering = gruppering.find((item) => item[0].tittel === current.tittel);
        if (!finnesIGruppering) {
            gruppering.push([current]);
        } else if (finnesIGruppering) {
            gruppering.find((item) => item[0].tittel === current.tittel)?.push(current);
        }
        return gruppering;
    }, []);

const hentMetadataGjennomsnitt = (
    grupperteProduksjonsdetaljer: ProduksjonsdetaljMedTittel[][]
): MetadataGjennomsnittsType[] =>
    grupperteProduksjonsdetaljer.map((proddetaljMedTittel) => {
        const harDosering = proddetaljMedTittel.some(
            (detalj) => detalj.erTørrstoffsprederPå || detalj.erVåtstoffsprederPå
        );
        const doseringsEnhet = harDosering
            ? proddetaljMedTittel.some((detalj) => detalj.erTørrstoffsprederPå)
                ? "g/m²"
                : "ml/m²"
            : undefined;
        const snitthastighetAvg = hentGjennomsnittsverdi(
            proddetaljMedTittel.map((detalj) => detalj.hastighetKmHAvg)
        );
        const snitthastighetMaks = hentGjennomsnittsverdi(
            proddetaljMedTittel.map((detalj) => detalj.hastighetKmHMax)
        );

        const dosering: number[] = proddetaljMedTittel.reduce((acc: number[], detalj) => {
            if (detalj.dosering !== undefined) {
                acc.push(detalj.dosering);
            }
            return acc;
        }, []);

        const sprederbredde: number[] = proddetaljMedTittel.reduce((acc: number[], detalj) => {
            if (detalj.sprederbredde !== undefined) {
                acc.push(detalj.sprederbredde);
            }
            return acc;
        }, []);

        const snittDosering = harDosering ? hentGjennomsnittsverdi(dosering) : undefined;
        const snittSprederbredde = harDosering ? hentGjennomsnittsverdi(sprederbredde) : undefined;

        return {
            tittel: proddetaljMedTittel[0].tittel,
            snitthastighetAvg: snitthastighetAvg,
            snitthastighetMaks: snitthastighetMaks,
            snittDosering: snittDosering,
            snittSprederbredde: snittSprederbredde,
            antallMålinger: proddetaljMedTittel.length,
            doseringsEnhet: doseringsEnhet,
        };
    });

export const hentGjennomsnittsverdi = (nummerliste: number[]) => {
    const snitt = nummerliste.reduce((a, b) => a + b, 0) / nummerliste.length;
    return Math.round(snitt);
};
