import React, { MutableRefObject, useEffect, useMemo, useState } from "react";
import { Produksjonslenke } from "../kartSide/types/produksjonslenke";
import { MapContainer, TileLayer } from "react-leaflet";
import { getGeoJsonØktVeglenke } from "../kartSide/Kartvisning/geoJson/geoJSONutilsVeglenker";
import { ChangeView } from "../kartSide/Kartvisning/ChangeView";
import { crsUtm33N } from "../kartSide/utils/crs";
import styles from "../kartSide/Kartvisning/Kartvisning.module.css";
import { finnKartBoundsForProduksjonslenker } from "../kartSide/Kartvisning/kartBoundsUtils";
import { ZoomKnapp } from "../kartSide/Kartvisning/ZoomKnapp";
import "leaflet/dist/leaflet.css";
import { useWindowSize } from "../hooks/useWindowSize";
import { useUntabableClusters } from "../hooks/useUntabableClusters";
import { defaultBounds } from "../kartSide/utils/kartUtils";

export interface ØktKarvisningProps {
    produksjonslenker: Produksjonslenke[];
    containerRef: MutableRefObject<any>;
    aktivHoverRecord?: number[];
}

export const ØktKartvisning = ({
    produksjonslenker,
    containerRef,
    aktivHoverRecord,
}: ØktKarvisningProps) => {
    const [elementStørrelse, setElementStørrelse] = useState({ width: 400, height: 400 });
    const { windowWidth } = useWindowSize();
    const [skalOppdatereKartbounds, setSkalOppdatereKartbounds] = useState(true);
    useUntabableClusters();

    const kartBounds = useMemo(() => {
        if (produksjonslenker.length > 0) {
            return finnKartBoundsForProduksjonslenker(produksjonslenker);
        }
        return defaultBounds;
    }, [produksjonslenker]);

    const øktGeoJson = useMemo(
        () => getGeoJsonØktVeglenke(produksjonslenker, aktivHoverRecord),
        [produksjonslenker, aktivHoverRecord]
    );

    useEffect(() => {
        const _elementStørrelse = finnElementStørrelse(containerRef);
        _elementStørrelse && setElementStørrelse(_elementStørrelse);
    }, [windowWidth]);

    useEffect(() => {
        setSkalOppdatereKartbounds(false);
    }, []);

    return (
        <MapContainer
            className={styles.kartvisningOkt}
            center={[64.6, 9.5022]}
            scrollWheelZoom
            crs={crsUtm33N}
            zoom={16}
            minZoom={4}
            maxZoom={16}
            zoomControl={false}
            zoomDelta={0.8}
            zoomSnap={0}
            keyboard={false}
        >
            <TileLayer
                attribution="© NVDB, Geovekst, kommunene og Open Street Map contributors (utenfor Norge)"
                url="https://services.geodataonline.no/arcgis/rest/services/Trafikkportalen/GeocacheTrafikkJPG/MapServer/tile/{z}/{y}/{x}"
                subdomains="123456789"
            />
            <ChangeView
                kartBounds={kartBounds}
                skalOppdatere={skalOppdatereKartbounds}
                medPadding={true}
            />
            {øktGeoJson}
            <ZoomKnapp
                size="small"
                relativeProps={{
                    relative: true,
                    parentHeight: elementStørrelse.height,
                    parentWidth: elementStørrelse.width,
                }}
            />
        </MapContainer>
    );
};

const finnElementStørrelse = (ref: any) => {
    if (ref.current) {
        const { current } = ref;
        const boundingRect = current.getBoundingClientRect();
        const { width, height } = boundingRect;
        return { width: Math.round(width), height: Math.round(height) };
    }
};
