import React, { useContext, useEffect, useState } from "react";
import { ØktdetaljerVisning } from "./ØktdetaljerVisning";
import { SideHeader } from "../components/sideheader/SideHeader";
import Head from "next/head";
import styled from "styled-components";
import { hentDetaljerUrlParam, ØktDetaljerContext } from "../api/context/ØktDetaljerContext";
import { ØktfilterContext } from "../api/context/ØktContext";
import dayjs from "dayjs";
import { ApiRespons } from "../api/api";
import { Produksjonslenke } from "../kartSide/types/produksjonslenke";
import { Spinner } from "../components/fellesKomponenter/spinner/Spinner";
import { KartvisningFeil } from "../kartSide/feil/KartvisningFeil";
import { hentFeil } from "../kartSide/utils/kartSideUtils";
import { Produksjonsdetaljer } from "../kartSide/types/produksjonsdetaljer";
import ErrorBoundary from "../components/feil/ErrorBoundary";

export const ØktdetaljerSide = () => {
    const { hentØktDetaljer } = useContext(ØktDetaljerContext);
    const { hentØkt } = useContext(ØktfilterContext);

    const [produksjonslenkerRespons, setProduksjonslenkerRespons] =
        useState<ApiRespons<Produksjonslenke[]>>();
    const [produksjonsdetaljerRespons, setProduksjonsdetaljerRespons] =
        useState<ApiRespons<Produksjonsdetaljer[]>>();
    const [produksjonslenker, setProduksjonslenker] = useState<Produksjonslenke[]>();
    const [produksjonsdetaljer, setProduksjonsdetaljer] = useState<Produksjonsdetaljer[]>();

    const øktId = hentDetaljerUrlParam()?.økt;
    useEffect(() => {
        const hentProduksjonslenker = async (øktId: string) => {
            const økt = await hentØkt(øktId, dayjs());
            setProduksjonslenkerRespons(økt);
            if (økt?.suksess) {
                setProduksjonslenker(økt.data);
            }
        };

        const hentDetaljer = async (øktId: string) => {
            const øktDetaljer = await hentØktDetaljer(øktId);
            setProduksjonsdetaljerRespons(øktDetaljer);
            if (øktDetaljer?.suksess) {
                setProduksjonsdetaljer(øktDetaljer.data);
            }
        };

        if (øktId) {
            hentProduksjonslenker(øktId);
            hentDetaljer(øktId);
        } else {
            setProduksjonslenker([]);
            setProduksjonsdetaljer([]);
            setProduksjonslenkerRespons({ suksess: true, data: [] });
            setProduksjonsdetaljerRespons({ suksess: true, data: [] });
        }
    }, []);

    return (
        <ErrorBoundary info={`Øktid:  ${øktId ?? "Ingen øktid"}, side: Øktdetaljer`}>
            <Head>
                <title>Øktdetaljer - Driftsportal</title>
            </Head>
            <Container>
                <SideHeader side={"øktdetaljer"} />
                {!produksjonslenker ||
                !produksjonsdetaljer ||
                !produksjonslenkerRespons ||
                !produksjonsdetaljerRespons ? (
                    <Spinner />
                ) : !produksjonslenkerRespons.suksess || !produksjonsdetaljerRespons.suksess ? (
                    <KartvisningFeil
                        feilmeldinger={hentFeil([
                            produksjonslenkerRespons,
                            produksjonsdetaljerRespons,
                        ])}
                    />
                ) : (
                    <ØktdetaljerVisning
                        produksjonslenker={produksjonslenker}
                        produksjonsdetaljer={produksjonsdetaljer}
                    />
                )}
            </Container>
        </ErrorBoundary>
    );
};

const Container = styled.div`
    margin: 20px 20px 100px 20px;
    background-color: var(--white);
`;
