import React, { useEffect, useRef, useState } from "react";
import styled from "styled-components";
import { Grafer } from "./grafer/Grafer";
import { Metadata } from "./metadata/Metadata";
import { ØktKarvisningProps } from "./ØktKartvisning";
import { Produksjonslenke } from "../kartSide/types/produksjonslenke";
import dynamic from "next/dynamic";
import BrøytebilSpinner from "../kartSide/brøytebilSpinner/BrøytebilSpinner";
import { RowWrapper } from "../components/fellesKomponenter/wrappers/wrappers";
import { Produksjonsdetaljer } from "../kartSide/types/produksjonsdetaljer";
import { VelgNyØkt } from "./metadata/VelgNyØkt";
import { breakpointNettbrett } from "../utils/constants";

interface ØktdetaljerVisningProps {
    produksjonslenker: Produksjonslenke[];
    produksjonsdetaljer: Produksjonsdetaljer[];
}

export const ØktdetaljerVisning = ({
    produksjonslenker,
    produksjonsdetaljer,
}: ØktdetaljerVisningProps) => {
    const [aktivHoverRecord, setAktivHoverRecord] = useState<number[]>();
    const kartvisningContainerRef = useRef<any>(null);
    const [seVelgØkt, setSeVelgØkt] = useState<boolean>(produksjonsdetaljer.length === 0);

    return (
        <>
            <RowWrapper>
                <KartvisningContainer ref={kartvisningContainerRef}>
                    <DynamicKartvisning
                        produksjonslenker={produksjonslenker}
                        containerRef={kartvisningContainerRef}
                        aktivHoverRecord={aktivHoverRecord}
                    />
                </KartvisningContainer>
                <Metadata
                    produksjonsdetaljer={produksjonsdetaljer}
                    seVelgØkt={seVelgØkt}
                    setSeVelgØkt={setSeVelgØkt}
                />
            </RowWrapper>
            {produksjonsdetaljer.length > 0 && (
                <Grafer
                    produksjonsdetaljer={produksjonsdetaljer}
                    setAktivHoverRecord={setAktivHoverRecord}
                />
            )}
        </>
    );
};

const DynamicKartvisning = dynamic<ØktKarvisningProps>(
    () => import("./ØktKartvisning").then((mod) => mod.ØktKartvisning),
    {
        loading: () => <BrøytebilSpinner />,
        ssr: false,
    }
);

const KartvisningContainer = styled.div`
    height: 400px;
    width: 70%;
    @media (max-width: ${breakpointNettbrett}) {
        width: 100%;
    }
`;
